## @file
# GTI Filtering based on S/N

#========================================================================
sub PG_gti_filter(){
    print "\n----> Running PG GTI Filtering for $_[0] .....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Running PG GTI Filtering for $_[0] ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for PG GTI Filtering ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for PG GTI Filtering ..... ";}

    use vars qw($intrument
		$event_list
		$bkg_event_list
		$eregionanal_file
		$image_file
		$lc_src_outfile
		$lc_bkg_outfile 
		$lc_src_counts_file
		$lc_bkg_counts_file
		$pnidl_scriptfile_name
		$m1idl_scriptfile_name
		$m2idl_scriptfile_name
		 );

    my $source_obsid = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $src_ra       = $gv->SRC_RA;
    my $src_dec      = $gv->SRC_DEC;

    $instrument         = $_[0];
    $bkg_event_list     = $gti_directory."PG_bkg_event_list.fits";
    $src_regionset      = $gti_directory."PG_src_region.ds";
    $bkg_regionset      = $gti_directory."PG_bkg_region.ds";   
    $src_list           = $images_directory.$instrument."_emllist.fits";
    $image_file         = $gti_directory."PG_".$instrument."_img_raw.img";
    $eregionanal_file   = $spectra_directory.$source_name."_".$instrument."_eregionanalyse.txt";
    $lc_src_outfile     = $gti_directory.$source_name."_PG_".$instrument."lightcurve_src_newgti.fit";
    $lc_bkg_outfile     = $gti_directory.$source_name."_PG_".$instrument."lightcurve_bkg_newgti.fit";
    $lc_src_counts_file = $gti_directory.$source_name."_PG_".$instrument."lc_counts_src.txt";
    $lc_bkg_counts_file = $gti_directory.$source_name."_PG_".$instrument."lc_counts_bkg.txt";
    $pnidl_scriptfile_name = $gti_directory.$source_name."_PG_".$instrument."_gti.pro";
    $m1idl_scriptfile_name = $gti_directory.$source_name."_PG_".$instrument."_gti.pro";
    $m2idl_scriptfile_name = $gti_directory.$source_name."_PG_".$instrument."_gti.pro";

    if ($instrument eq "pn"){$event_list = $pn_event_list;}
    if ($instrument eq "mos1"){$event_list = $m1_event_list;}
    if ($instrument eq "mos2"){$event_list = $m2_event_list;}

    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

#.. Get Starting and Ending Time of the Observation

    ($mode,$filter,$dmode,$utc_obsdate,$obs_duration,$ontime,$livetime,$tstart,$tend) = &event_file_inf($event_list,$instrument);

#.. Generating the ligth curve off the source and the background ...'

#.. Produce Instrument Image (needed by eregioanalyse)

    &produce_image($event_list,$image_file,$instrument);
    
#.. Using the instrument image to work out the source centroid
    
    my($bore_img_x,$bore_img_y
       ,$bore_optrad_asec,$bore_optradius
       ,$bkg_inner_rad_asec,$bkg_inner_rad
       ,$bkg_outer_rad_asec,$bkg_outer_rad) = (0,0,0,0,0,0,0,0);	
    ($bore_img_x,$bore_img_y
     ,$bore_optrad_asec,$bore_optradius
     ,$bkg_inner_rad_asec,$bkg_inner_rad
     ,$bkg_outer_rad_asec,$bkg_outer_rad)
	=&run_eregionanalyse($image_file,$src_ra,$src_dec,$instrument,$eregionanal_file,"PG");

#.. Something has gone wrong so wont produce anything

    if (($bore_img_x == 0 && $bore_img_y == 0) || $bore_optrad_asec == 300){
	my $rate = 1000.;                                       # set to silly default value
	if ($instrument eq "pn"){$gv->GTI_PN_Rate($rate);}      # Set the PN rate 
	if ($instrument eq "mos1"){$gv->GTI_MOS1_Rate($rate);}  # Set the MOS1 rate       
	if ($instrument eq "mos2"){$gv->GTI_MOS2_Rate($rate);}  # Set the MOS2 rate  
	&error_code(10); 
	return(0);
    }

#.. Set the values of the PG_ parameters: source/background center and radii set to the values found with eregionanalyse

    if ($gv->PG_timebinning == 0){$gv->PG_timebinning(100);} # secs (default value)
    if ($gv->PG_timestart == 0){$gv->PG_timestart($tstart);} # secs
    if ($gv->PG_timeend == 0){$gv->PG_timeend($tend);}       # secs
    if ($gv->PG_minpi == 0){$gv->PG_minpi(200);}             # eV (default value)
    if ($gv->PG_maxpi == 0){$gv->PG_maxpi(10000);}           # eV (default value)
    if ($gv->PG_aringcx == 0){$gv->PG_aringcx($bore_img_x);}    
    if ($gv->PG_aringcy == 0){$gv->PG_aringcy($bore_img_y);} 
    if ($gv->PG_innerrad == 0){$gv->PG_innerrad($bkg_inner_rad);}
    if ($gv->PG_outerrad == 0){$gv->PG_outerrad($bkg_outer_rad);}
    if ($gv->PG_sx == 0){$gv->PG_sx($bore_img_x);}
    if ($gv->PG_sy == 0){$gv->PG_sy($bore_img_y);}
    if ($gv->PG_source_rad == 0){$gv->PG_source_rad($bore_optradius);}

    $src_area = $pi*$gv->PG_source_rad*$gv->PG_source_rad;
    $bkg_area = $pi*$gv->PG_outerrad*$gv->PG_outerrad
	- $pi*$gv->PG_innerrad*$gv->PG_innerrad;
    if ($bkg_area != 0) {$gv->PG_areafactor($src_area/$bkg_area);} else {$gv->PG_areafactor(0.);}

    print "   \#> Source and Background Extraction Regions to be used: \n";
    print "       $instrument \n";
    print "         $instrument Region SRC : circle \n";
    print "         $instrument SRC XC     : ",$gv->PG_sx,"\n";
    print "         $instrument SRC YC     : ",$gv->PG_sy,"\n";
    print "         $instrument SRC RC     : ",$gv->PG_source_rad,"\n";
    print "         $instrument Region BKG : annulus \n";
    print "         $instrument BKG XC     : ",$gv->PG_aringcx,"\n";
    print "         $instrument BKG YC     : ",$gv->PG_aringcy,"\n";
    print "         $instrument BKG RC     : ",$gv->PG_innerrad," - ",$gv->PG_outerrad,"\n";
    print "         $instrument BKG AREA SCALE FACTOR : ",$gv->PG_areafactor," (Only area taken into account)\n";
    if ($gv->PG_outerrad == 6000){print "         WARNING : $instrument regions : Maximum allowed radius reached";}

#.. Produce background event list with sources removed to work out background rates: run edetect_chain, region and eveselect

    &get_bkg_filterset($event_list, $instrument);

#.. Produce Source and Background light Curves

    &PG_produce_lightcurve_file("circle",$instrument,$event_list,$lc_src_outfile);      # Source LC
    &PG_produce_lightcurve_file("annulus",$instrument,$bkg_event_list,$lc_bkg_outfile); # BKG LC

#.. Extraction of the Counts columns....

    &PG_extract_counts($lc_src_outfile,$lc_src_counts_file); # Source
    &PG_extract_counts($lc_bkg_outfile,$lc_bkg_counts_file); # BKG

#.. Run IDL procedure to optimize S/N

    my $rate    = 1000.; # set to silly default value
    my $src_rad = 1000.; # set to silly default value
    my $max_sn  = 1000.; # set to silly default value 
    ($src_rad,$max_sn,$rate)=&PG_Optimize_SN($source_name,$instrument,$mode,$lc_src_counts_file,$lc_bkg_counts_file);
     
#.. Set the values of the correspondent GTI_ parameters 

    if ($instrument eq "pn"){$gv->GTI_PN_Rate($rate);}      # Set the PN rate 
    if ($instrument eq "mos1"){$gv->GTI_MOS1_Rate($rate);}  # Set the MOS1 rate       
    if ($instrument eq "mos2"){$gv->GTI_MOS2_Rate($rate);}  # Set the MOS2 rate      

    &error_code(0);
    return(0);
}

#========================================================================
sub get_bkg_filterset(){

    my $event_list = $_[0];
    my $instr      = $_[1];

    chdir($gti_directory); 
    &run_edetectchain;
    chdir($gti_directory); 

    if ($instr eq "pn") {$expr="\#XMMEA_EP && (FLAG==0) && (PATTERN<=4) && region($bkg_regionset:REGION)";}
    if ($instr eq "mos1" or $instr eq "mos2") {$expr="\#XMMEA_EM && (FLAG==0) && (PATTERN<=12) && region($bkg_regionset:REGION)";}

   @args = ("region eventset=$event_list tempset=temp.ds srclisttab=$src_list operationstyle=global shrinkconfused=yes radiusstyle=contour bkgratestyle=col regionset=$src_regionset bkgregionset=$bkg_regionset");
    system(@args)== 0 or &error_code(10);

    @args = ("evselect table=$event_list filteredset=$bkg_event_list expression=\"$expr\"");
    system(@args)== 0 or &error_code(10);
    return(1);
}

#========================================================================
sub PG_produce_lightcurve_file(){
    
# Produce an event file for a given region and energy range
    
    my $E1         = $gv->PG_minpi;
    my $E2         = $gv->PG_maxpi;
    my $XC         = $gv->PG_sx;
    my $YC         = $gv->PG_sy;
    my $RSRC       = $gv->PG_source_rad;
    my $RBKG1      = $gv->PG_innerrad;
    my $RBKG2      = $gv->PG_outerrad;
    my $T          = $gv->PG_timebinning;
    my $tstart     = $gv->PG_timestart;
    my $tstop      = $gv->PG_timeend;
    my $region     = $_[0];
    my $instr      = $_[1];
    my $event_file = $_[2];
    my $outfile    = $_[3];

    if ($instr eq "pn") {$expr="\#XMMEA_EP && (FLAG==0) && (PATTERN<=4) && (PI>=$E1) && (PI<=$E2)";}
    if ($instr eq "mos1" or $instr eq "mos2") {$expr="\#XMMEA_EM && (FLAG==0) && (PATTERN<=12) && (PI>=$E1) && (PI<=$E2)";}

    if ($region eq "circle") {$areaexpr="((X,Y) in circle($XC,$YC,$RSRC))";}
    if ($region eq "annulus") {$areaexpr="((X,Y) in annulus($XC,$YC,$RBKG1,$RBKG2))";}
 
    print "       $instr : $region ($areaexpr): Energy Range: $E1 - $E2 keV \n";
    print "       $instr : $region ($areaexpr): Time Bining : $T sec\n";
    print "       (Event File: $event_file ; Output File: $outfile) \n";
    if ($Log_Flag == 0) {print MYOUTFILE "       $instr : $region ($areaexpr): Energy Range: $E1 - $E2 keV \n";}
    if ($Log_Flag == 0) {print MYOUTFILE "       $instr : $region ($areaexpr): Time Bining : $T sec\n";}
    if ($Log_Flag == 0) {print MYOUTFILE "       (Event File: $event_file ; Output File: $outfile) \n";}
 
    @args = ("evselect table=$event_file:EVENTS withrateset=yes rateset=$outfile maketimecolumn=yes timecolumn=TIME timebinsize=$T expression=\"$expr \&\& $areaexpr\" makeratecolumn=no withtimeranges=yes timemin=$tstart timemax=$tstop");

    system(@args)== 0 or &error_code(10);
    
    return(1);
}

#========================================================================
sub PG_extract_counts(){
    
    my $event_file  = $_[0];
    my $out_file    = $_[1];

    @args = ("rm -f $out_file");
    system(@args)== 0 or &error_code(10);

    @args = ("fdump $event_file+1 $out_file \"COUNTS TIME\" - prhead=no showcol=no prdata=yes showunit=no showrow=no");
    system(@args)== 0 or &error_code(10);

    return(1);
}


#========================================================================
sub PG_Optimize_SN(){
    
    my $source_name     = $_[0];
    my $instrument      = $_[1];
    my $frame           = $_[2];
    my $src_counts_file = $_[3];
    my $bkg_counts_file = $_[4];

    my $lc_points;
    chomp($lc_points    = `grep -v \^\$ $bkg_counts_file | wc | awk '{print \$1}'`);

    my $opt = 0;
    if ($frame eq "PrimeFullWindow") {
	$opt = 1;
    } elsif ($frame eq "PrimeFullWindowExtended") {
	$opt = 1;
    } elsif ($frame eq "PrimeLargeWindow") {
	$opt = 2;
    } elsif ($frame eq "PrimeSmallWindow") {
	$opt = 3;
    }
 
    $T             = $gv->PG_timebinning;
    $RSRC          = $gv->PG_source_rad;
    $annulus[0]    = $gv->PG_innerrad;
    $annulus[1]    = $gv->PG_outerrad;
    $rectangle[0]  = 0.;
    $rectangle[1]  = 0.;
    
    my $idl_init_file = $Script_directory."xmm_idl_init.pro";
    my $idl_logfile_name  = $gti_directory."PG_gti_new.log";

    my $templatefile_name  = $Script_directory."IDL\_gti\_new\.template";      # Master template file
    my $scriptfile_name    = $Script_directory."default\.pro";
    if ($instrument eq "pn")   {$scriptfile_name = $pnidl_scriptfile_name;}           
    if ($instrument eq "mos1") {$scriptfile_name = $m1idl_scriptfile_name;}           
    if ($instrument eq "mos2") {$scriptfile_name = $m2idl_scriptfile_name;} 

    open MASTER_TEMPLATE,  "<$templatefile_name";
    open OUTPUT,           ">$scriptfile_name ";
    
    while (defined(my $template_line=<MASTER_TEMPLATE>)) {
	chomp($template_line);
	$template_line =~ s/%P1\b/$source_name/g;
	$template_line =~ s/%P2\b/PG/g;
    	$template_line =~ s/%P3\b/$lc_points/g;
    	$template_line =~ s/%P4\b/$src_counts_file/g;
    	$template_line =~ s/%P5\b/$bkg_counts_file/g;
    	$template_line =~ s/%P6\b/$T/g;
    	$template_line =~ s/%P7\b/$opt/g;
    	$template_line =~ s/%P8\b/$RSRC/g;
    	$template_line =~ s/%P9\b/0/g;
    	$template_line =~ s/%P10\b/$rectangle[0]/g;
    	$template_line =~ s/%P11\b/$rectangle[1]/g;
    	$template_line =~ s/%P12\b/$annulus[0]/g;
    	$template_line =~ s/%P13\b/$annulus[1]/g;    	
	$template_line =~ s/%P14\b/$instrument/g;
	$template_line =~ s/%P15\b/$idl_logfile_name/g;
	print OUTPUT "$template_line \n";
    }
    close(OUTPUT);
    close(MASTER_TEMPLATE);

    @args = ("idl $idl_init_file $scriptfile_name");
    system(@args) == 0 or &error_code(10);

#. Read IDL log file
    $src_rad       = $RSRC; # default
    $max_sn        = 1000.; # default
    $bkg_count_rate= 1000.; # default

    open INFO,  "<$idl_logfile_name";
    while (defined(my $template_line=<INFO>)) {
	chomp($template_line);
	@split = split/:/,$template_line;
	if ($template_line =~ m/^SourceRadius/i){$src_rad=$split[1]; $src_rad =~ s/\s+//;}
	if ($template_line =~ m/^Max_SN/i){$max_sn=$split[1]; $max_sn =~ s/\s+//;}
	if ($template_line =~ m/^CountRate/i){$bkg_count_rate=$split[1]; $bkg_count_rate=~ s/\s+//;}
    }
    close(INFO);

    return($src_rad,$max_sn,$bkg_count_rate);
}
1;
