## @file
# Compute GTI the standard way

## @method void compute_gti()
# Compute GTI the standard way
# @return Void

#========================================================================
sub compute_gti(){
    print "\n----> Working out GTI .....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Working out GTI ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for GTI ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for GTI ..... ";}
    
    use vars qw($m1_GTI_lightcurvefile 
		$m1_GTI_file           
		$m2_GTI_lightcurvefile 
		$m2_GTI_file           
		$pn_GTI_lightcurvefile 
		$pn_GTI_file 
		$m1_GTI_timinglightcurvefile 
		$m1_timingGTI_file           
		$m2_GTI_timinglightcurvefile 
		$m2_timingGTI_file           
		$pn_GTI_timinglightcurvefile 
		$pn_timingGTI_file 
		 );

#.. Parameters for GTI 
    
    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $source_ra    = $gv->SRC_RA;
    my $source_dec   = $gv->SRC_DEC;
    
    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {&error_code(3);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {&error_code(3);}
    &set_odfingest($odfingestfile);
    
#.. Print sas_setup    
    &print_sas_setup();

#.. Get event files
    @test_pnevt=&test_pneventfiles();
    $pn_event_list=$test_pnevt[1];
    @test_pntimingevt=&test_pntimingeventfiles();
    $pn_timingevent_list=$test_pntimingevt[1];

    @test_m1evt=&test_mos1eventfiles();
    $m1_event_list=$test_m1evt[1];
    @test_m1timingevt=&test_mos1timingeventfiles();
    $m1_timingevent_list=$test_m1timingevt[1];

    @test_m2evt=&test_mos2eventfiles();
    $m2_event_list=$test_m2evt[1];
    @test_m2timingevt=&test_mos2timingeventfiles();
    $m2_timingevent_list=$test_m2timingevt[1];

    if ((!$test_pnevt[0] && !$test_m1evt[0] && !$test_m2evt[0]) &&
	(!$test_pntimingevt[0] && !$test_m1timingevt[0] && !$test_m2timingevt[0])) {&error_code(4);} 

#.. Get Information from event files 
    if ($Log_Evt_Flag == 0) {
	my $event_log=$results_directory."event_".$source_name."\.log";
	if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}    
	open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

    if ($test_pnevt[0]) {&event_file_inf($pn_event_list,"pn");}
    if ($test_m1evt[0]) {&event_file_inf($m1_event_list,"mos1");}
    if ($test_m2evt[0]) {&event_file_inf($m2_event_list,"mos2");}
    if ($test_pntimingevt[0]) {&event_file_inf($pn_timingevent_list,"pn");}
    if ($test_m1timingevt[0]) {&event_file_inf($m1_timingevent_list,"mos1");}
    if ($test_m2timingevt[0]) {&event_file_inf($m2_timingevent_list,"mos2");}

    if ($Log_Evt_Flag == 0) {
	close(EVTLOG);
	$Log_Evt_Flag = 1;	
    }

    my $optimize_SN_flag = $gv->GTI_optimize_SN_flag;
#.. This routine will work out a new PN_Rate, MOS1_Rate and MOS2_Rate
    if ($optimize_SN_flag eq "T" && $test_pnevt[0]){&PG_gti_filter("pn");}  
#    if ($optimize_SN_flag eq "T" && $test_m1evt[0]){&PG_gti_filter("mos1");}  
#    if ($optimize_SN_flag eq "T" && $test_m2evt[0]){&PG_gti_filter("mos2");}  

    chdir($gti_directory);
 
    my $mos1energy_cut   = $gv->GTI_mos1energy_cut; 
    my $mos2energy_cut   = $gv->GTI_mos2energy_cut; 
    my $pnenergy_cut     = $gv->GTI_pnenergy_cut;	 
    my $MOS1_Rate        = $gv->GTI_MOS1_Rate;	 
    my $MOS2_Rate        = $gv->GTI_MOS2_Rate;	 
    my $PN_Rate          = $gv->GTI_PN_Rate;	 
    my $timebin          = $gv->GTI_timebin;        

    print "\n   \#> Parameters for GTI\n";
    if ($optimize_SN_flag eq "F"){
	print "           Background Light Curve Parameters: \n";
	print "             PN Energy Cut  : $pnenergy_cut eV\n";
	print "             MOS1 Energy Cut: $mos1energy_cut eV\n";
	print "             MOS2 Energy Cut: $mos2energy_cut eV\n";
    }	
    print "           Definition of GTI: \n";
    print "             PN Rate   <    : $PN_Rate cts/sec\n";
    print "             MOS1 Rate <    : $MOS1_Rate cts/sec\n";
    print "             MOS2 Rate <    : $MOS2_Rate cts/sec\n";
    print "             Time Bin       : $timebin sec\n\n";
    
    if ($Log_Flag == 0) {
	print MYOUTFILE "\n   \#> Parameters for GTI\n";
	if ($optimize_SN_flag eq "F"){
	    print MYOUTFILE "           Background Light Curve Parameters: \n";
	    print MYOUTFILE "             PN Energy Cut  : $pnenergy_cut eV\n";
	    print MYOUTFILE "             MOS1 Energy Cut: $mos1energy_cut eV\n";
	    print MYOUTFILE "             MOS2 Energy Cut: $mos2energy_cut eV\n";
	}	
	print MYOUTFILE "           Definition of GTI: \n";
	print MYOUTFILE "             PN Rate   <    : $PN_Rate cts/sec\n";
	print MYOUTFILE "             MOS1 Rate <    : $MOS1_Rate cts/sec\n";
	print MYOUTFILE "             MOS2 Rate <    : $MOS2_Rate cts/sec\n";
	print MYOUTFILE "             Time Bin       : $timebin sec\n\n";
    }
    
#.. File definition

    $pn_GTI_lightcurvefile = $gti_directory."pn_back_lightc.fits";
    $pn_GTI_file           = &set_gtifilename("pn","imaging");
    $m1_GTI_lightcurvefile = $gti_directory."m1_back_lightc.fits";
    $m1_GTI_file           = &set_gtifilename("mos1","imaging");
    $m2_GTI_lightcurvefile = $gti_directory."m2_back_lightc.fits";
    $m2_GTI_file           = &set_gtifilename("mos2","imaging");
    $pn_GTI_timinglightcurvefile = $gti_directory."pn_back_lightc_timing.fits";
    $pn_timingGTI_file           = &set_gtifilename("pn","timing");
    $m1_GTI_timinglightcurvefile = $gti_directory."m1_back_lightc_timing.fits";
    $m1_timingGTI_file           = &set_gtifilename("mos1","timing");
    $m2_GTI_timinglightcurvefile = $gti_directory."m2_back_lightc_timing.fits";
    $m2_timingGTI_file           = &set_gtifilename("mos2","timing");

#.. Create PN GTI File
    if ($Log_GTIEvt_Flag == 0) {
	my $gtievent_log=$results_directory."gtievent_".$source_name."\.log";
	if (-e $gtievent_log) {unlink("$gtievent_log") or die "Cannot rm file: $!";}    
	open(GTILOG, "> $gtievent_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

    if ($test_pnevt[0]){

	@args = ("evselect","table=$pn_event_list:EVENTS","expression=\#XMMEA_EP&&(PI>$pnenergy_cut)&&(PATTERN==0)"
		 ,"rateset=$pn_GTI_lightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
		 ,"makeratecolumn=yes");
	system(@args)== 0 or &error_code(4);

	@args = ("tabgtigen","table=$pn_GTI_lightcurvefile","expression=RATE<$PN_Rate","gtiset=$pn_GTI_file");
	system(@args)== 0 or &error_code(4);
	&gti_file_inf($pn_GTI_file,"pn");	
    }
    
#.. Create M1 GTI File
    if ($test_m1evt[0]){

	@args = ("evselect","table=$m1_event_list:EVENTS","expression=\#XMMEA_EM&&(PI>$mos1energy_cut)&&(PATTERN==0)"
		    ,"rateset=$m1_GTI_lightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
		    ,"makeratecolumn=yes");
	system(@args)== 0 or &error_code(4);

	@args = ("tabgtigen","table=$m1_GTI_lightcurvefile","expression=RATE<$MOS1_Rate","gtiset=$m1_GTI_file");
	system(@args)== 0 or &error_code(4);
	&gti_file_inf($m1_GTI_file,"mos1");
    }
#.. Create M2 GTI File
    if ($test_m2evt[0]){

	@args = ("evselect","table=$m2_event_list:EVENTS","expression=\#XMMEA_EM&&(PI>$mos2energy_cut)&&(PATTERN==0)"
		 ,"rateset=$m2_GTI_lightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
		 ,"makeratecolumn=yes");
	system(@args)== 0 or &error_code(4);

	@args = ("tabgtigen","table=$m2_GTI_lightcurvefile","expression=RATE<$MOS2_Rate","gtiset=$m2_GTI_file");
	system(@args)== 0 or &error_code(4);
	&gti_file_inf($m2_GTI_file,"mos2");
    }

#.. Create PN Timing GTI File
    if ($test_pntimingevt[0]){

	@args = ("evselect","table=$pn_timingevent_list:EVENTS","expression=\#XMMEA_EP&&(PI>$pnenergy_cut)&&(PATTERN==0)"
		 ,"rateset=$pn_GTI_timinglightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
		 ,"makeratecolumn=yes");
	system(@args)== 0 or &error_code(4);

	@args = ("tabgtigen","table=$pn_GTI_timinglightcurvefile","expression=RATE<$PN_Rate","gtiset=$pn_timingGTI_file");
	system(@args)== 0 or &error_code(4);
	&gti_file_inf($pn_timingGTI_file,"pn");	
    }
    
#.. Create M1 Timing GTI File
    if ($test_m1timingevt[0]){

	@args = ("evselect","table=$m1_timingevent_list:EVENTS","expression=\#XMMEA_EM&&(PI>$mos1energy_cut)&&(PATTERN==0)"
		    ,"rateset=$m1_GTI_timinglightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
		    ,"makeratecolumn=yes");
	system(@args)== 0 or &error_code(4);

	@args = ("tabgtigen","table=$m1_GTI_timinglightcurvefile","expression=RATE<$MOS1_Rate","gtiset=$m1_timingGTI_file");
	system(@args)== 0 or &error_code(4);
	&gti_file_inf($m1_timingGTI_file,"mos1");
    }
#.. Create M2 Timing GTI File
    if ($test_m2timingevt[0]){

	@args = ("evselect","table=$m2_timingevent_list:EVENTS","expression=\#XMMEA_EM&&(PI>$mos2energy_cut)&&(PATTERN==0)"
		 ,"rateset=$m2_GTI_timinglightcurvefile","timebinsize=$timebin","withrateset=yes","maketimecolumn=yes"
		 ,"makeratecolumn=yes");
	system(@args)== 0 or &error_code(4);

	@args = ("tabgtigen","table=$m2_GTI_timinglightcurvefile","expression=RATE<$MOS2_Rate","gtiset=$m2_timingGTI_file");
	system(@args)== 0 or &error_code(4);
	&gti_file_inf($m2_timingGTI_file,"mos2");
    }

    if ($Log_GTIEvt_Flag == 0) {
	close(GTILOG);
	$Log_GTIEvt_Flag = 1;	
    }

    &error_code(0);
    return(0);
}

## @method set_gtifilename($instr,$mode)
# Set the name of the GTI file for EPIC
# @param instr Instrument (pn,mos1,mos2)
# @param mode Observing Mode (imaging,timing)
# @return GTI File name for a EPIC instr/mode

#========================================================================
sub set_gtifilename(){

    my $instr = $_[0];
    my $mode  = $_[1];

    if ($instr eq "pn"   && $mode eq "imaging") {$gtifile ="pn_gti_imaging.fits"};
    if ($instr eq "mos1" && $mode eq "imaging") {$gtifile ="m1_gti_imaging.fits"};
    if ($instr eq "mos2" && $mode eq "imaging") {$gtifile ="m2_gti_imaging.fits"};
    if ($instr eq "pn"   && $mode eq "timing") {$gtifile ="pn_gti_timing.fits"};
    if ($instr eq "mos1" && $mode eq "timing") {$gtifile ="m1_gti_timing.fits"};
    if ($instr eq "mos2" && $mode eq "timing") {$gtifile ="m2_gti_timing.fits"};

    return($gtifile)
}

## @method test_pngtifile()
# Test for existance of EPIC-pn GTI file and get the name of the GTI file if it exist
# @return EPIC-pn GTI File name

#========================================================================
sub test_pngtifile(){
    chdir ($gti_directory);

    my $flag = 0;
    my $file = "No file";

    $pn_GTI_file=&set_gtifilename("pn","imaging");
    if (-e $pn_GTI_file) {
	print "   \#> Imaging PN GTI File found: $pn_GTI_file \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Imaging PN GTI File found: $pn_GTI_file \n";}
	$flag=1;
    } else {
	print "   \#> Imaging PN GTI File not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Imaging PN GTI File not found \n";}	
    }

    if (! $flag) {              # if Imaging gti file does not exist, check for timing one
	$pn_GTI_file=&set_gtifilename("pn","timing");
	if (-e $pn_GTI_file) {
	    print "   \#> Timing PN GTI File found: $pn_GTI_file \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Timing PN GTI File found: $pn_GTI_file \n";}
	    $flag=1;
	} else {
	    print "   \#> Timing PN GTI File not found \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Timing PN GTI File not found \n";}	
	}
    }

    if ($flag) {$file=$gti_directory.$pn_GTI_file};
    return($flag,$file);     

}

## @method test_mos1gtifile()
# Test for existance of EPIC-MOS1 GTI file and get the name of the GTI file if it exist
# @return EPIC-MOS1 GTI File name

#========================================================================
sub test_mos1gtifile(){
    chdir ($gti_directory);
  
    my $flag = 0;
    my $file = "No file";
    
    $m1_GTI_file=&set_gtifilename("mos1","imaging");
    if (-e $m1_GTI_file) {
	print "   \#> Imaging MOS1 GTI File found: $m1_GTI_file \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Imaging MOS1 GTI File found: $m1_GTI_file \n";}
	$flag=1;
    } else {
	print "   \#> Imaging MOS1 GTI File not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Imaging MOS1 GTI File not found \n";}
    }
    
    if (! $flag) {              # if Imaging gti file does not exist, check for timing one
	$m1_GTI_file=&set_gtifilename("mos1","timing");
	if (-e $m1_GTI_file) {
	    print "   \#> Timing MOS1 GTI File found: $m1_GTI_file \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Timing MOS1 GTI File found: $m1_GTI_file \n";}
	    $flag=1;
	} else {
	    print "   \#> Timing MOS1 GTI File not found \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Timing MOS1 GTI File not found \n";}	
	}
    }

    if ($flag) {$file=$gti_directory.$m1_GTI_file};

    return($flag,$file);     
}

## @method test_mos2gtifile()
# Test for existance of EPIC-MOS2 GTI file and get the name of the GTI file if it exist
# @return EPIC-MOS2 GTI File name

#========================================================================
sub test_mos2gtifile(){
    chdir ($gti_directory);

    my $flag = 0;
    my $file = "No file";
 
    $m2_GTI_file=&set_gtifilename("mos2","imaging");
    if (-e $m2_GTI_file) {
	print "   \#> Imaging MOS2 GTI File found: $m2_GTI_file \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Imaging MOS2 GTI File found: $m2_GTI_file \n";}
	$flag=1;
    } else {
	print "   \#> Imaging MOS2 GTI File not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Imaging MOS2 GTI File not found \n";}
    }

    if (! $flag) {              # if Imaging gti file does not exist, check for timing one
	$m2_GTI_file=&set_gtifilename("mos2","timing");
	if (-e $m2_GTI_file) {
	    print "   \#> Timing MOS2 GTI File found: $m2_GTI_file \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Timing MOS2 GTI File found: $m2_GTI_file \n";}
	    $flag=1;    
	} else {
	    print "   \#> Timing MOS2 GTI File not found \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Timing MOS2 GTI File not found \n";}	
	}
    }

    if ($flag) {$file=$gti_directory.$m2_GTI_file};
    return($flag,$file);   
}

## @method void produce_gticleanfile($even_file,$even_gtifile,$gti_file,$subMode,$instr)
# Given a gti file and event file, produce a gti clean event file.
# @param even_file Event File Name
# @param even_gtifile Event GTI Clean File Name
# @param gti_file GTI File Name
# @param subMode instrument mode
# @param instr Instrument
# @return Void

#========================================================================
sub produce_gticleanfile(){

    my $even_file    = $_[0];
    my $even_gtifile = $_[1]; 
    my $gti_file     = $_[2];
    my $subMode      = $_[3];
    my $instr        = $_[4];
    

    print "   \#> Producing GTI clean event file from $even_file .... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Producing GTI clean event file from $even_file .... ";}
    
    if ($instr eq "pn") {$expr="\#XMMEA_EP && gti($gti_file,TIME) && (FLAG==0) && (PATTERN<=4) && (PI>100)";}
    if ($instr eq "mos") {
    	if (uc($subMode) eq "TIMING")
    		{$expr="\#XMMEA_EM && gti($gti_file,TIME) && (FLAG==0) && (PATTERN==0) && (PI>100)";}
    	else
    		{$expr="\#XMMEA_EM && gti($gti_file,TIME) && (FLAG==0) && (PATTERN<=12) && (PI>100)";}
    }	

    @args = ("evselect table=\"$even_file:EVENTS\" withfilteredset=Y filteredset=$even_gtifile destruct=Y keepfilteroutput=T expression=\"$expr\"");
    system(@args) == 0 or &error_code(6);

    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

    return(1);
}
1;
