## @file
# Produce OM event lists

## @method produce_omeventlist()
# Starts and controls the production of OM Event List.
# @return Flag of error/success for OM.

#========================================================================
sub produce_omeventlist(){
#.. Run omichain
    print "\n----> Start OM Analysis ....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Start OM Analysis ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for omichain ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for omichain  ..... ";}
 
    use vars qw($source_omregion_file
		$sourceomdet
		);

    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $source_ra    = $gv->SRC_RA;
    my $source_dec   = $gv->SRC_DEC; 
    
   $sourceomdet = "YES";  # Initial condition
    
    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

#.. Definition of file names
    $source_omregion_file   = $images_directory.$source_name."_omregion\.reg";    

#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {print "      Run cifbuild first \n"; &error_code(3);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(3);}
    &set_odfingest($odfingestfile);
    
#.. Print sas_setup    
    &print_sas_setup();

#.. Run omichain
    print "\n----> omichain ....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Running omichain ....\n";}
    chdir ($om_directory);

    $omevt_flag    = 0;
    @test_omevt=&test_omeventfiles();
    $om_event_list=$test_omevt[1];

    if ($test_omevt[0]) {               # If event file already exist ...
	print "   \#> OM Image Event List already exist: $om_event_list \n"; 
	print "   \#> Not running omichain \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "   \#> OM Image Event List already exist: $om_event_list \n"; 
	    print MYOUTFILE "   \#> Not running omichain \n";
	}
	$omevt_flag = 1;
    }
    if (!$omevt_flag) {                # If event files do not exist ...
	@args = ("omichain"); 
	system(@args) == 0 or return($omevt_flag);       # Dont add error if it fails, OM analysis is secundary to the whole chain:  == 0 or &error_code(3);
	$omevt_flag    = 0;
	@test_omevt=&test_omeventfiles();
	$om_event_list=$test_omevt[1];
	
	if ($test_omevt[0]) {           # Once omichain has run, do I have event files ?
	    print "   \#> After running omichain: OM Image Event List found: $om_event_list \n"; 
	    print "   \#> omichain OK \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running omichain: OM Image Event List found: $om_event_list \n"; 
		print MYOUTFILE "   \#> omichain OK \n";
	    }
	    $omevt_flag = 1;
	} 
	
	if (!$omevt_flag){
	    print "   \#> After running omichain: No OM Image Event List found \n"; 
	    print "   \#> omichain FAILED (no Events file found) \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running omichain: No OM Image Event List found \n"; 
		print MYOUTFILE "   \#> omichain FAILED (no Events file found) \n";	    
	    }
	}

    }

    if ($Log_Evt_Flag == 0) {
	my $event_log=$results_directory."event_".$source_name."\.log";
#	if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}      # Delete if exists
	if (-e $event_log) {open(EVTLOG, ">> $event_log") or die "Cant append to output file: $! \n";}  # Append if exists	
	else {open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";}   # Open for write, overwrite
	if ($test_omevt[0]) {&event_file_inf($om_event_list,"om");}
	close(EVTLOG);
#	$Log_Evt_Flag = 1;
    }

#.. Producing images 

    @om_closesource = (0.,0.,1000.);

    if ($omevt_flag){  
	print "   \#> Producing OM MOSAIC Image and Image file \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "   \#> Producing OM MOSAIC Image and Image file \n"; 
	}
	@test_omsrc=&test_omComboSRCfiles();
	$om_src_list=$test_omsrc[1];
	if ($test_omsrc[0]) {
	    @om_closesource=&om_image($om_src_list,$om_event_list);	   
	    &om_mosaic;
	}

#.. OM Cross Correlation Information 
	if ($om_closesource[2] <= $gv->OM_sourcematch){$sourceomdet = "YES";}else{$sourceomdet = "NO";} #.. Has our the source been detected in OM ?  

#.. Output log information

	if ($Log_OMEdet_Flag == 0) {
	    my $omedetect_log=$results_directory."om_chain_".$source_name."\.log";
	    if (-e $omedetect_log) {unlink("$omedetect_log") or die "Cannot rm file: $!";}    
	    open(OMLOG, "> $omedetect_log") or die "Cant open output file: $! \n";    #open for write, overwrite
	}
	print "\n   \#> Cross Correlation Information for OM: \n";
	print "       SOURCE           OM  \n";
	print "  RA       DEC     RA       DEC  \n";
	print " $source_ra $source_dec $om_closesource[0] $om_closesource[1] \n";
	print "  Distance to closest source: \n";
	print "  (If RA,DEC of the source is within ", join(", ", $gv->OM_sourcematch)," deg of RA,DEC of detected \n";
	print "  camera source, it is assumed the source is the same) \n";
	print "    SOURCE-OM : $om_closesource[2] deg. Same source ?: $sourceomdet \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "\n   \#> Cross Correlation Information for OM: \n";
	    print MYOUTFILE "       SOURCE           OM  \n";
	    print MYOUTFILE "  RA       DEC     RA       DEC  \n";
	    print MYOUTFILE " $source_ra $source_dec $om_closesource[0] $om_closesource[1] \n";
	    print MYOUTFILE "  Distance to closest source: \n";
	    print MYOUTFILE "  (If RA,DEC of the source is within ", join(", ", $gv->OM_sourcematch)," deg of RA,DEC of detected \n";
	    print MYOUTFILE "  camera source, it is assumed the source is the same) \n";
	    print MYOUTFILE "    SOURCE-OM : $om_closesource[2] deg. Same source ?: $sourceomdet \n";
	}
	if ($Log_OMEdet_Flag == 0) {
	    print OMLOG "\n   \#> Cross Correlation Information for OM: \n";
	    print OMLOG "       SOURCE           OM  \n";
	    print OMLOG "  RA       DEC     RA       DEC  \n";
	    print OMLOG " $source_ra $source_dec $om_closesource[0] $om_closesource[1] \n";
	    print OMLOG "  Distance to closest source: \n";
	    print OMLOG "  (If RA,DEC of the source is within ", join(", ", $gv->OM_sourcematch)," deg of RA,DEC of detected \n";
	    print OMLOG "  camera source, it is assumed the source is the same) \n";
	    print OMLOG "    SOURCE-OM : $om_closesource[2] deg. Same source ?: $sourceomdet \n";
	}
    }
    
    if ($Log_OMEdet_Flag == 0) {close(OMLOG);}
    &error_code(0); 

    return($omevt_flag);
}

## @method test_omeventfiles()
# Test the existance of OM Imaging Event List
# @return File Name

#========================================================================
sub test_omeventfiles(){
    chdir ($om_directory);
    my @OM_Evtlist = glob('P*OM*SIMAGE*.FIT');
    if ($#OM_Evtlist >= 0){
	print "   \#> OM Event List found: \n"; foreach (@OM_Evtlist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Event List found: \n"; foreach (@OM_Evtlist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#OM_Evtlist > 0){
	    print "      Using only $OM_Evtlist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $OM_Evtlist[0] \n";}
	}
	my $om_event_list = $OM_Evtlist[0];
	return(1,$om_directory.$om_event_list);
    } else {
	print "   \#> OM Event List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Event List not found \n";}
	return(0,"No File");
    }
}

## @method test_omSRCfiles()
# Test the existance of OM Source List
# @return File Name

#========================================================================
sub test_omSRCfiles(){
    chdir ($om_directory);
    my @OM_SRClist = glob('P*OM*SRLI*.FIT');
    if ($#OM_SRClist >= 0){
	print "   \#> OM Source List found: \n"; foreach (@OM_SRClist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Source List found: \n"; foreach (@OM_SRClist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#OM_SRClist > 0){
	    print "      Using only $OM_SRClist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $OM_SRClist[0] \n";}
	}
	my $om_src_list = $OM_SRClist[0];
	return(1,$om_directory.$om_src_list);
    } else {
	print "   \#> OM Source List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Source List not found \n";}
	return(0,"No File");
    }
}

## @method test_test_omComboSRCfiles()
# Test the existance of OM Combine Observations Source List
# @return File Name

#========================================================================
sub test_omComboSRCfiles(){
    chdir ($om_directory);
    my @OM_ComboSRClist = glob('P*OMCOMBOBSMLI*.FIT');
    if ($#OM_ComboSRClist >= 0){
	print "   \#> OM Source Combo List found: \n"; foreach (@OM_ComboSRClist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Source Combo List found: \n"; foreach (@OM_ComboSRClist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#OM_ComboSRClist > 0){
	    print "      Using only $OM_ComboSRClist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $OM_ComboSRClist[0] \n";}
	}
	my $om_Combosrc_list = $OM_ComboSRClist[0];
	return(1,$om_directory.$om_Combosrc_list);
    } else {
	print "   \#> OM Source Combo List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Source Combo List not found \n";}
	return(0,"No File");
    }
}

## @method om_image($om_srclist_file,$om_fullimage)
# Produce an OM Image and region file from Source List. Produces an OM Image with regions from region file overlaid.
# @param om_srclist_file Name of OM Source List
# @param om_fullimage Name of input Image file to be plotted with regions overlaid
# @return RA (deg) and DEC (deg) of closest OM detected source to the source of interes. Angular distance (deg) between OM and source of interest.

#========================================================================
sub om_image(){
    chdir ($images_directory);

    my $om_srclist_file = $_[0];
    my $om_fullimage    = $_[1];
     
    my $omregion_file   = $images_directory."omregion\.txt"; 

    my $source_ra     = $gv->SRC_RA;
    my $source_dec    = $gv->SRC_DEC;
    my $source_name   = $gv->SRC_Name;

#.. Run the SAS commands srcdisplay to produce region file
    @args=("srcdisplay"
	   ,"boxlistset=$om_srclist_file" 
	   ,"imageset=$om_fullimage"
	   ,"sourceradius=0.0011"
	   ,"withregionfile=yes"
	   ,"regionfile=$omregion_file"); 
    system(@args);                       

#.. From the list of detected sources in the region file find the closest source to yours
#.. and add your source to the region file to be plotted. 
    if (-e $omregion_file){
	@omclose_source=&find_closest_source($omregion_file);
	&addsourcetoregionfile($omregion_file,$omclose_source[0],$omclose_source[1],0.,0.0008,"red","","circle","fk5");
	&addsourcetoregionfile($omregion_file,$source_ra,$source_dec,0.,0.0010,"green",$source_name,"circle","fk5");	
    } else {
	print "   \#> OM Region file $omregion_file does not exist \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Region file $omregion_file does not exist \n";}	
	return(100.,100.,100.);
    }

    rename($omregion_file,$source_omregion_file);
    print "   \#> OM Region file produced: $source_omregion_file \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> OM Region file produced: $source_omregion_file \n";}

#.. Run DS9. Arguments: image file, region file, title, output name
    &produce_ps_file($om_fullimage,$source_omregion_file,"OM",$images_directory.$source_name."_OM_image.ps");

    return($omclose_source[0],$omclose_source[1],$omclose_source[2]);
}

## @method void om_mosaic()
# Produce an OM Mosaic Image with region file ovelaid
# @return Void

#========================================================================
sub om_mosaic(){
    chdir ($om_directory);

    my $obsid         = $gv->SRC_OBSID;
    my $source_name   = $gv->SRC_Name;

    my $om_mosaic_image= $om_directory."P".$obsid."OMMOSAIC.FIT";
   
    my @OM_event_file_list= glob('P*OMS*SIMAGE*.FIT');

    if ($#OM_event_file_list < 0) {print "   \#> No Images found for Mosaic Image \n"; return(0);}
    print "   \#> Mosaic Image $om_mosaic_image will be created \n";
    print "   \#> OM List for Mosaic Image: \n"; foreach (@OM_event_file_list) {print "\t".$_ ."\n";}   

    @args=("ommosaic"
	   ,"imagesets=@OM_event_file_list" 
	   ,"mosaicedset=$om_mosaic_image"
	  ); 
    system(@args);   


#.. Run DS9. Arguments: image file, region file, title, output name
    &produce_ps_file($om_mosaic_image,$source_omregion_file,"OM",$images_directory.$source_name."_OM_Mosaic_image.ps");

    return(1);

}
1;
