## @file
# Produce RGS event lists

## @method produce_rgseventlist()
# Starts and controls the production of RGS Event List.
# @return Flag of error/success for RGS.

#========================================================================
sub produce_rgseventlist(){

#.. Run rgsproc
    print "\n----> Start RGS Analysis ....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Start RGS Analysis ....\n";}

    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $source_ra    = $gv->SRC_RA;
    my $source_dec   = $gv->SRC_DEC; 

#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {print "      Run cifbuild first \n"; &error_code(3);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(3);}
    &set_odfingest($odfingestfile);
    
#.. Print sas_setup    
    &print_sas_setup();


#.. Run rgsproc
    print "\n----> Running rgsproc ....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Running rgsproc ....\n";}
    chdir ($rgs_directory);

    $rgs1evt_flag    = 0;
    @test_rgs1evt=&test_rgs1eventfiles();
    $rgs1_event_list=$test_rgs1evt[1];

    $rgs2evt_flag    = 0;
    @test_rgs2evt=&test_rgs2eventfiles();
    $rgs2_event_list=$test_rgs2evt[1];

    if ($test_rgs1evt[0]) {               # If event file already exist ...
	print "   \#> RGS1 Event List already exist: $rgs1_event_list \n"; 
	print "   \#> Not running rgsproc \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "   \#> RGS1 Event List already exist: $rgs1_event_list \n"; 
	    print MYOUTFILE "   \#> Not running rgsproc \n";
	}
	$rgs1evt_flag = 1;
    }
    if ($test_rgs2evt[0]) {               # If event file already exist ...
	print "   \#> RGS2 Event List already exist: $rgs2_event_list \n"; 
	print "   \#> Not running rgsproc \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "   \#> RGS2 Event List already exist: $rgs2_event_list \n"; 
	    print MYOUTFILE "   \#> Not running rgsproc \n";
	}
	$rgs2evt_flag = 1;
    }

    if (!$rgs1evt_flag && !$rgs2evt_flag)  { # If RGS1 or RGS2 event files do not exist ...
	
	@args = ("rgsproc"); 
	system(@args) == 0 or return($rgs1evt_flag,$rgs2evt_flag); #&error_code(3);
	$pnevt_flag    = 0;
	$rgs1evt_flag    = 0;
	@test_rgs1evt=&test_rgs1eventfiles();
	$rgs1_event_list=$test_rgs1evt[1];
	$rgs2evt_flag    = 0;
	@test_rgs2evt=&test_rgs2eventfiles();
	$rgs2_event_list=$test_rgs2evt[1];

	if ($test_rgs1evt[0]) {           # Once rgsproc has run, do I have event files ?
	    print "   \#> After running rgsproc: RGS1 Event List found: $rgs1_event_list \n"; 
	    print "   \#> rgsproc OK \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running rgsproc: RGS1 Event List found: $rgs1_event_list \n"; 
		print MYOUTFILE "   \#> rgsproc OK \n";
	    }
	    $rgs1evt_flag = 1;
	} else {
	    print "   \#> After running rgsproc: RGS1 Event List not found \n"; 
	    print "   \#> rgsproc FAILED \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running rgsproc: RGS1 Event not found \n"; 
		print MYOUTFILE "   \#> rgsproc FAILED \n";
	    }
	    $rgs1evt_flag = 0;
	}

	if ($test_rgs2evt[0]) {           # Once rgsproc has run, do I have event files ?
	    print "   \#> After running rgsproc: RGS2 Event List found: $rgs2_event_list \n"; 
	    print "   \#> rgsproc OK \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running rgsproc: RGS2 Event List found: $rgs2_event_list \n"; 
		print MYOUTFILE "   \#> rgsproc OK \n";
	    }
	    $rgs2evt_flag = 1;
	} else {
	    print "   \#> After running rgsproc: RGS2 Event List not found \n"; 
	    print "   \#> rgsproc FAILED \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running rgsproc: RGS2 Event not found \n"; 
		print MYOUTFILE "   \#> rgsproc FAILED \n";
	    }
	    $rgs2evt_flag = 0;
	}
	
	if (!$rgs1evt_flag && !$rgs2evt_flag)  {
	    print "   \#> After running rgsproc: No RGS1 or RGS2 Event List were found \n"; 
	    print "   \#> rgsproc FAILED (no Events file found) \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> After running rgsproc: No RGS1 or RGS2 Event List were found \n"; 
		print MYOUTFILE "   \#> rgsproc FAILED (no Events file found) \n";
	    }	
#	    &error_code(3) && return;
	}  
    }
    
    if ($Log_Evt_Flag == 0) {
	my $event_log=$results_directory."event_".$source_name."\.log";
#	if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}      # Delete if exists
	if (-e $event_log) {open(EVTLOG, ">> $event_log") or die "Cant append to output file: $! \n";}  # Append if exists	
	else {open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";}   # Open for write, overwrite
	if ($test_rgs1evt[0]) {&event_file_inf($rgs1_event_list,"rgs1");}
	if ($test_rgs2evt[0]) {&event_file_inf($rgs2_event_list,"rgs2");}
	close(EVTLOG);
#	$Log_Evt_Flag = 1;
    }

#.. Display the dispersion versus cross dispersion and dispersion versus energy images and 
#.. overlay the selected region masks.

    if ($rgs1evt_flag || $rgs2evt_flag){
	print "   \#> Producing dispersion vs cross dispersion and vs energy plots ... \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "   \#> Producing dispersion vs cross dispersion and vs energy plots ... \n"; 
	}
    }
    if ($rgs1evt_flag){   
	@test_rgs1src=&test_rgs1SRCfiles();
	$rgs1_SRCList_file=$test_rgs1src[1];
	if ($test_rgs1src[0]) {&get_rgsSRCList_info($rgs1_SRCList_file,"rgs1");}
	if ($test_rgs1evt[0]) {
	    $rgs1_plot_flag=&make_rgs_plot($rgs1_event_list,$rgs1_SRCList_file,"rgs1");
	    if (!$rgs1_plot_flag) {
		print "       Failed to produce plot for RGS1 \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       Failed to produce plot for RGS1 \n";}
	    }
	}      
    }
     if ($rgs2evt_flag){  
	@test_rgs2src=&test_rgs2SRCfiles();
	$rgs2_SRCList_file=$test_rgs2src[1];
	if ($test_rgs2src[0]) {&get_rgsSRCList_info($rgs2_SRCList_file,"rgs2");}
	if ($test_rgs2evt[0]) {
	    $rgs2_plot_flag=&make_rgs_plot($rgs2_event_list,$rgs2_SRCList_file,"rgs2");
	    if (!$rgs2_plot_flag) {
		print "       Failed to produce plot for RGS2 \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       Failed to produce plot for RGS2 \n";}
	    }
	}
    }

    &error_code(0); 
    return($rgs1evt_flag,$rgs2evt_flag);
}

## @method test_rgs1eventfiles()
# Test the existance of RGS1 Event List
# @return File Name

#========================================================================
sub test_rgs1eventfiles{
    chdir ($rgs_directory);
    my @RGS1_Evtlist = glob('*R1*EVENLI0000.FIT');
    if ($#RGS1_Evtlist >= 0){
	print "   \#> RGS1 Event List found: \n"; foreach (@RGS1_Evtlist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS1 Event List found: \n"; foreach (@RGS1_Evtlist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#RGS1_Evtlist > 0){
	    print "      Using only $RGS1_Evtlist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $RGS1_Evtlist[0] \n";}
	}
	my $rgs1_event_list = $RGS1_Evtlist[0];
	return(1,$rgs_directory.$rgs1_event_list);
    } else {
	print "   \#> RGS1 Event List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS1 Event List not found \n";}
	return(0,"No File");
    }
}

## @method test_rgs2eventfiles()
# Test the existance of RGS2 Event List
# @return File Name

#========================================================================
sub test_rgs2eventfiles{
    chdir ($rgs_directory);
    my @RGS2_Evtlist = glob('*R2*EVENLI0000.FIT');
    if ($#RGS2_Evtlist >= 0){
	print "   \#> RGS2 Event List found: \n"; foreach (@RGS2_Evtlist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS2 Event List found: \n"; foreach (@RGS2_Evtlist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#RGS2_Evtlist > 0){
	    print "      Using only $RGS2_Evtlist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $RGS2_Evtlist[0] \n";}
	}
	my $rgs2_event_list = $RGS2_Evtlist[0];
	return(1,$rgs_directory.$rgs2_event_list);
    } else {
	print "   \#> RGS2 Event List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS2 Event List not found \n";}
	return(0,"No File");
    }
}

## @method test_rgs1SRCfiles()
# Test the existance of RGS1 Source List
# @return File Name

#========================================================================
sub test_rgs1SRCfiles{
    chdir ($rgs_directory);
    my @RGS1_SRClist = glob('*R1*SRCLI_0000.FIT ');
    if ($#RGS1_SRClist >= 0){
	print "   \#> RGS1 Source List found: \n"; foreach (@RGS1_SRClist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS1 Source List found: \n"; foreach (@RGS1_SRClist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#RGS1_SRClist > 0){
	    print "      Using only $RGS1_SRClist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $RGS1_SRClist[0] \n";}
	}
	my $rgs1_src_list = $RGS1_SRClist[0];
	return(1,$rgs_directory.$rgs1_src_list);
    } else {
	print "   \#> RGS1 Source List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS1 Source List not found \n";}
	return(0,"No File");
    }
}

## @method test_rgs2SRCfiles()
# Test the existance of RGS2 Source List
# @return File Name

#========================================================================
sub test_rgs2SRCfiles{
    chdir ($rgs_directory);
    my @RGS2_SRClist = glob('*R2*SRCLI_0000.FIT ');
    if ($#RGS2_SRClist >= 0){
	print "   \#> RGS2 Source List found: \n"; foreach (@RGS2_SRClist) {print "\t".$_ ."\n";}   
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS2 Source List found: \n"; foreach (@RGS2_SRClist) {print MYOUTFILE "\t".$_ ."\n";}}
	if ($#RGS2_SRClist > 0){
	    print "      Using only $RGS2_SRClist[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      Using only $RGS2_SRClist[0] \n";}
	}
	my $rgs2_src_list = $RGS2_SRClist[0];
	return(1,$rgs_directory.$rgs2_src_list);
    } else {
	print "   \#> RGS2 Source List not found \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> RGS2 Source List not found \n";}
	return(0,"No File");
    }
}

## @method void get_rgsSRCList_info($in_file,$instr)
# Obtain information from RGS1/RGS2 Source List for instrument instr
# @param src_file Input Source List
# @param instr Instrument (RGS1/RGS2)
# @return Void

#========================================================================
sub get_rgsSRCList_info{

    my $src_file     = $_[0];
    my $temp_file    = $src_file."\.temp";
    my $instr        = $_[1];

    chdir($rgs_directory);
    
    @args = ("fdump \'$src_file\[1\]\' $temp_file \"RA\" 1 prhead=no showcol=no showunit=no showrow=no clobber=yes");
    system(@args)== 0 or &error_code(3);
    chomp($RGS_Proposal[0] = `cat $temp_file`);
    $RGS_Proposal[0] =~ s/\s+//;
    unlink("$temp_file") or die "Cannot rm file $temp_file: $!";
    
    @args = ("fdump \'$src_file\[1\]\' $temp_file \"RA\" 2 prhead=no showcol=no showunit=no showrow=no clobber=yes");
    system(@args)== 0 or &error_code(3);
    chomp($RGS_OnAxis[0] = `cat $temp_file`);
    $RGS_OnAxis[0] =~ s/\s+//;
    unlink("$temp_file") or die "Cannot rm file $temp_file: $!";

    @args = ("fdump \'$src_file\[1\]\' $temp_file \"DEC\" 1 prhead=no showcol=no showunit=no showrow=no clobber=yes");
    system(@args)== 0 or &error_code(3);
    chomp($RGS_Proposal[1] = `cat $temp_file`);
    $RGS_Proposal[1] =~ s/\s+//;
    unlink("$temp_file") or die "Cannot rm file $temp_file: $!";

    @args = ("fdump \'$src_file\[1\]\' $temp_file \"DEC\" 2 prhead=no showcol=no showunit=no showrow=no clobber=yes");
    system(@args)== 0 or &error_code(3);
    chomp($RGS_OnAxis[1] = `cat $temp_file`);
    $RGS_OnAxis[1] =~ s/\s+//;
    unlink("$temp_file") or die "Cannot rm file $temp_file: $!";

    $rgs_dist = &Angular_separation($RGS_Proposal[0],$RGS_OnAxis[0],$RGS_Proposal[1],$RGS_OnAxis[1]);
    print "        $instr SRC Separation (PROPOSAL and ONAXIS Source) : ",$rgs_dist*3600.," : arcsec \n";

    return(1);
}

## @method void make_rgs_plot($in_file,$src_list,$instr)
# Obtain information from RGS1/RGS2 Source List
# @param in_file Input Event List 
# @param src_list Input Source List 
# @param instr Instrument (RGS1/RGS2)
# @return Void

#========================================================================
sub make_rgs_plot{

    my $in_file    = $_[0];
    my $src_list   = $_[1];
    my $instr      = $_[2];

    my $pi_file      = "pi_".$instr."\.fit";
    my $spatial_file = "spatial_".$instr."\.fit";
    my $ps_file      = "image_".$instr."\.ps";

    if ($instr eq "rgs1"){$expr="RGS1_SRC1_SPATIAL,BETA_CORR,XDSP_CORR"}
    if ($instr eq "rgs2"){$expr="RGS2_SRC1_SPATIAL,BETA_CORR,XDSP_CORR"}
    
    @args = ("evselect table=\'$in_file:EVENTS\' imageset=\'$spatial_file\' xcolumn=\'BETA_CORR\' ycolumn=\'XDSP_CORR\'");
    system(@args) == 0 or return(0);
    
    @args = ("evselect table=\'$in_file:EVENTS\' imageset=\'$pi_file\' xcolumn=\'BETA_CORR\' ycolumn=\'PI\' yimagemin=0 yimagemax=3000 expression=\'REGION($src_list:$expr)\'");
    system(@args) == 0 or return(0);

    @args = ("rgsimplot endispset=\'$pi_file\' spatialset=\'$spatial_file\' srcidlist='1' srclistset=\'$src_list\' plotfile=\'$ps_file\' device=\/ps");
    system(@args) == 0 or return(0);

    return(1);
}
1;
