## @file
# Produce EPIC point source spectrum

## @method void produce_spectrum()
# Produce EPIC spectrum products
# @return Void

#========================================================================
sub produce_spectrum(){
    print "\n----> Working out Source Spectrum ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Working out Source Spectrum ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for Source Spectrum ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for Source Spectrum ..... ";}
 
    use vars qw($pn_event_list
		$m1_event_list
		$m2_event_list
		$pn_GTI_file
		$m1_GTI_file
		$m2_GTI_file
		$pn_gti_cleanevents_file         	  
		$pn_gti_image_file               	  
		$pn_gti_smoothimage_file         	  
		$pn_eregionanal_file             	  
		$pn_spectral_flag
		$pn_spectral_image_psfile        	  
		$pn_spectral_pileupimage_psfile  	  
		$pn_spectral_region_file       	  
		$pn_spectral_bkgregion_file    
		$pn_spectral_combinedregion_file    
		$pn_ancillary_file               	  
		$pn_redistribution_matrix_file   	  
		$pn_sourcebkgspectrum_file           	  
		$pn_bkgspectrum_file                      	  
		$pn_spectrum_psfile   
		$pn_emllist_file
		$source_pnregion_file
		$m1_gti_cleanevents_file         	  
		$m1_gti_image_file               	  
		$m1_gti_smoothimage_file         	  
		$m1_eregionanal_file    
		$m1_spectral_flag
		$m1_spectral_image_psfile        	  
		$m1_spectral_pileupimage_psfile  	  
		$m1_spectral_region_file       	  
		$m1_spectral_bkgregion_file   
		$m1_spectral_combinedregion_file 
		$m1_ancillary_file               	  
		$m1_redistribution_matrix_file   	  
		$m1_sourcebkgspectrum_file            	  
		$m1_bkgspectrum_file            
		$m1_spectrum_psfile   
		$m1_emllist_file
		$source_m1region_file
		$m2_gti_cleanevents_file         	  
		$m2_gti_image_file               	  
		$m2_gti_smoothimage_file         	  
		$m2_eregionanal_file  
		$m2_spectral_flag
		$m2_spectral_image_psfile        	  
		$m2_spectral_pileupimage_psfile  	  
		$m2_spectral_region_file       	  
		$m2_spectral_bkgregion_file
		$m2_spectral_combinedregion_file
		$m2_ancillary_file               	  
		$m2_redistribution_matrix_file   	  
		$m2_sourcebkgspectrum_file            	  
		$m2_bkgspectrum_file       
		$m2_spectrum_psfile 
		$m2_emllist_file
		$source_m2region_file
		$pnsrc_livetime
		$pnsrc_backscale
		$pnsrc_areascal
		$m1src_livetime
		$m1src_backscale
		$m1src_areascal
		$m2src_livetime
		$m2src_backscale
		$m2src_areascal
		$pnbkg_livetime
		$pnbkg_backscale
		$pnbkg_areascal
		$m1bkg_livetime
		$m1bkg_backscale
		$m1bkg_areascal
		$m2bkg_livetime
		$m2bkg_backscale
		$m2bkg_areascal
		%model_name
		);

    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $source_ra    = $gv->SRC_RA;
    my $source_dec   = $gv->SRC_DEC;
    
    $pn_gti_cleanevents_file         = $gti_directory."pn_events_gtifiltered.fit";  		       
    $pn_gti_image_file               = $gti_directory."pn_img_gtifiltered.img";			       
    $pn_gti_smoothimage_file         = $gti_directory."pn_img_gtifiltered_smooth.img";		       
    $pn_eregionanal_file             = $spectra_directory.$source_name."_pn_eregionanalyse.txt";   
    $pn_emllist_file                 = $images_directory."pn_emllist\.fits";
    $source_pnregion_file            = $images_directory.$source_name."_pnregion\.reg";    
    $pn_spectral_image_psfile        = $spectra_directory.$source_name."_pn.ps";		       
    $pn_spectral_pileupimage_psfile  = $spectra_directory.$source_name."_pn_pileup.ps";				       
    $pn_spectral_region_file         = $spectra_directory.$source_name."_pn_region.reg";	       
    $pn_spectral_bkgregion_file      = $spectra_directory.$source_name."_pn_bkgregion.reg"; 
    $pn_spectral_combinedregion_file = $spectra_directory.$source_name."_pn_combinedregion.reg"; 
    $pn_ancillary_file               = $source_name."_pn_src.arf";		       
    $pn_redistribution_matrix_file   = $source_name."_pn_src.rmf";		       
    $pn_sourcebkgspectrum_file       = $source_name."_pn_source_spectrum.fits";     
    $pn_bkgspectrum_file             = $source_name."_pn_background_spectrum.fits"; 
    $pn_spectrum_psfile              = $spectra_directory.$source_name."_pn_spectrum.ps";              
    $m1_gti_cleanevents_file         = $gti_directory."m1_events_gtifiltered.fit";  		       	  
    $m1_gti_image_file               = $gti_directory."m1_img_gtifiltered.img";			       	
    $m1_gti_smoothimage_file         = $gti_directory."m1_img_gtifiltered_smooth.img";		       	
    $m1_eregionanal_file             = $spectra_directory.$source_name."_m1_eregionanalyse.txt"; 
    $m1_emllist_file                 = $images_directory."m1_emllist\.fits";
    $source_m1region_file            = $images_directory.$source_name."_m1region\.reg";
    $m1_spectral_image_psfile        = $spectra_directory.$source_name."_m1.ps";		       	
    $m1_spectral_pileupimage_psfile  = $spectra_directory.$source_name."_m1_pileup.ps";				       	
    $m1_spectral_region_file         = $spectra_directory.$source_name."_m1_region.reg";	       	
    $m1_spectral_bkgregion_file      = $spectra_directory.$source_name."_m1_bkgregion.reg"; 
    $m1_spectral_combinedregion_file = $spectra_directory.$source_name."_m1_combinedregion.reg";    	       	  
    $m1_ancillary_file               = $source_name."_m1_src.arf";		       	  
    $m1_redistribution_matrix_file   = $source_name."_m1_src.rmf";		       	  
    $m1_sourcebkgspectrum_file       = $source_name."_m1_source_spectrum.fits";     	  
    $m1_bkgspectrum_file             = $source_name."_m1_background_spectrum.fits"; 	  
    $m1_spectrum_psfile   	     = $spectra_directory.$source_name."_m1_spectrum.ps";                            
    $m2_gti_cleanevents_file         = $gti_directory."m2_events_gtifiltered.fit";  		       		  
    $m2_gti_image_file               = $gti_directory."m2_img_gtifiltered.img";			       		  
    $m2_gti_smoothimage_file         = $gti_directory."m2_img_gtifiltered_smooth.img";		       		  
    $m2_eregionanal_file             = $spectra_directory.$source_name."_m2_eregionanalyse.txt";   
    $m2_emllist_file                 = $images_directory."m2_emllist\.fits";
    $source_m2region_file            = $images_directory.$source_name."_m2region\.reg";
    $m2_spectral_image_psfile        = $spectra_directory.$source_name."_m2.ps";		       		  
    $m2_spectral_pileupimage_psfile  = $spectra_directory.$source_name."_m2_pileup.ps";				       		  
    $m2_spectral_region_file         = $spectra_directory.$source_name."_m2_region.reg";	       		  
    $m2_spectral_bkgregion_file      = $spectra_directory.$source_name."_m2_bkgregion.reg";  
    $m2_spectral_combinedregion_file = $spectra_directory.$source_name."_m2_combinedregion.reg"; 
    $m2_ancillary_file               = $source_name."_m2_src.arf";		       		  
    $m2_redistribution_matrix_file   = $source_name."_m2_src.rmf";		       		  
    $m2_sourcebkgspectrum_file       = $source_name."_m2_source_spectrum.fits";     		  
    $m2_bkgspectrum_file             = $source_name."_m2_background_spectrum.fits"; 		  
    $m2_spectrum_psfile  	     = $spectra_directory.$source_name."_m2_spectrum.ps";                  

    $pn_spectral_flag                = 1; # Flag that determines whether the spectra should be worked out (1: True)
    $m1_spectral_flag                = 1; # Flag that determines whether the spectra should be worked out (1: True)
    $m2_spectral_flag                = 1; # Flag that determines whether the spectra should be worked out (1: True)

# Check if the user select MOS1, MOS2, PN for analysing data
	$pn_spectral_flag = $gv->PN;
	$m1_spectral_flag = $gv->MOS;	
	$m2_spectral_flag = $gv->MOS;
    
    print " OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE " OK \n";}

    if ($Log_Lcur_Flag == 0) {$temp_Log_Lcur_Flag=$Log_Lcur_Flag; $Log_Lcur_Flag = 1;} # Deactivate Spectral Log flag if active

#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {print "      Run cifbuild first \n"; &error_code(6);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {
	&transfer_odf();
	&run_odfingest();
    }
    @test_odfingest=&test_odfingest();
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(6);}
    &set_odfingest($odfingestfile);
    
#.. Print sas_setup    
    &print_sas_setup();

#.. RA & DEC Coordinates of position to analyze

#.. If run_edetect chain has been ran, use the closest source to the RA DEC given by the user for analysis
    my ($ra_pnclosesource, $dec_pnclosesource, $pnmindist_deg) = &find_closest_source($source_pnregion_file);
    my ($ra_m1closesource, $dec_m1closesource, $m1mindist_deg) = &find_closest_source($source_m1region_file);
    my ($ra_m2closesource, $dec_m2closesource, $m2mindist_deg) = &find_closest_source($source_m2region_file);
    if ($pnmindist_deg != 1000. && $pnmindist_deg <= $gv->EDC_sourcematch){
	print "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to pn).\n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";}
	$ra_anal  = $ra_pnclosesource;
	$dec_anal = $dec_pnclosesource;
    } elsif ($m1mindist_deg != 1000. && $m1mindist_deg <= $gv->EDC_sourcematch) {
	print "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to mos1).\n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";}
	$ra_anal  = $ra_m1closesource;
	$dec_anal = $dec_m1closesource;
    } elsif ($m2mindist_deg != 1000. && $m2mindist_deg <= $gv->EDC_sourcematch) { 
	print "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed (according to mos2).\n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> WARNING: analysis: RA and DEC of source to analyze has been changed.\n";}
	$ra_anal  = $ra_m2closesource;
	$dec_anal = $dec_m2closesource;
    } else {
	$ra_anal  = $source_ra;     # (deg) User Coordinates
	$dec_anal = $source_dec;    # (deg) User coordinates
    }
    print "   \#> RA & DEC of source to analyse: \n";
    print "        RA : $ra_anal deg \n";
    print "        DEC: $dec_anal deg \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> RA & DEC of source to analyse: \n";
	print MYOUTFILE "        RA : $ra_anal deg \n";
	print MYOUTFILE "        DEC: $dec_anal deg \n";
    }

#.. Test for existance of GTI files
    if ($Log_GTIEvt_Flag == 0) {
	my $gtievent_log=$results_directory."gtievent_".$source_name."\.log";
	if (-e $gtievent_log) {unlink("$gtievent_log") or die "Cannot rm file: $!";}    
	open(GTILOG, "> $gtievent_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

    @pnGTI=&test_pngtifile();   # PN          
    if ($pnGTI[0]){
	@pninfoGTI=&gti_file_inf($pn_GTI_file,"pn");
	if ($pninfoGTI[0] != 0.0){$pn_GTI_file=$pnGTI[1];} else {$pn_spectral_flag = 0;}
    } else {$pn_spectral_flag = 0;}

    @m1GTI=&test_mos1gtifile(); # MOS1
    if ($m1GTI[0]){
	@m1infoGTI=&gti_file_inf($m1_GTI_file,"mos1");
        if ($m1infoGTI[0] != 0.0) {$m1_GTI_file=$m1GTI[1];} else {$m1_spectral_flag = 0;} 
    } else {$m1_spectral_flag = 0;} 

    @m2GTI=&test_mos2gtifile(); # MOS2    
       if ($m2GTI[0]){
	@m2infoGTI=&gti_file_inf($m2_GTI_file,"mos2");
	if ($m2infoGTI[0] != 0.0) {$m2_GTI_file=$m2GTI[1];} else {$m2_spectral_flag = 0;} 
    } else {$m2_spectral_flag = 0;}

    if ($Log_GTIEvt_Flag == 0) {
	close(GTILOG);
	$Log_GTIEvt_Flag = 1;	
    }

#.. Test for existance of PN and MOS event files
    if ($Log_Evt_Flag == 0) {
	my $event_log=$results_directory."event_".$source_name."\.log";
	if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}    
	open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    } 

    @test_pnIevt=&test_pneventfiles();                                              # PN Imaging
    @test_pnTevt=&test_pntimingeventfiles();                                        # PN Timing
    if ($test_pnIevt[0]) {
	$pn_event_list=$test_pnIevt[1];
	&event_file_inf($pn_event_list,"pn");
    } elsif ($test_pnTevt[0]) { 
	$pn_event_list=$test_pnTevt[1];
	&event_file_inf($pn_event_list,"pn");
    } else {$pn_spectral_flag = 0;}
    
    @test_m1Ievt=&test_mos1eventfiles();                                            # MOS1 Imaging
    @test_m1Tevt=&test_mos1timingeventfiles();                                      # MOS1 Timing
    if ($test_m1Tevt[0]) {
	$m1_event_list=$test_m1Tevt[1];
	&event_file_inf($m1_event_list,"mos1");
    } elsif ($test_m1Ievt[0]) {
	$m1_event_list=$test_m1Ievt[1];
	&event_file_inf($m1_event_list,"mos1");
    } else {$m1_spectral_flag = 0;}
    
    @test_m2Ievt=&test_mos2eventfiles();                                            # MOS2 Imaging
    @test_m2Tevt=&test_mos2timingeventfiles();                                      # MOS2 Timing
    if ($test_m2Tevt[0]) {
	$m2_event_list=$test_m2Tevt[1];
	&event_file_inf($m2_event_list,"mos2");
    } elsif($test_m2Ievt[0]) {
	$m2_event_list=$test_m2Ievt[1];
	&event_file_inf($m2_event_list,"mos2");
    } else {$m2_spectral_flag = 0;}

#.. Get instrument's observing mode and prepare things according to shape of source and background regions.  
    $pn_obsmode = $obsid_instruments{$source_obsID}{'pn'}{'mode'};
    $m1_obsmode = $obsid_instruments{$source_obsID}{'mos1'}{'mode'};
    $m2_obsmode = $obsid_instruments{$source_obsID}{'mos2'}{'mode'};
    $pn_obssubmode    = $obsid_instruments{$source_obsID}{'pn'}{'submode'};
    $m1_obssubmode    = $obsid_instruments{$source_obsID}{'mos1'}{'submode'};
    $m2_obssubmode    = $obsid_instruments{$source_obsID}{'mos2'}{'submode'};

#.. Create GTI filtered event lists.
    if ($pn_spectral_flag ) 
    {&produce_gticleanfile($pn_event_list,$pn_gti_cleanevents_file,$pn_GTI_file,$pn_obsmode,"pn");}  # PN 
    if ($m1_spectral_flag) 
    {&produce_gticleanfile($m1_event_list,$m1_gti_cleanevents_file,$m1_GTI_file,$m1_obsmode,"mos");} # MOS1
    if ($m2_spectral_flag) 
    {&produce_gticleanfile($m2_event_list,$m2_gti_cleanevents_file,$m2_GTI_file,$m2_obsmode,"mos");} # MOS2

#.. GET some information out of the GTI filtered event lists.
    print "   \#> Information from GTI Filtered Event Files: \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Information from GTI Filtered Event Files: \n";}
    if ($pn_spectral_flag) {&event_file_inf($pn_gti_cleanevents_file,"pn");}    # PN
    if ($m1_spectral_flag) {&event_file_inf($m1_gti_cleanevents_file,"mos1");}  # MOS1
    if ($m2_spectral_flag) {&event_file_inf($m2_gti_cleanevents_file,"mos2");}  # MOS2

    if ($Log_Evt_Flag == 0) {
	close(EVTLOG);
	$Log_Evt_Flag = 1;	
    }

#.. Make an image using the filtered event file to get source centroid
    if ($pn_spectral_flag) # && !-e $pn_gti_smoothimage_file) 
    {&produce_smooth_image($pn_gti_cleanevents_file,$pn_gti_image_file,$pn_gti_smoothimage_file,"pn");}  # PN
    if ($m1_spectral_flag) # && !-e $m1_gti_smoothimage_file) 
    {&produce_smooth_image($m1_gti_cleanevents_file,$m1_gti_image_file,$m1_gti_smoothimage_file,"mos1");} # MOS1
    if ($m2_spectral_flag) # && !-e $m2_gti_smoothimage_file) 
    {&produce_smooth_image($m2_gti_cleanevents_file,$m2_gti_image_file,$m2_gti_smoothimage_file,"mos2");} # MOS2

    
#.. Using the instrument's image to work out the source centroid. Only for Imaging.

    print "   \#> Working out the centroid of source to analyze (Physical Units) .... ";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> Working out the centroid of source to analyze (Physical Units) ....";
    }

    if ($pn_spectral_flag) {

	if ($pn_obsmode eq 'IMAGING' || $pn_obsmode eq 'imaging') {
#.. Get shape of src and bkg regions:
	    $src_region_shape =  $gv->SPC_pn_srcregion;
	    $bkg_region_shape =  $gv->SPC_pn_bkgregion;
	    

#.. Run eregionanalyze at the source ra and dec position.
	    ($pn_bore_img_x,$pn_bore_img_y
	     ,$pn_bore_optrad_asec,$pn_bore_optradius
	     ,$pn_bore_img_xbkg,$pn_bore_img_ybkg
	     ,$pn_bkg_inner_rad_asec,$pn_bkg_inner_rad
	     ,$pn_bkg_outer_rad_asec,$pn_bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);
	    ($pn_bore_img_x,$pn_bore_img_y
	     ,$pn_bore_optrad_asec,$pn_bore_optradius 
	     ,$pn_bore_img_xbkg,$pn_bore_img_ybkg
	     ,$pn_bkg_inner_rad_asec,$pn_bkg_inner_rad
	     ,$pn_bkg_outer_rad_asec,$pn_bkg_outer_rad)
		=&run_eregionanalyse($pn_gti_image_file,$ra_anal,$dec_anal,"pn",$pn_obssubmode,$pn_eregionanal_file,"SPC"); # PN 
		
	    if ($pn_bore_optrad_asec == 300){$pn_spectral_flag = 0;} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

#.. Check values of parameters defining source and background regions
	    $src_XC = $gv->SPC_pn_srcregion_X;
	    $src_YC = $gv->SPC_pn_srcregion_Y;
	    $src_R1 = $gv->SPC_pn_srcregion_R1;
	    $src_R2 = $gv->SPC_pn_srcregion_R2;
	    $src_XL = $gv->SPC_pn_srcregion_XLength;
	    $src_YL = $gv->SPC_pn_srcregion_YLength;
	    $bkg_XC = $gv->SPC_pn_bkgregion_X;
	    $bkg_YC = $gv->SPC_pn_bkgregion_Y;
	    $bkg_R1 = $gv->SPC_pn_bkgregion_R1;
	    $bkg_R2 = $gv->SPC_pn_bkgregion_R2;
	    $bkg_XL = $gv->SPC_pn_bkgregion_XLength;
	    $bkg_YL = $gv->SPC_pn_bkgregion_YLength;

	    if ($src_XC == 0 && $src_YC == 0) {  # if values are set to cero take the default; from eregionanalyse 
		$gv->SPC_pn_srcregion_X($pn_bore_img_x);
		$gv->SPC_pn_srcregion_Y($pn_bore_img_y);		    
	    }

	    if ($bkg_XC == 0 && $bkg_YC == 0) { # if values are set to cero take the default; from eregionanalyse 
		$gv->SPC_pn_bkgregion_X($pn_bore_img_xbkg);
		$gv->SPC_pn_bkgregion_Y($pn_bore_img_ybkg);	
	    }

#.. SRC Region
	    if ($src_region_shape eq "circle") {
		$gv->SPC_pn_srcregion_R1(0); 
		if ($src_R2 == 0){              # if values are set to cero take the default; from eregionanalyse
		    $gv->SPC_pn_srcregion_R2($pn_bore_optradius); 
		}
		$src_area = $pi*$gv->SPC_pn_srcregion_R2*$gv->SPC_pn_srcregion_R2
		    - $pi*$gv->SPC_pn_srcregion_R1*$gv->SPC_pn_srcregion_R1; 		
	    } elsif ($src_region_shape eq "annulus") {
		if ($src_R1 == 0 || $src_R2 == 0) {
		    print "       PN SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN SRC Region should be defined \n";}
		    &error_code(6);
		}
		$src_area = $pi*$gv->SPC_pn_srcregion_R2*$gv->SPC_pn_srcregion_R2
		    - $pi*$gv->SPC_pn_srcregion_R1*$gv->SPC_pn_srcregion_R1; 	
	    } elsif ($src_region_shape eq "box") {
		if ($src_XL == 0 || $src_YL == 0) {
		    print "       PN SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN SRC Region should be defined \n";}
		    &error_code(6);
		}		
		$src_area = $gv->SPC_pn_srcregion_XLength*$gv->SPC_pn_srcregion_YLength;
	    } else {
		print "       PN SRC Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       PN SRC Region not recognized \n";}
		&error_code(6);
	    }

#.. BKG Region	    
	    if ($bkg_region_shape eq "circle") { 		
		$gv->SPC_pn_bkgregion_R1(0);
		if ($bkg_R2 == 0){ 
		    print "       PN BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN BKG Region should be defined \n";}
		    &error_code(6);
		}
		$bkg_area = $pi*$gv->SPC_pn_bkgregion_R2*$gv->SPC_pn_bkgregion_R2
		    - $pi*$gv->SPC_pn_bkgregion_R1*$gv->SPC_pn_bkgregion_R1; 
	    } elsif ($bkg_region_shape eq "annulus") {
		if ($bkg_R1 == 0 || $bkg_R2 == 0) {      # if values are set to cero take the default; from eregionanalyse
		    $gv->SPC_pn_bkgregion_R1($pn_bkg_inner_rad);
		    $gv->SPC_pn_bkgregion_R2($pn_bkg_outer_rad);
		}
		$bkg_area = $pi*$gv->SPC_pn_bkgregion_R2*$gv->SPC_pn_bkgregion_R2
		    - $pi*$gv->SPC_pn_bkgregion_R1*$gv->SPC_pn_bkgregion_R1; 
	    } elsif ($bkg_region_shape eq "box") {
		if ($bkg_XL == 0 || $bkg_YL == 0) {
		    print "       PN BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       PN BKG Region should be defined \n";}
		    &error_code(6);
		}
		$bkg_area = $gv->SPC_pn_bkgregion_XLength*$gv->SPC_pn_bkgregion_YLength;
	    } else {
		print "       PN BKG Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       PN BKG Region not recognized \n";}
		&error_code(6);
	    }

	    $gv->SPC_pn_areafactor($src_area/$bkg_area);

	    if ($gv->SPC_pn_srcregion_X == 0 && $gv->SPC_pn_srcregion_Y == 0){$pn_spectral_flag = 0;}  # Something has gone wrong so wont produce spectra.

	} elsif ($pn_obsmode eq 'TIMING' || $pn_obsmode eq 'timing') {

	    if ($gv->SPC_pn_ener_low <= 0.5) {$gv->SPC_pn_ener_low(0.5)}; # If Timing mode, use 0.5. Kev for low energy limit for the fit, 
                                                                          # or whatever the user specifies as long as is above 0.5 keV
	    $gv->SPC_pn_srcregion("box");
	    $src_XC = $gv->SPC_pn_srcregion_X;
	    $src_YC = $gv->SPC_pn_srcregion_Y;
	    $src_XL = $gv->SPC_pn_srcregion_XLength;
	    $src_YL = $gv->SPC_pn_srcregion_YLength;
	    
	    if (($src_XC == 0 && $src_YC == 0)
		&& ($src_XL == 0 && $src_XL == 0)) {
		$gv->SPC_pn_srcregion_X(37.);
		$gv->SPC_pn_srcregion_Y(101.);
		$gv->SPC_pn_srcregion_XLength(16);
		$gv->SPC_pn_srcregion_YLength(199);
	    } 
	    
	    $gv->SPC_pn_bkgregion("box");
	    $bkg_XC = $gv->SPC_pn_bkgregion_X;
	    $bkg_YC = $gv->SPC_pn_bkgregion_Y;
	    $bkg_XL = $gv->SPC_pn_bkgregion_XLength;
	    $bkg_YL = $gv->SPC_pn_bkgregion_YLength;
	    
	    if (($bkg_XC == 0 && $bkg_YC == 0)
		&& ($bkg_XL == 0 && $bkg_XL == 0)) {
		$gv->SPC_pn_bkgregion_X($gv->SPC_pn_srcregion_XLength/2.+0.5); # +0.5 to keep it away from the edge of the detector
		$gv->SPC_pn_bkgregion_Y($gv->SPC_pn_srcregion_Y);
		$gv->SPC_pn_bkgregion_XLength($gv->SPC_pn_srcregion_XLength);
		$gv->SPC_pn_bkgregion_YLength($gv->SPC_pn_srcregion_YLength);
	    } 

	    $src_area = $gv->SPC_pn_srcregion_XLength*$gv->SPC_pn_srcregion_YLength;
	    $bkg_area = $gv->SPC_pn_bkgregion_XLength*$gv->SPC_pn_bkgregion_YLength;
	    $gv->SPC_pn_areafactor($src_area/$bkg_area);

	} else {
	    print "       PN Mode not recognized \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       PN Mode not recognized \n";}
	    &error_code(6);
	}

	print "       PN \n";
	print "         PN Region SRC : ",$gv->SPC_pn_srcregion,"\n";
	print "         PN SRC XC     : ",$gv->SPC_pn_srcregion_X,"\n";
	print "         PN SRC YC     : ",$gv->SPC_pn_srcregion_Y,"\n";
	if ($gv->SPC_pn_srcregion eq 'circle' || $gv->SPC_pn_srcregion eq 'annulus')
	{print "         PN SRC RC     : ",$gv->SPC_pn_srcregion_R1," - ",$gv->SPC_pn_srcregion_R2,"\n";} 
	if ($gv->SPC_pn_srcregion eq 'box' )
	{print "         PN SRC BOX    : ",$gv->SPC_pn_srcregion_XLength," - ",$gv->SPC_pn_srcregion_YLength,"\n";}
	print "         PN Region BKG : ",$gv->SPC_pn_bkgregion,"\n";
	print "         PN BKG XC     : ",$gv->SPC_pn_bkgregion_X,"\n";
	print "         PN BKG YC     : ",$gv->SPC_pn_bkgregion_Y,"\n";
	if ($gv->SPC_pn_bkgregion eq 'circle' || $gv->SPC_pn_bkgregion eq 'annulus')
	{
	    print "         PN BKG RC     : ",$gv->SPC_pn_bkgregion_R1," - ",$gv->SPC_pn_bkgregion_R2,"\n";
	    if ($gv->SPC_pn_srcregion_R2 > $gv->SPC_pn_bkgregion_R1) {
		print "         WARNING: PN regions : The inner background region radii is smaller than the source region radii \n";
	    } 
	} 
	if ($gv->SPC_pn_bkgregion eq 'box' )
	{print "         PN BKG BOX    : ",$gv->SPC_pn_bkgregion_XLength," - ",$gv->SPC_pn_bkgregion_YLength,"\n";}
	print "         PN BKG AREA SCALE FACTOR : ",$gv->SPC_pn_areafactor," (Only area taken into account)\n";
	if ($gv->SPC_pn_srcregion_R2 == 6000){print "         WARNING : PN regions : Maximum allowed radius reached";}
	if ($Log_Flag == 0) {
	    print MYOUTFILE "       PN \n";
	    print MYOUTFILE "         PN Region SRC  : ",$gv->SPC_pn_srcregion,"\n";	
	    print MYOUTFILE "         PN SRC XC      : ",$gv->SPC_pn_srcregion_X,"\n";
	    print MYOUTFILE "         PN SRC YC      : ",$gv->SPC_pn_srcregion_Y,"\n";
	    if ($gv->SPC_pn_srcregion eq 'circle' || $gv->SPC_pn_srcregion eq 'annulus')
	    {print MYOUTFILE "         PN SRC RC      : ",$gv->SPC_pn_srcregion_R1," - ",$gv->SPC_pn_srcregion_R2,"\n";} 
	    if ($gv->SPC_pn_srcregion eq 'box' )
	    {print MYOUTFILE "         PN SRC BOX     : ",$gv->SPC_pn_srcregion_XLength," - ",$gv->SPC_pn_srcregion_YLength,"\n";}
	    print MYOUTFILE "         PN Region BKG  : ",$gv->SPC_pn_bkgregion,"\n";
	    print MYOUTFILE "         PN BKG XC      : ",$gv->SPC_pn_bkgregion_X,"\n";
	    print MYOUTFILE "         PN BKG YC      : ",$gv->SPC_pn_bkgregion_Y,"\n";
	    if ($gv->SPC_pn_bkgregion eq 'circle' || $gv->SPC_pn_bkgregion eq 'annulus')
	    {
		print MYOUTFILE "         PN BKG RC      : ",$gv->SPC_pn_bkgregion_R1," - ",$gv->SPC_pn_bkgregion_R2,"\n";
		if ($gv->SPC_pn_srcregion_R2 > $gv->SPC_pn_bkgregion_R1) {
		    print MYOUTFILE "         WARNING: PN regions : The inner background region radii is smaller than the source region radii \n";
		} 
	    } 
	    if ($gv->SPC_pn_bkgregion eq 'box' )
	    {print MYOUTFILE "         PN BKG BOX     : ",$gv->SPC_pn_bkgregion_XLength," - ",$gv->SPC_pn_bkgregion_YLength,"\n";}
	    print MYOUTFILE "         PN BKG AREA SCALE FACTOR : ",$gv->SPC_pn_areafactor," (Only area taken into account)\n";
	    if ($gv->SPC_pn_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : PN regions : Maximum allowed radius reached";}
	}
    }
    
    if ($m1_spectral_flag) {

	if ($m1_obsmode eq 'IMAGING' || $m1_obsmode eq 'imaging') {
#.. Get shape of src and bkg regions:
	    $src_region_shape =  $gv->SPC_m1_srcregion;
	    $bkg_region_shape =  $gv->SPC_m1_bkgregion;
   
	    ($m1_bore_img_x,$m1_bore_img_y
	     ,$m1_bore_optrad_asec,$m1_bore_optradius 
	     ,$m1_bore_img_xbkg,$m1_bore_img_ybkg
	     ,$m1_bkg_inner_rad_asec,$m1_bkg_inner_rad
	     ,$m1_bkg_outer_rad_asec,$m1_bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);
	    ($m1_bore_img_x,$m1_bore_img_y
	     ,$m1_bore_optrad_asec,$m1_bore_optradius
	     ,$m1_bore_img_xbkg,$m1_bore_img_ybkg
	     ,$m1_bkg_inner_rad_asec,$m1_bkg_inner_rad
	     ,$m1_bkg_outer_rad_asec,$m1_bkg_outer_rad)
		=&run_eregionanalyse($m1_gti_image_file,$ra_anal,$dec_anal,"mos1",$m1_obssubmode,$m1_eregionanal_file,"SPC"); # MOS1

	    if ($m1_bore_optrad_asec == 300){$m1_spectral_flag = 0;} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

#.. Check values of parameters defining source and background regions	    
	    $src_XC = $gv->SPC_m1_srcregion_X;
	    $src_YC = $gv->SPC_m1_srcregion_Y;
	    $src_R1 = $gv->SPC_m1_srcregion_R1;
	    $src_R2 = $gv->SPC_m1_srcregion_R2;
	    $src_XL = $gv->SPC_m1_srcregion_XLength;
	    $src_YL = $gv->SPC_m1_srcregion_YLength;
	    $bkg_XC = $gv->SPC_m1_bkgregion_X;
	    $bkg_YC = $gv->SPC_m1_bkgregion_Y;
	    $bkg_R1 = $gv->SPC_m1_bkgregion_R1;
	    $bkg_R2 = $gv->SPC_m1_bkgregion_R2;
	    $bkg_XL = $gv->SPC_m1_bkgregion_XLength;
	    $bkg_YL = $gv->SPC_m1_bkgregion_YLength;

	    if ($src_XC ==0 && $src_YC == 0) {  # if values are set to cero take the default; from eregionanalyse 
		$gv->SPC_m1_srcregion_X($m1_bore_img_x);
		$gv->SPC_m1_srcregion_Y($m1_bore_img_y);		    
	    }
	    
	    if ($bkg_XC == 0 && $bkg_YC == 0) { # if values are set to cero take the default; from eregionanalyse 
		$gv->SPC_m1_bkgregion_X($m1_bore_img_xbkg);
		$gv->SPC_m1_bkgregion_Y($m1_bore_img_ybkg);	
	    }

#.. SRC Region	    
	    if ($src_region_shape eq "circle") { 
		$gv->SPC_m1_srcregion_R1(0); 
		if ($src_R2 == 0){              # if values are set to cero take the default; from eregionanalyse
		    $gv->SPC_m1_srcregion_R2($m1_bore_optradius); 
		}
		$src_area = $pi*$gv->SPC_m1_srcregion_R2*$gv->SPC_m1_srcregion_R2
		    - $pi*$gv->SPC_m1_srcregion_R1*$gv->SPC_m1_srcregion_R1;
	    } elsif ($src_region_shape eq "annulus") {
		if ($src_R1 == 0 || $src_R2 == 0) {
		    print "       MOS1 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 SRC Region should be defined \n";}
		    &error_code(6);
		}
		$src_area = $pi*$gv->SPC_m1_srcregion_R2*$gv->SPC_m1_srcregion_R2
		    - $pi*$gv->SPC_m1_srcregion_R1*$gv->SPC_m1_srcregion_R1;
	    } elsif ($src_region_shape eq "box") {
		if ($src_XL == 0 || $src_YL == 0) {
		    print "       MOS1 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 SRC Region should be defined \n";}
		    &error_code(6);
		}
		$src_area = $gv->SPC_m1_srcregion_XLength*$gv->SPC_m1_srcregion_YLength;
	    } else {
		print "       MOS1 SRC Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 SRC Region not recognized \n";}
		&error_code(6);
	    }
	    
#.. BKG Region	    
	    if ($bkg_region_shape eq "circle") { 		
		$gv->SPC_m1_bkgregion_R1(0);
		if ($bkg_R2 == 0){ 
		    print "       MOS1 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 BKG Region should be defined \n";}
		    &error_code(6);
		}
		$bkg_area = $pi*$gv->SPC_m1_bkgregion_R2*$gv->SPC_m1_bkgregion_R2
		    - $pi*$gv->SPC_m1_bkgregion_R1*$gv->SPC_m1_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "annulus") {
		if ($bkg_R1 == 0 || $bkg_R2 == 0) {      # if values are set to cero take the default; from eregionanalyse
		    $gv->SPC_m1_bkgregion_R1($m1_bkg_inner_rad);
		    $gv->SPC_m1_bkgregion_R2($m1_bkg_outer_rad);
		}
		$bkg_area = $pi*$gv->SPC_m1_bkgregion_R2*$gv->SPC_m1_bkgregion_R2
		    - $pi*$gv->SPC_m1_bkgregion_R1*$gv->SPC_m1_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "box") {
		if ($bkg_XL == 0 || $bkg_YL == 0) {
		    print "       MOS1 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 BKG Region should be defined \n";}
		    &error_code(6);
		}
		$bkg_area = $gv->SPC_m1_bkgregion_XLength*$gv->SPC_m1_bkgregion_YLength;
	    } else {
		print "       MOS1 BKG Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 BKG Region not recognized \n";}
		&error_code(6);
	    }
	    
	    $gv->SPC_m1_areafactor($src_area/$bkg_area);
	   
	    if ($gv->SPC_m1_srcregion_X == 0 && $gv->SPC_m1_srcregion_Y == 0){$m1_spectral_flag = 0;}      # Something has gone wrong so wont produce spectra
	    
	} elsif ($m1_obsmode eq 'TIMING' || $m1_obsmode eq 'timing') {

	    if ($gv->SPC_m1_ener_low <= 0.5) {$gv->SPC_m1_ener_low(0.5)}; # If Timing mode, use 0.5. Kev for low energy limit for the fit, 
                                                                          # or whatever the user specifies as long as is above 0.5 keV
	    $gv->SPC_m1_srcregion("box");
	    $src_XC = $gv->SPC_m1_srcregion_X;
	    $src_YC = $gv->SPC_m1_srcregion_Y;
	    $src_XL = $gv->SPC_m1_srcregion_XLength;
	    $src_YL = $gv->SPC_m1_srcregion_YLength;
	    if (($src_XC == 0 && $src_YC == 0)
		&& ($src_XL == 0 && $src_XL == 0)) {
		$t_start=$obsid_instruments{$source_obsID}{'mos1'}{'starting_time'};
		$t_end=$obsid_instruments{$source_obsID}{'mos1'}{'ending_time'};
		$interval=$t_end-$t_start;
		$mid_interval=$t_start+$interval/2.;		
		$gv->SPC_m1_srcregion_X(299.5);
		$gv->SPC_m1_srcregion_Y($mid_interval);
		$gv->SPC_m1_srcregion_XLength(20);
		$gv->SPC_m1_srcregion_YLength($interval);
	    } 
	    
	    $gv->SPC_m1_bkgregion("box");
	    $bkg_XC = $gv->SPC_m1_bkgregion_X;
	    $bkg_YC = $gv->SPC_m1_bkgregion_Y;
	    $bkg_XL = $gv->SPC_m1_bkgregion_XLength;
	    $bkg_YL = $gv->SPC_m1_bkgregion_YLength;
	    
	    if (($bkg_XC == 0 && $bkg_YC == 0)
		&& ($bkg_XL == 0 && $bkg_XL == 0)) {
		$gv->SPC_m1_bkgregion_X(265.5); # lowest column value in MOS Timing 256
		$gv->SPC_m1_bkgregion_Y($gv->SPC_m1_srcregion_Y);
		$gv->SPC_m1_bkgregion_XLength($gv->SPC_m1_srcregion_XLength);
		$gv->SPC_m1_bkgregion_YLength($gv->SPC_m1_srcregion_YLength);
	    }

	    $src_area = $gv->SPC_m1_srcregion_XLength*$gv->SPC_m1_srcregion_YLength;
	    $bkg_area = $gv->SPC_m1_bkgregion_XLength*$gv->SPC_m1_bkgregion_YLength;
	    $gv->SPC_m1_areafactor($src_area/$bkg_area);

	} else {
	    print "       MOS1 Mode not recognized \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       MOS1 Mode not recognized \n";}
	    &error_code(6);
	}

	print "       MOS1 \n";
	print "         MOS1 Region SRC  : ",$gv->SPC_m1_srcregion,"\n";
	print "         MOS1 SRC XC      : ",$gv->SPC_m1_srcregion_X,"\n";
	print "         MOS1 SRC YC      : ",$gv->SPC_m1_srcregion_Y,"\n";
	if ($gv->SPC_m1_srcregion eq 'circle' || $gv->SPC_m1_srcregion eq 'annulus')
	{print "         MOS1 SRC RC      : ",$gv->SPC_m1_srcregion_R1," - ",$gv->SPC_m1_srcregion_R2,"\n";} 
	if ($gv->SPC_m1_srcregion eq 'box' )
	{print "         MOS1 SRC BOX     : ",$gv->SPC_m1_srcregion_XLength," - ",$gv->SPC_m1_srcregion_YLength,"\n";}
	print "         MOS1 Region BKG  : ",$gv->SPC_m1_bkgregion,"\n";
	print "         MOS1 BKG XC      : ",$gv->SPC_m1_bkgregion_X,"\n";
	print "         MOS1 BKG YC      : ",$gv->SPC_m1_bkgregion_Y,"\n";	   
	if ($gv->SPC_m1_bkgregion eq 'circle' || $gv->SPC_m1_bkgregion eq 'annulus')
	{
	    print "         MOS1 BKG RC      : ",$gv->SPC_m1_bkgregion_R1," - ",$gv->SPC_m1_bkgregion_R2,"\n";
	    if ($gv->SPC_m1_srcregion_R2 > $gv->SPC_m1_bkgregion_R1) {
		print "         WARNING: MOS1 regions : The inner background region radii is smaller than the source region radii \n";
	    } 
	} 
	if ($gv->SPC_m1_bkgregion eq 'box' )
	{print "         MOS1 BKG BOX     : ",$gv->SPC_m1_bkgregion_XLength," - ",$gv->SPC_m1_bkgregion_YLength,"\n";}
	print "         MOS1 BKG AREA SCALE FACTOR : ",$gv->SPC_m1_areafactor," (Only area taken into account)\n";
	if ($gv->SPC_m1_srcregion_R2 == 6000){print "         WARNING : MOS1 regions : Maximum allowed radius reached";}	
	if ($Log_Flag == 0) {
	    print MYOUTFILE "       MOS1 \n";
	    print MYOUTFILE "         MOS1 Region SRC  : ",$gv->SPC_m1_srcregion,"\n";
	    print MYOUTFILE "         MOS1 SRC XC      : ",$gv->SPC_m1_srcregion_X,"\n";
	    print MYOUTFILE "         MOS1 SRC YC      : ",$gv->SPC_m1_srcregion_Y,"\n";
	    if ($gv->SPC_m1_srcregion eq 'circle' || $gv->SPC_m1_srcregion eq 'annulus')
	    {print MYOUTFILE "         MOS1 SRC RC      : ",$gv->SPC_m1_srcregion_R1," - ",$gv->SPC_m1_srcregion_R2,"\n";} 
	    if ($gv->SPC_m1_srcregion eq 'box' )
	    {print MYOUTFILE "         MOS1 SRC BOX     : ",$gv->SPC_m1_srcregion_XLength," - ",$gv->SPC_m1_srcregion_YLength,"\n";}
	    print MYOUTFILE "         MOS1 Region BKG  : ",$gv->SPC_m1_bkgregion,"\n";
	    print MYOUTFILE "         MOS1 BKG XC      : ",$gv->SPC_m1_bkgregion_X,"\n";
	    print MYOUTFILE "         MOS1 BKG YC      : ",$gv->SPC_m1_bkgregion_Y,"\n";
	    if ($gv->SPC_m1_bkgregion eq 'circle' || $gv->SPC_m1_bkgregion eq 'annulus')
	    {
		print MYOUTFILE "         MOS1 BKG RC      : ",$gv->SPC_m1_bkgregion_R1," - ",$gv->SPC_m1_bkgregion_R2,"\n";
		if ($gv->SPC_m1_srcregion_R2 > $gv->SPC_m1_bkgregion_R1) {
		    print MYOUTFILE "         WARNING: MOS1 regions : The inner background region radii is smaller than the source region radii \n";
		} 
	    } 
	    if ($gv->SPC_m1_bkgregion eq 'box' )
	    {print MYOUTFILE "         MOS1 BKG BOX     : ",$gv->SPC_m1_bkgregion_XLength," - ",$gv->SPC_m1_bkgregion_YLength,"\n";}
	    print MYOUTFILE "         MOS1 BKG AREA SCALE FACTOR : ",$gv->SPC_m1_areafactor," (Only area taken into account)\n";
	    if ($gv->SPC_m1_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : MOS1 regions : Maximum allowed radius reached";}
	}
    }

    if ($m2_spectral_flag) { 

	if ($m2_obsmode eq 'IMAGING' || $m2_obsmode eq 'imaging') {
#.. Get shape of src and bkg regions:
	    $src_region_shape =  $gv->SPC_m2_srcregion;
	    $bkg_region_shape =  $gv->SPC_m2_bkgregion;
	    
	    ($m2_bore_img_x,$m2_bore_img_y
	     ,$m2_bore_optrad_asec,$m2_bore_optradius 
	     ,$m2_bore_img_xbkg,$m2_bore_img_ybkg
	     ,$m2_bkg_inner_rad_asec,$m2_bkg_inner_rad
	     ,$m2_bkg_outer_rad_asec,$m2_bkg_outer_rad) = (0,0,0,0,0,0,0,0,0,0);	    
	    ($m2_bore_img_x,$m2_bore_img_y
	     ,$m2_bore_optrad_asec,$m2_bore_optradius
	     ,$m2_bore_img_xbkg,$m2_bore_img_ybkg
	     ,$m2_bkg_inner_rad_asec,$m2_bkg_inner_rad
	     ,$m2_bkg_outer_rad_asec,$m2_bkg_outer_rad)
		=&run_eregionanalyse($m2_gti_image_file,$ra_anal,$dec_anal,"mos2",$m2_obssubmode,$m2_eregionanal_file,"SPC"); # MOS2 

	    if ($m2_bore_optrad_asec == 300){$m2_spectral_flag = 0;} # Something has gone wrong so wont produce spectra. 300 is maximum allowed value.

#.. Check values of parameters defining source and background regions	    	    
	    $src_XC = $gv->SPC_m2_srcregion_X;
	    $src_YC = $gv->SPC_m2_srcregion_Y;
	    $src_R1 = $gv->SPC_m2_srcregion_R1;
	    $src_R2 = $gv->SPC_m2_srcregion_R2;
	    $src_XL = $gv->SPC_m2_srcregion_XLength;
	    $src_YL = $gv->SPC_m2_srcregion_YLength;
	    $bkg_XC = $gv->SPC_m2_bkgregion_X;
	    $bkg_YC = $gv->SPC_m2_bkgregion_Y;
	    $bkg_R1 = $gv->SPC_m2_bkgregion_R1;
	    $bkg_R2 = $gv->SPC_m2_bkgregion_R2;
	    $bkg_XL = $gv->SPC_m2_bkgregion_XLength;
	    $bkg_YL = $gv->SPC_m2_bkgregion_YLength;

	    if ($src_XC ==0 && $src_YC == 0) {  # if values are set to cero take the default; from eregionanalyse 
		$gv->SPC_m2_srcregion_X($m2_bore_img_x);
		$gv->SPC_m2_srcregion_Y($m2_bore_img_y);		    
	    }
	    
	    if ($bkg_XC == 0 && $bkg_YC == 0) { # if values are set to cero take the default; from eregionanalyse 
		$gv->SPC_m2_bkgregion_X($m2_bore_img_xbkg);
		$gv->SPC_m2_bkgregion_Y($m2_bore_img_ybkg);	
	    }

#.. SRC Region	    
	    if ($src_region_shape eq "circle") { 
		$gv->SPC_m2_srcregion_R1(0); 
		if ($src_R2 == 0){              # if values are set to cero take the default; from eregionanalyse
		    $gv->SPC_m2_srcregion_R2($m2_bore_optradius); 
		}
		$src_area = $pi*$gv->SPC_m2_srcregion_R2*$gv->SPC_m2_srcregion_R2
		    - $pi*$gv->SPC_m2_srcregion_R1*$gv->SPC_m2_srcregion_R1;
	    } elsif ($src_region_shape eq "annulus") {
		if ($src_R1 == 0 || $src_R2 == 0) {
		    print "       MOS2 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 SRC Region should be defined \n";}
		    &error_code(6);
		}
		$src_area = $pi*$gv->SPC_m2_srcregion_R2*$gv->SPC_m2_srcregion_R2
		    - $pi*$gv->SPC_m2_srcregion_R1*$gv->SPC_m2_srcregion_R1;
	    } elsif ($src_region_shape eq "box") {
		if ($src_XL == 0 || $src_YL == 0) {
		    print "       MOS2 SRC Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 SRC Region should be defined \n";}
		    &error_code(6);
		}
		$src_area = $gv->SPC_m2_srcregion_XLength*$gv->SPC_m2_srcregion_YLength;
	    } else {
		print "       MOS2 SRC Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 SRC Region not recognized \n";}
		&error_code(6);
	    }
	    
#.. BKG Region	    
	    if ($bkg_region_shape eq "circle") { 
		$gv->SPC_m2_bkgregion_R1(0);
		if ($bkg_R2 == 0){ 
		    print "       MOS2 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 BKG Region should be defined \n";}
		    &error_code(6);
		}
		$bkg_area = $pi*$gv->SPC_m2_bkgregion_R2*$gv->SPC_m2_bkgregion_R2
		    - $pi*$gv->SPC_m2_bkgregion_R1*$gv->SPC_m2_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "annulus") {	
		if ($bkg_R1 == 0 || $bkg_R2 == 0) {      # if values are set to cero take the default; from eregionanalyse
		    $gv->SPC_m2_bkgregion_R1($m2_bkg_inner_rad);
		    $gv->SPC_m2_bkgregion_R2($m2_bkg_outer_rad);
		}
		$bkg_area = $pi*$gv->SPC_m2_bkgregion_R2*$gv->SPC_m2_bkgregion_R2
		    - $pi*$gv->SPC_m2_bkgregion_R1*$gv->SPC_m2_bkgregion_R1;
	    } elsif ($bkg_region_shape eq "box") {
		if ($bkg_XL == 0 || $bkg_YL == 0) {
		    print "       MOS2 BKG Region should be defined \n";
		    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 BKG Region should be defined \n";}
		    &error_code(6);
		}
		$bkg_area = $gv->SPC_m2_bkgregion_XLength*$gv->SPC_m2_bkgregion_YLength;
	    } else {
		print "       MOS2 BKG Region not recognized \n";
		if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 BKG Region not recognized \n";}
		&error_code(6);
	    }	    

	    $gv->SPC_m2_areafactor($src_area/$bkg_area);

	    if ($gv->SPC_m2_srcregion_X == 0 && $gv->SPC_m2_srcregion_Y){$m2_spectral_flag = 0;}      # Something has gone wrong so wont produce spectra
	    
	} elsif ($m2_obsmode eq 'TIMING' || $m2_obsmode eq 'timing') {

	    if ($gv->SPC_m2_ener_low <= 0.5) {$gv->SPC_m2_ener_low(0.5)}; # If Timing mode, use 0.5. Kev for low energy limit for the fit, 
                                                                          # or whatever the user specifies as long as is above 0.5 keV

	    $gv->SPC_m2_srcregion("box");
	    $src_XC = $gv->SPC_m2_srcregion_X;
	    $src_YC = $gv->SPC_m2_srcregion_Y;
	    $src_XL = $gv->SPC_m2_srcregion_XLength;
	    $src_YL = $gv->SPC_m2_srcregion_YLength;
	    
	    if (($src_XC == 0 && $src_YC == 0)
		&& ($src_XL == 0 && $src_XL == 0)) {
		$t_start=$obsid_instruments{$source_obsID}{'mos2'}{'starting_time'};
		$t_end=$obsid_instruments{$source_obsID}{'mos2'}{'ending_time'};
		$interval=$t_end-$t_start;
		$mid_interval=$t_start+$interval/2.;
		$gv->SPC_m2_srcregion_X(299.5);
		$gv->SPC_m2_srcregion_Y($mid_interval);
		$gv->SPC_m2_srcregion_XLength(20);
		$gv->SPC_m2_srcregion_YLength($interval);
	    } 
	    
	    $gv->SPC_m2_bkgregion("box");
	    $bkg_XC = $gv->SPC_m2_bkgregion_X;
	    $bkg_YC = $gv->SPC_m2_bkgregion_Y;
	    $bkg_XL = $gv->SPC_m2_bkgregion_XLength;
	    $bkg_YL = $gv->SPC_m2_bkgregion_YLength;

	    if (($bkg_XC == 0 && $bkg_YC == 0)
		&& ($bkg_XL == 0 && $bkg_XL == 0)) {
		$gv->SPC_m2_bkgregion_X(265.);  # lowest column value in MOS Timing 131
		$gv->SPC_m2_bkgregion_Y($gv->SPC_m2_srcregion_Y);
		$gv->SPC_m2_bkgregion_XLength($gv->SPC_m2_srcregion_XLength);
		$gv->SPC_m2_bkgregion_YLength($gv->SPC_m2_srcregion_YLength);
	    }
	    
	    $src_area = $gv->SPC_m2_srcregion_XLength*$gv->SPC_m2_srcregion_YLength;
	    $bkg_area = $gv->SPC_m2_bkgregion_XLength*$gv->SPC_m2_bkgregion_YLength;
	    $gv->SPC_m2_areafactor($src_area/$bkg_area);

	} else {
	    print "       MOS2 Mode not recognized \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       MOS2 Mode not recognized \n";}
	    &error_code(6);
	}
	
	print "       MOS2 \n";
	print "         MOS2 Region SRC  : ",$gv->SPC_m2_srcregion,"\n";
	print "         MOS2 SRC XC      : ",$gv->SPC_m2_srcregion_X,"\n";
	print "         MOS2 SRC YC      : ",$gv->SPC_m2_srcregion_Y,"\n";
	if ($gv->SPC_m2_srcregion eq 'circle' || $gv->SPC_m2_srcregion eq 'annulus')
	{print "         MOS2 SRC RC      : ",$gv->SPC_m2_srcregion_R1," - ",$gv->SPC_m2_srcregion_R2,"\n";} 
	if ($gv->SPC_m2_srcregion eq 'box' )
	{print "         MOS2 SRC BOX     : ",$gv->SPC_m2_srcregion_XLength," - ",$gv->SPC_m2_srcregion_YLength,"\n";}
	print "         MOS2 Region BKG  : ",$gv->SPC_m2_bkgregion,"\n";
	print "         MOS2 BKG XC      : ",$gv->SPC_m2_bkgregion_X,"\n";
	print "         MOS2 BKG YC      : ",$gv->SPC_m2_bkgregion_Y,"\n";	   
	if ($gv->SPC_m2_bkgregion eq 'circle' || $gv->SPC_m2_bkgregion eq 'annulus')
	{
	    print "         MOS2 BKG RC      : ",$gv->SPC_m2_bkgregion_R1," - ",$gv->SPC_m2_bkgregion_R2,"\n";
	    if ($gv->SPC_m2_srcregion_R2 > $gv->SPC_m2_bkgregion_R1) {
		print "         WARNING: MOS2 regions : The inner background region radii is smaller than the source region radii \n";
	    } 
	} 
	if ($gv->SPC_m2_bkgregion eq 'box' )
	{print "         MOS2 BKG BOX     : ",$gv->SPC_m2_bkgregion_XLength," - ",$gv->SPC_m2_bkgregion_YLength,"\n";}
	print "         MOS2 BKG AREA SCALE FACTOR : ",$gv->SPC_m2_areafactor," (Only area taken into account)\n";
	if ($gv->SPC_m2_srcregion_R2 == 6000){print "         WARNING : MOS2 regions : Maximum allowed radius reached";}
	if ($Log_Flag == 0) {
	    print MYOUTFILE "       MOS2 \n";
	    print MYOUTFILE "         MOS2 Region SRC  : ",$gv->SPC_m2_srcregion,"\n";
	    print MYOUTFILE "         MOS2 SRC XC      : ",$gv->SPC_m2_srcregion_X,"\n";
	    print MYOUTFILE "         MOS2 SRC YC      : ",$gv->SPC_m2_srcregion_Y,"\n";
	    if ($gv->SPC_m2_srcregion eq 'circle' || $gv->SPC_m2_srcregion eq 'annulus')
	    {print MYOUTFILE "         MOS2 SRC RC      : ",$gv->SPC_m2_srcregion_R1," - ",$gv->SPC_m2_srcregion_R2,"\n";} 
	    if ($gv->SPC_m2_srcregion eq 'box' )
	    {print MYOUTFILE "         MOS2 SRC BOX     : ",$gv->SPC_m2_srcregion_XLength," - ",$gv->SPC_m2_srcregion_YLength,"\n";}
	    print MYOUTFILE "         MOS2 Region BKG  : ",$gv->SPC_m2_bkgregion,"\n";
	    print MYOUTFILE "         MOS2 BKG XC      : ",$gv->SPC_m2_bkgregion_X,"\n";
	    print MYOUTFILE "         MOS2 BKG YC      : ",$gv->SPC_m2_bkgregion_Y,"\n";
	    if ($gv->SPC_m2_bkgregion eq 'circle' || $gv->SPC_m2_bkgregion eq 'annulus')
	    {
		print MYOUTFILE "         MOS2 BKG RC      : ",$gv->SPC_m2_bkgregion_R1," - ",$gv->SPC_m2_bkgregion_R2,"\n";
		if ($gv->SPC_m2_srcregion_R2 > $gv->SPC_m2_bkgregion_R1) {
		    print MYOUTFILE "         WARNING: MOS2 regions : The inner background region radii is smaller than the source region radii \n";
		} 
	    } 
	    if ($gv->SPC_m2_bkgregion eq 'box' )	
	    {print MYOUTFILE "         MOS2 BKG BOX     : ",$gv->SPC_m2_bkgregion_XLength," - ",$gv->SPC_m2_bkgregion_YLength,"\n";}
	    print MYOUTFILE "         MOS2 BKG AREA SCALE FACTOR : ",$gv->SPC_m2_areafactor," (Only area taken into account)\n";
	    if ($gv->SPC_m2_srcregion_R2 == 6000){print MYOUTFILE "         WARNING : MOS2 regions : Maximum allowed radius reached";}
	}
    }

#.. Prepare spectrum log file and output information
    if ($Log_Spec_Flag == 0) {
	my $spectrum_log=$results_directory."spectra_".$source_name."\.log";
	if (-e $spectrum_log) {unlink("$spectrum_log") or die "Cannot rm file: $!";}    
	open(SPECTRALOG, "> $spectrum_log") or die "Cant open output file: $! \n";    #open for write, overwrite
	print SPECTRALOG "   \#> Centroid of source to analyze (Physical Units) \n";
	print SPECTRALOG "       PN \n";
	print SPECTRALOG "         SRC Region : ",$gv->SPC_pn_srcregion,"\n";
	print SPECTRALOG "         SRC XC     : ",$gv->SPC_pn_srcregion_X," \n";
	print SPECTRALOG "         SRC YC     : ",$gv->SPC_pn_srcregion_Y," \n";
	if ($gv->SPC_pn_srcregion eq "circle" || $gv->SPC_pn_srcregion eq "annulus")
	{print SPECTRALOG "         SRC RADII  : ",$gv->SPC_pn_srcregion_R1," - ",$gv->SPC_pn_srcregion_R2," \n";}
	if ($gv->SPC_pn_srcregion eq "box") 
	{print SPECTRALOG "         SRC BOX    : ",$gv->SPC_pn_srcregion_XLength," - ",$gv->SPC_pn_srcregion_YLength," \n";}
	print SPECTRALOG "         BKG Region : ",$gv->SPC_pn_bkgregion,"\n";
	print SPECTRALOG "         BKG XC     : ",$gv->SPC_pn_bkgregion_X," \n";
	print SPECTRALOG "         BKG YC     : ",$gv->SPC_pn_bkgregion_Y," \n";
	if ($gv->SPC_pn_srcregion eq "circle" || $gv->SPC_pn_srcregion eq "annulus") 
	{
	    print SPECTRALOG "         BKG RADII  : ",$gv->SPC_pn_bkgregion_R1," - ",$gv->SPC_pn_bkgregion_R2," \n";
	    if ($gv->SPC_pn_srcregion_R2 > $gv->SPC_pn_bkgregion_R1)
	    {print SPECTRALOG "         WARNING: PN regions : The inner background region radii is smaller than the source region radii \n";} 	    
	}
	if ($gv->SPC_pn_srcregion eq "box") 
	{print SPECTRALOG "         BKG BOX    : ",$gv->SPC_pn_bkgregion_XLength," - ",$gv->SPC_pn_bkgregion_YLength," \n";}
	print SPECTRALOG "         BKG AREA SCALE FACTOR : ",$gv->SPC_pn_areafactor," (Only area taken into account)\n";
	if ($gv->SPC_pn_srcregion_R2 == 6000){print SPECTRALOG  "         WARNING : PN regions : Maximum allowed radius reached";}
	print SPECTRALOG "       MOS1 \n";
	print SPECTRALOG "         SRC Region : ",$gv->SPC_m1_srcregion,"\n";
	print SPECTRALOG "         SRC XC     : ",$gv->SPC_m1_srcregion_X," \n";
	print SPECTRALOG "         SRC YC     : ",$gv->SPC_m1_srcregion_Y," \n";
	if ($gv->SPC_m1_srcregion eq "circle" || $gv->SPC_m1_srcregion eq "annulus")
	{print SPECTRALOG "         SRC RADII  : ",$gv->SPC_m1_srcregion_R1," - ",$gv->SPC_m1_srcregion_R2," \n";}
	if ($gv->SPC_m1_srcregion eq "box") 
	{print SPECTRALOG "         SRC BOX    : ", $gv->SPC_m1_srcregion_XLength," - ",$gv->SPC_m1_srcregion_YLength," \n";}
	print SPECTRALOG "         BKG Region : ",$gv->SPC_m1_bkgregion,"\n";
	print SPECTRALOG "         BKG XC     : ",$gv->SPC_m1_bkgregion_X," \n";
	print SPECTRALOG "         BKG YC     : ",$gv->SPC_m1_bkgregion_Y," \n";
	if ($gv->SPC_m1_srcregion eq "circle" || $gv->SPC_m1_srcregion eq "annulus") 
	{
	    print SPECTRALOG "         BKG RADII  : ",$gv->SPC_m1_bkgregion_R1," - ",$gv->SPC_m1_bkgregion_R2," \n";
	    if ($gv->SPC_m1_srcregion_R2 > $gv->SPC_m1_bkgregion_R1)	
	    {print SPECTRALOG "         WARNING: MOS1 regions : The inner background region radii is smaller than the source region radii \n";}
	}
	if ($gv->SPC_m1_srcregion eq "box") 
	{print SPECTRALOG "         BKG BOX    : ",$gv->SPC_m1_bkgregion_XLength," - ",$gv->SPC_m1_bkgregion_YLength," \n";}
	print SPECTRALOG "         BKG AREA SCALE FACTOR : ",$gv->SPC_m1_areafactor," (Only area taken into account)\n";
	if ($gv->SPC_m1_srcregion_R2 == 6000){print SPECTRALOG  "         WARNING : MOS1 regions : Maximum allowed radius reached";}
	print SPECTRALOG "       MOS2 \n";
	print SPECTRALOG "         SRC Region : ",$gv->SPC_m2_srcregion,"\n";
	print SPECTRALOG "         SRC XC     : ",$gv->SPC_m2_srcregion_X," \n";
	print SPECTRALOG "         SRC YC     : ",$gv->SPC_m2_srcregion_Y," \n";
	if ($gv->SPC_m2_srcregion eq "circle" || $gv->SPC_m2_srcregion eq "annulus")
	{print SPECTRALOG "         SRC RADII  : ",$gv->SPC_m2_srcregion_R1," - ",$gv->SPC_m2_srcregion_R2," \n";}
	if ($gv->SPC_m2_srcregion eq "box") 
	{print SPECTRALOG "         SRC BOX    : ", $gv->SPC_m2_srcregion_XLength," - ",$gv->SPC_m2_srcregion_YLength," \n";}
	print SPECTRALOG "         BKG Region : ",$gv->SPC_m2_bkgregion,"\n";
	print SPECTRALOG "         BKG XC     : ",$gv->SPC_m2_bkgregion_X," \n";
	print SPECTRALOG "         BKG YC     : ",$gv->SPC_m2_bkgregion_Y," \n";
	if ($gv->SPC_m2_srcregion eq "circle" || $gv->SPC_m2_srcregion eq "annulus") 
	{
	    print SPECTRALOG "         BKG RADII  : ",$gv->SPC_m2_bkgregion_R1," - ",$gv->SPC_m2_bkgregion_R2," \n";
	    if ($gv->SPC_m2_srcregion_R2 > $gv->SPC_m2_bkgregion_R1) 
	    {print SPECTRALOG "         WARNING: MOS2 regions : The inner background region radii is smaller than the source region radii \n";}
	}
	if ($gv->SPC_m2_srcregion eq "box") 
	{print SPECTRALOG "         BKG BOX    : ",$gv->SPC_m2_bkgregion_XLength," - ",$gv->SPC_m2_bkgregion_YLength," \n";}
  	print SPECTRALOG "         BKG AREA SCALE FACTOR : ",$gv->SPC_m2_areafactor," (Only area taken into account)\n";	
	if ($gv->SPC_m2_srcregion_R2 == 6000){print SPECTRALOG  "         WARNING : MOS2 regions : Maximum allowed radius reached";}
    }
    
    if (!$pn_spectral_flag && !$m1_spectral_flag && !$m2_spectral_flag) {
	print "   \#> No information to produce spectra in any camera .... exiting \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> No information to produce spectra in any camera .... exiting \n";}
	if ($Log_Spec_Flag == 0) {print SPECTRALOG "   \#> No information to produce spectra in any camera .... exiting \n";}
	if ($Log_Spec_Flag == 0) {close(SPECTRALOG);}
	
	&error_code(0); 
	return(0);
    }

    chdir($spectra_directory);
    
#.. Make image plots and create region files used to extract the spectrum    
    print "\n   \#> Producing plots and region files ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Producing plots and region files ..... \n";}

    if (-e $pn_spectral_region_file) {unlink("$pn_spectral_region_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m1_spectral_region_file) {unlink("$m1_spectral_region_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m2_spectral_region_file) {unlink("$m2_spectral_region_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $pn_spectral_bkgregion_file) {unlink("$pn_spectral_bkgregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m1_spectral_bkgregion_file) {unlink("$m1_spectral_bkgregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m2_spectral_bkgregion_file) {unlink("$m2_spectral_bkgregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $pn_spectral_combinedregion_file) {unlink("$pn_spectral_combinedregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m1_spectral_combinedregion_file) {unlink("$m1_spectral_combinedregion_file") or die "Cannot rm file $temp_file: $!";}
    if (-e $m2_spectral_combinedregion_file) {unlink("$m2_spectral_combinedregion_file") or die "Cannot rm file $temp_file: $!";}

#.. SRC

    if ($gv->SPC_pn_srcregion eq "circle" || $gv->SPC_pn_srcregion eq "annulus") 
    {$x= $gv->SPC_pn_srcregion_X; $y= $gv->SPC_pn_srcregion_Y; $L1=$gv->SPC_pn_srcregion_R1; $L2 =$gv->SPC_pn_srcregion_R2;}
    if ($gv->SPC_pn_srcregion eq "box") 
    {$x= $gv->SPC_pn_srcregion_X; $y= $gv->SPC_pn_srcregion_Y; $L1=$gv->SPC_pn_srcregion_XLength; $L2 =$gv->SPC_pn_srcregion_YLength;}
    if ($pn_spectral_flag) {&addsourcetoregionfile($pn_spectral_region_file,$x,$y,$L1,$L2,"yellow","",$gv->SPC_pn_srcregion,"physical");} # PN
    if ($pn_spectral_flag) {&addsourcetoregionfile($pn_spectral_combinedregion_file,$x,$y,$L1,$L2,"yellow","",$gv->SPC_pn_srcregion,"physical");} # PN

    if ($gv->SPC_m1_srcregion eq "circle" || $gv->SPC_m1_srcregion eq "annulus") 
    {$x= $gv->SPC_m1_srcregion_X; $y= $gv->SPC_m1_srcregion_Y; $L1=$gv->SPC_m1_srcregion_R1; $L2 =$gv->SPC_m1_srcregion_R2;}
    if ($gv->SPC_m1_srcregion eq "box") 
    {$x= $gv->SPC_m1_srcregion_X; $y= $gv->SPC_m1_srcregion_Y; $L1=$gv->SPC_m1_srcregion_XLength; $L2 =$gv->SPC_m1_srcregion_YLength;}
    if ($m1_spectral_flag) {&addsourcetoregionfile($m1_spectral_region_file,$x,$y,$L1,$L2,"yellow","",$gv->SPC_m1_srcregion,"physical");} # MOS1 
    if ($m1_spectral_flag) {&addsourcetoregionfile($m1_spectral_combinedregion_file,$x,$y,$L1,$L2,"yellow","",$gv->SPC_m1_srcregion,"physical");} # MOS1 

    if ($gv->SPC_m2_srcregion eq "circle" || $gv->SPC_m2_srcregion eq "annulus") 
    {$x= $gv->SPC_m2_srcregion_X; $y= $gv->SPC_m2_srcregion_Y; $L1=$gv->SPC_m2_srcregion_R1; $L2 =$gv->SPC_m2_srcregion_R2;}
    if ($gv->SPC_m2_srcregion eq "box") 
    {$x= $gv->SPC_m2_srcregion_X; $y= $gv->SPC_m2_srcregion_Y; $L1=$gv->SPC_m2_srcregion_XLength; $L2 =$gv->SPC_m2_srcregion_YLength;}
    if ($m2_spectral_flag) {&addsourcetoregionfile($m2_spectral_region_file,$x,$y,$L1,$L2,"yellow","",$gv->SPC_m2_srcregion,"physical");} # MOS2
    if ($m2_spectral_flag) {&addsourcetoregionfile($m2_spectral_combinedregion_file,$x,$y,$L1,$L2,"yellow","",$gv->SPC_m2_srcregion,"physical");} # MOS2

#.. BKG

    if ($gv->SPC_pn_bkgregion eq "circle" || $gv->SPC_pn_bkgregion eq "annulus") 
    {$x= $gv->SPC_pn_bkgregion_X; $y= $gv->SPC_pn_bkgregion_Y; $L1=$gv->SPC_pn_bkgregion_R1; $L2 =$gv->SPC_pn_bkgregion_R2;}
    if ($gv->SPC_pn_bkgregion eq "box") 
    {$x= $gv->SPC_pn_bkgregion_X; $y= $gv->SPC_pn_bkgregion_Y; $L1=$gv->SPC_pn_bkgregion_XLength; $L2 =$gv->SPC_pn_bkgregion_YLength;}
    if ($pn_spectral_flag) {&addsourcetoregionfile($pn_spectral_bkgregion_file,$x,$y,$L1,$L2,"red","",$gv->SPC_pn_bkgregion,"physical");} # PN
    if ($pn_spectral_flag) {&addsourcetoregionfile($pn_spectral_combinedregion_file,$x,$y,$L1,$L2,"red","",$gv->SPC_pn_bkgregion,"physical");} # PN

    if ($gv->SPC_m1_bkgregion eq "circle" || $gv->SPC_m1_bkgregion eq "annulus") 
    {$x= $gv->SPC_m1_bkgregion_X; $y= $gv->SPC_m1_bkgregion_Y; $L1=$gv->SPC_m1_bkgregion_R1; $L2 =$gv->SPC_m1_bkgregion_R2;}
    if ($gv->SPC_m1_bkgregion eq "box") 
    {$x= $gv->SPC_m1_bkgregion_X; $y= $gv->SPC_m1_bkgregion_Y; $L1=$gv->SPC_m1_bkgregion_XLength; $L2 =$gv->SPC_m1_bkgregion_YLength;}
    if ($m1_spectral_flag) {&addsourcetoregionfile($m1_spectral_bkgregion_file,$x,$y,$L1,$L2,"red","",$gv->SPC_m1_bkgregion,"physical");} # MOS1   
    if ($m1_spectral_flag) {&addsourcetoregionfile($m1_spectral_combinedregion_file,$x,$y,$L1,$L2,"red","",$gv->SPC_m1_bkgregion,"physical");} # MOS1   

    if ($gv->SPC_m2_bkgregion eq "circle" || $gv->SPC_m2_bkgregion eq "annulus") 
    {$x= $gv->SPC_m2_bkgregion_X; $y= $gv->SPC_m2_bkgregion_Y; $L1=$gv->SPC_m2_bkgregion_R1; $L2 =$gv->SPC_m2_bkgregion_R2;}
    if ($gv->SPC_m2_bkgregion eq "box") 
    {$x= $gv->SPC_m2_bkgregion_X; $y= $gv->SPC_m2_bkgregion_Y; $L1=$gv->SPC_m2_bkgregion_XLength; $L2 =$gv->SPC_m2_bkgregion_YLength;}
    if ($m2_spectral_flag) {&addsourcetoregionfile($m2_spectral_bkgregion_file,$x,$y,$L1,$L2,"red","",$gv->SPC_m2_bkgregion,"physical");} # MOS2
    if ($m2_spectral_flag) {&addsourcetoregionfile($m2_spectral_combinedregion_file,$x,$y,$L1,$L2,"red","",$gv->SPC_m2_bkgregion,"physical");} # MOS2

#.. PS Plots

    if ($pn_spectral_flag) {&produce_ps_file($pn_gti_smoothimage_file,$pn_spectral_combinedregion_file,"PN",$pn_spectrum_psfile);}   # PN
    if ($m1_spectral_flag) {&produce_ps_file($m1_gti_smoothimage_file,$m1_spectral_combinedregion_file,"MOS1",$m1_spectrum_psfile);} # MOS1
    if ($m2_spectral_flag) {&produce_ps_file($m2_gti_smoothimage_file,$m2_spectral_combinedregion_file,"MOS2",$m2_spectrum_psfile);} # MOS2

#.. run epatplot
	if ($gv->ANA_epatplot)
	{
    	if ($pn_spectral_flag) {&run_epatplot($pn_gti_cleanevents_file,"pn","SPC",$pn_obsmode);}   # PN
	    if ($m1_spectral_flag) {&run_epatplot($m1_gti_cleanevents_file,"m1","SPC",$m1_obsmode);} # MOS1
    	if ($m2_spectral_flag) {&run_epatplot($m2_gti_cleanevents_file,"m2","SPC",$m2_obsmode);} # MOS2
    	# Set the flag to false. If run_all has been selected, produce_lightcurve will not run againg epatplot. 
    	$gv->ANA_epatplot(0);
	}

#.. Obtain spectrum files for PN and MOS: source, bkg, arf and rmf files 
    print "   \#> Producing source/background spectrum files and arf/rmf files (especget) ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Producing source/background spectrum files and arf/rmf files (especget) ..... \n";}
    if ($Log_Spec_Flag == 0) {print SPECTRALOG "   \#> Producing source/background spectrum files and arf/rmf files (especget) ..... \n";}
    
    if ($pn_spectral_flag) {  
	$SRC_XC    = $gv->SPC_pn_srcregion_X;
	$SRC_YC    = $gv->SPC_pn_srcregion_Y;
	if ($gv->SPC_pn_srcregion eq "circle" || $gv->SPC_pn_srcregion eq "annulus") {$RSRC1     = $gv->SPC_pn_srcregion_R1};
	if ($gv->SPC_pn_srcregion eq "circle" || $gv->SPC_pn_srcregion eq "annulus") {$RSRC2     = $gv->SPC_pn_srcregion_R2};
	if ($gv->SPC_pn_srcregion eq "box") {$RSRC1     = $gv->SPC_pn_srcregion_XLength};
	if ($gv->SPC_pn_srcregion eq "box") {$RSRC2     = $gv->SPC_pn_srcregion_YLength};
	$BKG_XC    = $gv->SPC_pn_bkgregion_X;
	$BKG_YC    = $gv->SPC_pn_bkgregion_Y;
	if ($gv->SPC_pn_bkgregion eq "circle" || $gv->SPC_pn_bkgregion eq "annulus") {$RBKG1     = $gv->SPC_pn_bkgregion_R1};
	if ($gv->SPC_pn_bkgregion eq "circle" || $gv->SPC_pn_bkgregion eq "annulus") {$RBKG2     = $gv->SPC_pn_bkgregion_R2};
	if ($gv->SPC_pn_bkgregion eq "box") {$RBKG1     = $gv->SPC_pn_bkgregion_XLength};
	if ($gv->SPC_pn_bkgregion eq "box") {$RBKG2     = $gv->SPC_pn_bkgregion_YLength};
	if (!&test_pnspetralfiles()){
	    &run_especget($pn_gti_cleanevents_file,"pn",$SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$gv->SPC_pn_srcregion,$BKG_XC,$BKG_YC,$RBKG1,$RBKG2,$gv->SPC_pn_bkgregion);  # PN
	} else {
	    print "\n   \#> PN spectral files found ! \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> PN spectral files found ! \n";}
	}
	($pnsrc_livetime,$pnsrc_backscale,$pnsrc_areascal)=&event_file_spectralinf($pn_sourcebkgspectrum_file,"pn");   # PN
	($pnbkg_livetime,$pnbkg_backscale,$pnbkg_areascal)=&event_file_spectralinf($pn_bkgspectrum_file,"pn");         # PN
	if ($pnsrc_livetime eq "unknown" or $pnsrc_backscale eq "unknown" or $pnsrc_areascal eq "unknown"){&error_code(6);} 
	if ($pnbkg_livetime eq "unknown" or $pnbkg_backscale eq "unknown" or $pnbkg_areascal eq "unknown"){&error_code(6);}     
	$gv->SPC_pn_areafactor($pnsrc_backscale/$pnbkg_backscale);
    } else {
	print "\n   \#> No PN Spectra can be worked out \n";
	if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> No PN Spectra can be worked out \n";}	
    }
    
    if ($m1_spectral_flag) {  
	$SRC_XC    = $gv->SPC_m1_srcregion_X;
	$SRC_YC    = $gv->SPC_m1_srcregion_Y;
	if ($gv->SPC_m1_srcregion eq "circle" || $gv->SPC_m1_srcregion eq "annulus") {$RSRC1     = $gv->SPC_m1_srcregion_R1};
	if ($gv->SPC_m1_srcregion eq "circle" || $gv->SPC_m1_srcregion eq "annulus") {$RSRC2     = $gv->SPC_m1_srcregion_R2};
	if ($gv->SPC_m1_srcregion eq "box") {$RSRC1     = $gv->SPC_m1_srcregion_XLength};
	if ($gv->SPC_m1_srcregion eq "box") {$RSRC2     = $gv->SPC_m1_srcregion_YLength};
	$BKG_XC    = $gv->SPC_m1_bkgregion_X;
	$BKG_YC    = $gv->SPC_m1_bkgregion_Y;
	if ($gv->SPC_m1_bkgregion eq "circle" || $gv->SPC_m1_bkgregion eq "annulus") {$RBKG1     = $gv->SPC_m1_bkgregion_R1};
	if ($gv->SPC_m1_bkgregion eq "circle" || $gv->SPC_m1_bkgregion eq "annulus") {$RBKG2     = $gv->SPC_m1_bkgregion_R2};
	if ($gv->SPC_m1_bkgregion eq "box") {$RBKG1     = $gv->SPC_m1_bkgregion_XLength};
	if ($gv->SPC_m1_bkgregion eq "box") {$RBKG2     = $gv->SPC_m1_bkgregion_YLength};
	if (!&test_mos1spetralfiles()){
	    &run_especget($m1_gti_cleanevents_file,"mos1",$SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$gv->SPC_m1_srcregion,$BKG_XC,$BKG_YC,$RBKG1,$RBKG2,$gv->SPC_m1_bkgregion); # MOS1
	} else {
	    print "\n   \#> MOS1 spectral files found ! \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> MOS1 spectral files found ! \n";}
	}	    
	($m1src_livetime,$m1src_backscale,$m1src_areascal)=&event_file_spectralinf($m1_sourcebkgspectrum_file,"mos1");  # MOS1
	($m1bkg_livetime,$m1bkg_backscale,$m1bkg_areascal)=&event_file_spectralinf($m1_bkgspectrum_file,"mos1");        # MOS1
	if ($m1src_livetime eq "unknown" or $m1src_backscale eq "unknown" or $m1src_areascal eq "unknown"){&error_code(6);} 
	if ($m1bkg_livetime eq "unknown" or $m1bkg_backscale eq "unknown" or $m1bkg_areascal eq "unknown"){&error_code(6);} 
	$gv->SPC_m1_areafactor($m1src_backscale/$m1bkg_backscale);
    } else {
	print "\n   \#> No MOS1 Spectra can be worked out \n";
	if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> No MOS1 Spectra can be worked out \n";}	
    }
    
    if ($m2_spectral_flag) { 
	$SRC_XC    = $gv->SPC_m2_srcregion_X;
	$SRC_YC    = $gv->SPC_m2_srcregion_Y;
	if ($gv->SPC_m2_srcregion eq "circle" || $gv->SPC_m2_srcregion eq "annulus") {$RSRC1     = $gv->SPC_m2_srcregion_R1};
	if ($gv->SPC_m2_srcregion eq "circle" || $gv->SPC_m2_srcregion eq "annulus") {$RSRC2     = $gv->SPC_m2_srcregion_R2};
	if ($gv->SPC_m2_srcregion eq "box") {$RSRC1     = $gv->SPC_m2_srcregion_XLength};
	if ($gv->SPC_m2_srcregion eq "box") {$RSRC2     = $gv->SPC_m2_srcregion_YLength};
	$BKG_XC    = $gv->SPC_m2_bkgregion_X;
	$BKG_YC    = $gv->SPC_m2_bkgregion_Y;
	if ($gv->SPC_m2_bkgregion eq "circle" || $gv->SPC_m2_bkgregion eq "annulus") {$RBKG1     = $gv->SPC_m2_bkgregion_R1};
	if ($gv->SPC_m2_bkgregion eq "circle" || $gv->SPC_m2_bkgregion eq "annulus") {$RBKG2     = $gv->SPC_m2_bkgregion_R2};
	if ($gv->SPC_m2_bkgregion eq "box") {$RBKG1     = $gv->SPC_m2_bkgregion_XLength};
	if ($gv->SPC_m2_bkgregion eq "box") {$RBKG2     = $gv->SPC_m2_bkgregion_YLength};
	if (!&test_mos2spetralfiles()){
	    &run_especget($m2_gti_cleanevents_file,"mos2",$SRC_XC,$SRC_YC,$RSRC1,$RSRC2,$gv->SPC_m2_srcregion,$BKG_XC,$BKG_YC,$RBKG1,$RBKG2,$gv->SPC_m2_bkgregion); # MOS2
	} else {
	    print "\n   \#> MOS2 spectral files found ! \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> MOS2 spectral files found ! \n";}
	}
	($m2src_livetime,$m2src_backscale,$m2src_areascal)=&event_file_spectralinf($m2_sourcebkgspectrum_file,"mos2");   # MOS2
	($m2bkg_livetime,$m2bkg_backscale,$m2bkg_areascal)=&event_file_spectralinf($m2_bkgspectrum_file,"mos2");         # MOS2
	if ($m2src_livetime eq "unknown" or $m2src_backscale eq "unknown" or $m2src_areascal eq "unknown"){&error_code(6);} 
	if ($m2bkg_livetime eq "unknown" or $m2bkg_backscale eq "unknown" or $m2bkg_areascal eq "unknown"){&error_code(6);} 
	$gv->SPC_m2_areafactor($m2src_backscale/$m2bkg_backscale);
    } else {
	print "\n   \#> No MOS2 Spectra can be worked out \n";
	if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> No MOS2 Spectra can be worked out \n";}	
    }
        
    
#.. Write Summary
    
    print "\n   \#> Spectral Analysis Summary :\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Spectral Analysis Summary :\n";}
    if ($Log_Spec_Flag == 0) {print SPECTRALOG "\n   \#> Spectral Analysis Summary :\n";}    
    if (!$pn_spectral_flag){$pn_summ_ana_msg = "FAILED";} else {$pn_summ_ana_msg = "OK";} 
    if (!$m1_spectral_flag){$m1_summ_ana_msg = "FAILED";} else {$m1_summ_ana_msg = "OK";} 
    if (!$m2_spectral_flag){$m2_summ_ana_msg = "FAILED";} else {$m2_summ_ana_msg = "OK";} 
    print "      pn : analysis : $pn_summ_ana_msg \n";
    if ($Log_Flag == 0) {print MYOUTFILE "      pn : analysis : $pn_summ_ana_msg \n";}   
    if ($Log_Spec_Flag == 0) {print SPECTRALOG "      pn : analysis : $pn_summ_ana_msg \n";}  
   
    print "      mos1 : analysis : $m1_summ_ana_msg \n";
    
    if ($Log_Flag == 0) {print MYOUTFILE "      mos1 : analysis : $m1_summ_ana_msg \n";}   
    if ($Log_Spec_Flag == 0) {print SPECTRALOG "      mos1 : analysis : $m1_summ_ana_msg \n";}  

    print "      mos2 : analysis : $m2_summ_ana_msg \n";
    
    if ($Log_Flag == 0) {print MYOUTFILE "      mos2 : analysis : $m2_summ_ana_msg \n";}
    if ($Log_Spec_Flag == 0) {print SPECTRALOG "      mos2 : analysis : $m2_summ_ana_msg \n";}  
      
    if ($Log_Spec_Flag == 0) {close(SPECTRALOG);}

    if ($temp_Log_Lcur_Flag == 0) {$Log_Lcur_Flag = 0;} # Re-activate LCurve Log flag if active

    &error_code(0);
    return(0);
}


## @method run_eregionanalyse($image_file,$r,$d,$instr,$submode,$eregionanal_file,$analysis)
# Run the SAS task eregionanalyse (Find optimum source extraction region). The source region is a circle and the background region is an annulus centered on the source region, whose inner radius is a given distance away from the source radius.
# @param image_file Image File Name
# @param r Source RA
# @param d Source DEC
# @param instr Instrument
# @param submode Observation Mode
# @param eregionanal_file File for output depending on instrument
# @param analysis Which analysis routine is calling eregionanalyse (LC,SPC,PG)
# @return Source X_Centroid, Source Y_Centroid, Optimum Source Radius (sky coord), Optimum Source Radius (image coord), Background Inner Radius (Sky coord), Background Inner Radius (Image coord), Background Outer Radius (Sky coord), Background Outer Radius (Image coord)

#========================================================================
sub run_eregionanalyse(){

    my $image_file         = $_[0];
    my $r                  = $_[1];
    my $d                  = $_[2];
    my $instr              = $_[3];
    my $submode            = $_[4];
    my $eregionanal_file   = $_[5]; # Select file for output depending on instrument
    my $analysis           = $_[6]; # Which analysis routine is calling eregionanalyse

    print "\n   \#> Running eregionanalyse over $image_file .... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Running eregionanalyse over $image_file .... \n";}

#.. Work out Annulus that will define the BKG Region
    if ($instr eq "pn") { 
	if ($analysis eq "LC") {$areafactor          = $gv->LC_pn_areafactor;}     # How much bigger the BKG area should be with respect to the on-source Area
	if ($analysis eq "SPC") {$areafactor         = $gv->SPC_pn_areafactor;}    
	if ($analysis eq "PG") {$areafactor          = 1.0;} 
    }
    if ($instr eq "mos1") { 
	if ($analysis eq "LC") {$areafactor          = $gv->LC_m1_areafactor;}     # How much bigger the BKG area should be with respect to the on-source Area
	if ($analysis eq "SPC") {$areafactor         = $gv->SPC_m1_areafactor;}    
	if ($analysis eq "PG") {$areafactor          = 1.0;} 
    }
    if ($instr eq "mos2") { 
	if ($analysis eq "LC") {$areafactor          = $gv->LC_m2_areafactor;}     # How much bigger the BKG area should be with respect to the on-source Area
	if ($analysis eq "SPC") {$areafactor         = $gv->SPC_m2_areafactor;}         
	if ($analysis eq "PG") {$areafactor          = 1.0;} 
    }

#.. NOTE: 0.001666667deg = 6arcsecs (EPIC-pn PSF which the worse case scenario (EPIC-MOS = 5arcsec)) 
#..       The area covered by the bkg region is chosen to cover the same area as that of the source region

    my $sky2img         = 1./0.05;                                  # 1 sky pixel = 0.05arcsec

#    my $src_radius      = 0.001666667;                             # Initial value (= EPIC-pn PSF)    
    my $src_radius      = 0.005555555556;                           # (5 June 2008) After talking to Richard, this initial region 
                                                                    # is used to work out the centroid. So it should be bigger than 6",
                                                                    # 20" is a good starting point.   
    my $bkg_rad_factor  = 1;                                        # How far from the src region is the background region
    my $bkg_innerradius = 0.01666667;                               # Initial value (Should be around 60arcsec, the values used in the pipeline)

    my $bkg_outerradius = sqrt($bkg_innerradius**2+$areafactor*$src_radius**2); # Initial value

#.. Return Values    
    my $x = 1;
    my $y = 1;
    my $xc = 1;
    my $yc = 1;
    my $ropt    = 1;
    my $roptsky = 1;
    my $bkg_innerrsky = 0;
    my $bkg_outerrsky = 0.;
    my $bkg_innerrimg = 0.;
    my $bkg_outerrimg = 0.;

    $src_expr = "((RA,DEC) in circle($r,$d,$src_radius))";
    $bkg_expr = "((RA,DEC) in annulus($r,$d,$bkg_innerradius,$bkg_outerradius))";
    if (($instr eq "mos1" || $instr eq "mos2") && $submode eq "PrimePartialW2"){
	print "       WARNING: $instr Entering Small Window Settings.\n";
	if ($Log_Flag == 0) {print MYOUTFILE "       WARNING: $instr Entering Small Window Settings. \n";}
	$rad1 = 0.;
	$rad2 = sqrt($areafactor)*3600.*$src_radius*$sky2img; 
	$xc   = 17821.0 + $rad2; # in CCD 4
	$yc   = 33726.0 + $rad2; # in CCD 4
	$bkg_expr = "((X,Y) in annulus($xc,$yc,$rad1,$rad2))";
    } 

    @args = ("eregionanalyse imageset=\"$image_file\" srcexp=\"$src_expr\" backexp=\"$bkg_expr\" -V 4 > $eregionanal_file"); 
    system(@args) == 0 or &error_code(6);
   
    if (-e $eregionanal_file){
	
	open(EREGION, "$eregionanal_file") or die "Cant open output file: $! \n";; #open for write, overwrite
	print "   \#> INFORMATION OUT OF eregionanalyse ($eregionanal_file) :\n"; 
	print "\t $instr : Run on location: RA: $r - DEC: $d \n";

	if ($Log_Flag == 0) {print MYOUTFILE "   \#> INFORMATION OUT OF eregionanalyse ($eregionanal_file) :\n";}
	while(<EREGION>) {
	    print MYOUTFILE "\t $_";
	    next if /^eregionanalyse/;
	    next if /^(\s)*$/;
	    print "\t $instr : $_";
	    if (/xcentroid/) {
		@split = split/:/,$_;
		chomp($x     = $split[1]);
		$x =~ s/\s+//;
	    }
	    if (/ycentroid/) {
		@split = split/:/,$_;
		chomp($y     = $split[1]);
		$y =~ s/\s+//;
	    }
	    if (/optradius/) {
		($roptsky,$roptimg) = (split/\s+/,$_)[2,4];
		$roptsky =~ s/\s+//;
		$roptimg =~ s/\s+//;
	    }
	}
	close(EREGION);
    }
    
#.. Background center region
    $xb = $x;
    $yb = $y;
     if (($instr eq "mos1" || $instr eq "mos2") && $submode eq "PrimePartialW2"){
	$xb              = $xc; 
	$yb              = $yc;
	$bkg_innerradius = 0.;
    }

#.. Source Region Size
    if  (($instr eq "mos1" || $instr eq "mos2") && $submode eq "PrimePartialW2"){
	if ($roptsky > 40.0){
	    $roptsky = 40.0;   # arcsec
	    $roptimg = 800.0;  # image 
	    print "       WARNING: $instr Radius of Source Region greater than 40 arsec. Set to 40 arsec.\n";
	    if ($Log_Flag == 0) {print MYOUTFILE "       WARNING: $instr Radius of Source Region greater than 40 arsec. Set to 40 arsec.\n";}
	}	
    }

#.. In sky coordinates
    $bkg_innerrsky = 3600.*$bkg_innerradius;                               # Inner radius of BKG region in sky coordinates (arcsec)
    $bkg_outerrsky = sqrt($areafactor*$roptsky**2+$bkg_innerrsky**2);

#.. In image coordinates
    $bkg_innerrimg = 3600.*$bkg_innerradius*$sky2img;                      # Inner radius of BKG region in imag coordinates (arcsec)
    $bkg_outerrimg = sqrt($areafactor*$roptimg**2+$bkg_innerrimg**2);


    return($x,$y,$roptsky,$roptimg,$xb,$yb,$bkg_innerrsky,$bkg_innerrimg,$bkg_outerrsky,$bkg_outerrimg);
}



## @method void run_especget($event_file,$instr,$src_x,$src_y,$src_L1,$src_L2,$src_rshape,$bkg_x,$bkg_y,$bkg_L1,$bkg_L2,$bkg_rshape)
# Run the SAS task specget (produce EPIC spectral products)
# @param event_file Event Fele Name
# @param instr Instrument
# @param src_x Source X_coordinate (Image Coordinates)
# @param src_y Source Y_coordinate (Image Coordinates)
# @param src_L1 Source Region Inner Radius (circle,annulus), Source Region X_Box Size (box) (Image Coordinates)
# @param src_L2 Source Region Outer Radius (circle,annulus), Source Region Y_Box Size (box) (Image Coordinates)
# @param src_rshape Source Region Shape
# @param bkg_x Background X_coordinate (Image Coordinates)
# @param bkg_y Background Y_coordinate (Image Coordinates)
# @param bkg_L1 Background Region Inner Radius (circle,annulus), Background Region X_Box Size (box) (Image Coordinates)
# @param bkg_L2 Background Region Outer Radius (circle,annulus), Background Region Y_Box Size (box) (Image Coordinates)
# @param bkg_rshape Background Region Shape
# @return Void

#========================================================================
sub run_especget(){

    my $event_file = $_[0];
    my $instr      = $_[1];
    my $src_x      = $_[2];
    my $src_y      = $_[3];
    my $src_L1     = $_[4];
    my $src_L2     = $_[5];
    my $src_rshape = $_[6];
    my $bkg_x      = $_[7];
    my $bkg_y      = $_[8];
    my $bkg_L1     = $_[9];
    my $bkg_L2     = $_[10];
    my $bkg_rshape = $_[11];
    
    my $src_box_angle = 0.;
    my $bkg_box_angle = 0.;


# Note: especget cant take a '+' in the name of the file, so the '+' coming from the source name has to be replaced by a '_'.
    if ($instr eq "pn"){ 
	$pn_ancillary_file             =~ s/\+/\_/;
	$srcarfset  = $pn_ancillary_file;
	$pn_redistribution_matrix_file =~ s/\+/\_/;
	$srcrmfset  = $pn_redistribution_matrix_file;
	$pn_sourcebkgspectrum_file     =~ s/\+/\_/;
	$srcspecset = $pn_sourcebkgspectrum_file;
	$pn_bkgspectrum_file           =~ s/\+/\_/;
	$bckspecset = $pn_bkgspectrum_file;
    }
    
    if ($instr eq "mos1"){ 
	$m1_ancillary_file              =~ s/\+/\_/;
	$srcarfset  = $m1_ancillary_file;
	$m1_redistribution_matrix_file  =~ s/\+/\_/;
	$srcrmfset  = $m1_redistribution_matrix_file;
	$m1_sourcebkgspectrum_file      =~ s/\+/\_/;
	$srcspecset = $m1_sourcebkgspectrum_file;
	$m1_bkgspectrum_file            =~ s/\+/\_/;
	$bckspecset = $m1_bkgspectrum_file;
    }
    
    if ($instr eq "mos2"){ 
	$m2_ancillary_file               =~ s/\+/\_/;
	$srcarfset  = $m2_ancillary_file;
	$m2_redistribution_matrix_file   =~ s/\+/\_/;
	$srcrmfset  = $m2_redistribution_matrix_file;
	$m2_sourcebkgspectrum_file       =~ s/\+/\_/;
	$srcspecset = $m2_sourcebkgspectrum_file;
	$m2_bkgspectrum_file             =~ s/\+/\_/;
	$bckspecset = $m2_bkgspectrum_file;
    }
    
    my $mode = $obsid_instruments{$gv->SRC_OBSID}{$instr}{"mode"};
    if ($mode eq "IMAGING" || $mode eq "imaging") { 
	$xval = "X"; 
	$yval = "Y"; 
    } elsif (($mode eq "TIMING" || $mode eq "timing")
	     && ($instr eq "pn")) {
	$xval = "RAWX"; 
	$yval = "RAWY"; 
    } elsif (($mode eq "TIMING" || $mode eq "timing")
	&& ($instr eq "mos1" || $instr eq "mos2")) { 
	$xval = "RAWX"; 
	$yval = "TIME"; 
    } else {
	print "ERROR \n";
	if ($Log_Flag == 0) {print MYOUTFILE "ERROR \n";}
	return(0);
    }
    chdir($spectra_directory);
 
    if ($src_rshape eq "circle"){$src_rshape = "annulus";} # treat circle regions as annulus with R1=0
    if ($bkg_rshape eq "circle"){$bkg_rshape = "annulus";} # treat circle regions as annulus with R1=0

    if ($src_rshape eq "annulus" && $bkg_rshape eq "annulus") {
	@args = ("especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset");
    }
    
    if ($src_rshape eq "box" && $bkg_rshape eq "annulus") {
	$src_L1     = $src_L1/2.;
	$src_L2     = $src_L2/2.;
	@args = ("especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2,$src_box_angle))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset");
    }
 
    if ($src_rshape eq "annulus" && $bkg_rshape eq "box") {
	$bkg_L1     = $bkg_L1/2.;
	$bkg_L2     = $bkg_L2/2.;
	@args = ("especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2,$bkg_box_angle))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset");
    }

    if (($src_rshape eq "box") && ($bkg_rshape eq "box")  &&  ( ($instr eq "mos1") || ($instr eq "mos2") ) && (uc($mode) eq "TIMING")     ) {
	$src_L1     = $src_L1/2.;
	$src_L2     = $src_L2/2.;
	$bkg_L1     = $bkg_L1/2.;
	$bkg_L2     = $bkg_L2/2.;
	my $SX_min = $src_x-10;
	my $SX_max = $src_x+10;
	my $BX_min = $bkg_x-10;
	my $BX_max = $bkg_x+10;
	@args = ("especget -t table=\"$event_file\" srcexp=\"(($xval) in [$SX_min:$SX_max])\" backexp=\"(($xval) in [$BX_min:$BX_max])\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset");
    }
    else
    {
	$src_L1     = $src_L1/2.;
	$src_L2     = $src_L2/2.;
	$bkg_L1     = $bkg_L1/2.;
	$bkg_L2     = $bkg_L2/2.;
	@args = ("especget -t table=\"$event_file\" srcexp=\"(($xval,$yval) in $src_rshape($src_x,$src_y,$src_L1,$src_L2,$src_box_angle))\" backexp=\"(($xval,$yval) in $bkg_rshape($bkg_x,$bkg_y,$bkg_L1,$bkg_L2,$bkg_box_angle))\" withbadpixcorr=yes srcarfset=$srcarfset srcrmfset=$srcrmfset srcspecset=$srcspecset bckspecset=$bckspecset");
    }

   
    system(@args) == 0 or &error_code(6);

    print "   \#> $instr : Specget Summary:  \n";
    print "        SRC FILE  : $srcspecset \n";
    print "        BKG FILE  : $bckspecset \n";
    print "        RESP FILE : $srcrmfset \n";
    print "        ANCR FILE : $srcarfset \n";	
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> $instr : Specget Summary:  \n";
	print MYOUTFILE "        SRC FILE  : $srcspecset \n";
	print MYOUTFILE "        BKG FILE  : $bckspecset \n";
	print MYOUTFILE "        RESP FILE : $srcrmfset \n";
	print MYOUTFILE "        ANCR FILE : $srcarfset \n";
    }
    if ($Log_Spec_Flag == 0) {
	print SPECTRALOG "   \#> $instr : Specget Summary:  \n";
	print SPECTRALOG "        SRC FILE  : $srcspecset \n";
	print SPECTRALOG "        BKG FILE  : $bckspecset \n";
	print SPECTRALOG "        RESP FILE : $srcrmfset \n";
	print SPECTRALOG "        ANCR FILE : $srcarfset \n";
    }
    return(1);
}



## @method void test_pnspetralfiles()
# Test for the existance of pn spectral files
# @return Void

#========================================================================
sub test_pnspetralfiles{
# Specget does not take file names with '+' on them. They are changed by '_'.
    chdir ($spectra_directory);
    $pn_ancillary_file =~ s/\+/\_/;
    if (!-e $pn_ancillary_file) {print "PN Ancillery file does not exist \n";
				 return(0);} 
    $pn_redistribution_matrix_file =~ s/\+/\_/;
    if (!-e $pn_redistribution_matrix_file) {print "PN Redistribution Matrix file does not exist \n";
					     return(0);} 
    $pn_sourcebkgspectrum_file =~ s/\+/\_/;
    if (!-e $pn_sourcebkgspectrum_file) {print "PN Source Spectrum file does not exist \n";
					 return(0);}    
    $pn_bkgspectrum_file =~ s/\+/\_/; 
    if (!-e $pn_bkgspectrum_file) {print "PN Background Spectrum file does not exist \n";
				   return(0);}
    return(1);
}

## @method void test_mos1spetralfiles()
# Test for the existance of MOS1 spectral files
# @return Void

#========================================================================
sub test_mos1spetralfiles{
# Specget does not take file names with '+' on them. They are changed by '_'.
    chdir ($spectra_directory);
    $m1_ancillary_file =~ s/\+/\_/; 
    if (!-e $m1_ancillary_file) {print "MOS1 Ancillery file does not exist \n";
				 return(0);}               	  
    $m1_redistribution_matrix_file =~ s/\+/\_/; 
    if (!-e $m1_redistribution_matrix_file) {print "MOS1 Redistribution Matrix file does not exist \n";
					     return(0);} 	  
    $m1_sourcebkgspectrum_file =~ s/\+/\_/; 
    if (!-e $m1_sourcebkgspectrum_file) {print "MOS1 Source Spectrum file does not exist \n";
					 return(0);}          	  
    $m1_bkgspectrum_file =~ s/\+/\_/; 
    if (!-e $m1_bkgspectrum_file) {print "MOS1 Background Spectrum file does not exist \n";
				   return(0);}
    return(1);
}

## @method void test_mos2spetralfiles()
# Test for the existance of MOS2 spectral files
# @return Void

#========================================================================
sub test_mos2spetralfiles{
# Specget does not take file names with '+' on them. They are changed by '_'.
    chdir ($spectra_directory);
    $m2_ancillary_file =~ s/\+/\_/; 
    if (!-e $m2_ancillary_file) {print "MOS2 Ancillery file does not exist \n";
				 return(0);}               	  
    $m2_redistribution_matrix_file =~ s/\+/\_/; 
    if (!-e $m2_redistribution_matrix_file) {print "MOS2 Redistribution Matrix file does not exist \n";
					     return(0);} 	  
    $m2_sourcebkgspectrum_file =~ s/\+/\_/; 
    if (!-e $m2_sourcebkgspectrum_file) {print "MOS2 Source Spectrum file does not exist \n";
					 return(0);} 
    $m2_bkgspectrum_file =~ s/\+/\_/; 
    if (!-e $m2_bkgspectrum_file) {print "MOS2 Background Spectrum file does not exist \n";
				   return(0);}
    return(1);
}
1;
