## @file
# Run the SAS tasks cifbuild and odfingest

## @method void run_cifbuild()
# Runs the SAS tasks cifbuild and odfingest
# @return Void

#========================================================================
sub run_cifbuild(){

    chdir ($Analysis_directory);

#.. Run cifbuild
    print "\n----> Running cifbuild ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Running cifbuild ..... \n";}

#.. Print sas_setup
    &print_sas_setup();

#.. Test for ccf file
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if ($test_ccf[0]) {             # If ccf file already exist ...

	print "   \#> ccf File has already been produced ..... \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> ccf File has already been produced ..... \n";}	
	&set_ccf($ccffile);
	print "   \#> ccf File: $ccffile \n";
	print "   \#> cifbuild OK \n";
	if ($Log_Flag == 0) {
	    print MYOUTFILE "   \#> ccf File: $ccffile \n";
	    print MYOUTFILE "   \#> cifbuild OK \n";
	}


    } else {                       # If ccf file does not exist ...

	@args = ("cifbuild","withccfpath=yes","ccfpath=$ENV{'SAS_CCFPATH'}","fullpath=yes","withobservationdate=yes"); 
	system(@args) == 0 or &error_code(2);

#.. Test for ccf file. 	
	@test_ccf=&test_ccf();
	my $ccffile=$test_ccf[1];
	if ($test_ccf[0]) {

	    &set_ccf($ccffile);
	    print "   \#> ccf File: $ccffile \n";
	    print "   \#> cifbuild OK \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> ccf File: $ccffile \n";
		print MYOUTFILE "   \#> cifbuild OK \n";
	    }
	   	   

	} else {

	    print "   \#> File $ccffile does not exist .... Exiting \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> File $ccffile does not exist .... Exiting \n" && &error_code(2);}
	
	}

    }


#.. Run odfingest
    print "\n----> Running odfingest .....\n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Running odfingest ..... \n";}

#.. Test for odfingest file first and delete if it exist, i.e., ALWAYS run odfingest
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if ($test_odfingest[0]){system("rm","-f","$odfingestfile") == 0 or &error_code(2);}

#.. Run odfingest
    &run_odfingest();

#.. Test for odfingest file.     
    @test_odfingest=&test_odfingest();
    $odfingestfile=$test_odfingest[1];
    if ($test_odfingest[0]) { 

	print "   \#> Summary File: $odfingestfile \n"; 
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> Summary File: $odfingestfile \n";}
	&set_odfingest($odfingestfile);
	print "   \#> odfingest OK \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> odfingest OK \n";}

    } else {
	
	print "   \#> odfingest file does not exist .... Exiting \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> odfingest file does not exist .... Exiting \n" && &error_code(2)};
	
    }
    &error_code(0) && return;
}

## @method void run_odfingest()
# Runs the SAS tasks odfingest
# @return Void

#========================================================================
sub run_odfingest(){
    @args = ("odfingest");
    system(@args) == 0 or &error_code(2);
    return(1);
}

## @method test_ccf()
# Test for the existance of a CCF file
# @return File Name

#========================================================================
sub test_ccf{
    print "   #> Checking existance of CCF file ... "; 
    chdir ($Analysis_directory);
    my $ccffile = $Analysis_directory."ccf.cif";
    if (-e $ccffile) {    
	print "OK\n"; 
	return(1,$ccffile);
    }
    print "DOES NOT EXIST ... Running ... \n"; 
    return(0,"No File");
}

## @method void set_ccf()
# Sets the environment variable SAS_CCF
# @return Void

#========================================================================
sub set_ccf{
    $ENV{'SAS_CCF'} = $_[0];
    print "   \#> SAS_CCF: $ENV{'SAS_CCF'} \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> SAS_CCF: $ENV{'SAS_CCF'} \n";}
    return(1);
}

## @method test_odfingest()
# Test for the existance of a SUM.SAS file
# @return File Name

#========================================================================
sub test_odfingest{ 
    print "   #> Checking existance of SUM.SAS file ... "; 
    chdir ($Analysis_directory);
    @SAS_list         = glob('*SUM.SAS');
    if ($#SAS_list == 0) {                    
	my $odfingestfile = $Analysis_directory.$SAS_list[0];
	print "OK\n";
	return (1,$odfingestfile);
    }     
    print "DOES NOT EXIST ... Running ... \n"; 
    return (0,"No File");
}

## @method void set_odfingest()
# Sets the environment variable SAS_ODF
# @return Void

#========================================================================
sub set_odfingest{
    $ENV{'SAS_ODF'} = $_[0];
    print "   \#> NEW SAS_ODF: $ENV{'SAS_ODF'} \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> NEW SAS_ODF: $ENV{'SAS_ODF'} \n";}
    return(1);
}
1;
