## @file
# Run the SAS task edetect_chain on EPIC event files

## @method void run_edetectchain()
# Runs the SAS task edetect_chain and cross-correlates the user source position with the EPIC source list.
# @return Void

#======================================================================== 
sub run_edetectchain(){ print "\n----> Running edetect_chain ..... \n"; if
($Log_Flag == 0) {print MYOUTFILE "\n----> Running edetect_chain ....\n";}

#.. Define Global Variables
    print "   \#> Defining File Names for edetect chain ..... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Defining File Names for edetect chain  ..... ";}
 
    use vars qw($pn_event_list
		$m1_event_list
		$m2_event_list
		$pn_edetect_flag
		$m1_edetect_flag
		$m2_edetect_flag
		$pn_GTI_file
		$m1_GTI_file
		$m2_GTI_file
		$m1_fullimage
		$m1_emllist_file         
		$m1_eboxllist_file 
		$m1_eboxmlist_file
		$source_m1region_file
		$m2_fullimage
		$m2_emllist_file 
		$m2_eboxllist_file 
		$m2_eboxmlist_file
		$source_m2region_file
		$pn_fullimage 
		$pn_emllist_file
		$pn_eboxllist_file 
		$pn_eboxmlist_file
		$source_pnregion_file	       
		);

    my $source_obsID = $gv->SRC_OBSID;
    my $source_name  = $gv->SRC_Name;
    my $source_ra    = $gv->SRC_RA;
    my $source_dec   = $gv->SRC_DEC;

    print "OK \n";
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}

#.. Definition of file names
    $pn_gti_cleanevents_file         = $gti_directory."pn_events_gtifiltered.fit";
    $m1_gti_cleanevents_file         = $gti_directory."m1_events_gtifiltered.fit";  		       	  
    $m2_gti_cleanevents_file         = $gti_directory."m2_events_gtifiltered.fit";  		          
    $m1_fullimage      = $images_directory."m1_image_full\.fits";   
    $m1_emllist_file   = $images_directory."m1_emllist\.fits";
    $m1_eboxllist_file = $images_directory."m1_eboxlist_l\.fits";
    $m1_eboxmlist_file = $images_directory."m1_eboxlist_m\.fits";   
    $m2_fullimage      = $images_directory."m2_image_full\.fits";    
    $m2_emllist_file   = $images_directory."m2_emllist\.fits";
    $m2_eboxllist_file = $images_directory."m2_eboxlist_l\.fits";
    $m2_eboxmlist_file = $images_directory."m2_eboxlist_m\.fits"; 
    $pn_fullimage      = $images_directory."pn_image_full\.fits";   
    $pn_emllist_file   = $images_directory."pn_emllist\.fits";
    $pn_eboxllist_file = $images_directory."pn_eboxlist_l\.fits";
    $pn_eboxmlist_file = $images_directory."pn_eboxlist_m\.fits";    

    $source_pnregion_file   = $images_directory.$source_name."_pnregion\.reg";    
    $source_m1region_file   = $images_directory.$source_name."_m1region\.reg";
    $source_m2region_file   = $images_directory.$source_name."_m2region\.reg";
    
    $pn_edetect_flag                = 1; # Flag that determines whether the spectra should be worked out (1: True)
    $m1_edetect_flag                = 1; # Flag that determines whether the spectra should be worked out (1: True)
    $m2_edetect_flag                = 1; # Flag that determines whether the spectra should be worked out (1: True)

# Check if the user select MOS1, MOS2, PN for analysing data
	$pn_edetect_flag = $gv->PN;
	$m1_edetect_flag = $gv->MOS;	
	$m2_edetect_flag = $gv->MOS;

#.. Energy Ranges and detection likelihoods for PN and MOS

    @pn_ene_low  = $gv->EDC_pn_ene_low; 
    @pn_ene_high = $gv->EDC_pn_ene_high; 
    @m1_ene_low  = $gv->EDC_m1_ene_low;  
    @m1_ene_high = $gv->EDC_m1_ene_high; 
    @m2_ene_low  = $gv->EDC_m2_ene_low;   
    @m2_ene_high = $gv->EDC_m2_ene_high;  
    $pn_likelihood_detection  = $gv->EDC_pn_likelihood_detection; 
    $mos_likelihood_detection = $gv->EDC_mos_likelihood_detection;
		
#.. Test for ccf file and set ENV variable
    @test_ccf=&test_ccf();
    my $ccffile=$test_ccf[1];
    if (!$test_ccf[0]) {print "      Run cifbuild first \n"; &error_code(5);}
    &set_ccf($ccffile);

#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(5);}
    &set_odfingest($odfingestfile);
    
#.. Print sas_setup    
    &print_sas_setup();

#.. Test for existance of GTI files
    if ($Log_GTIEvt_Flag == 0) {
	my $gtievent_log=$results_directory."gtievent_".$source_name."\.log";
	if (-e $gtievent_log) {unlink("$gtievent_log") or die "Cannot rm file: $!";}    
	open(GTILOG, "> $gtievent_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

    @pnGTI=&test_pngtifile();
    if ($pnGTI[0]){
	@pninfoGTI=&gti_file_inf($pn_GTI_file,"pn");
	if ($pninfoGTI[0] != 0.0){$pn_GTI_file=$pnGTI[1];} else {$pn_edetect_flag = 0;}
    } else {$pn_edetect_flag = 0;}   
 
    @m1GTI=&test_mos1gtifile();
    if ($m1GTI[0]){
	@m1infoGTI=&gti_file_inf($m1_GTI_file,"mos1");
        if ($m1infoGTI[0] != 0.0) {$m1_GTI_file=$m1GTI[1];} else {$m1_edetect_flag = 0;} 
    } else {$m1_edetect_flag = 0;} 

    @m2GTI=&test_mos2gtifile();
    if ($m2GTI[0]){
	@m2infoGTI=&gti_file_inf($m2_GTI_file,"mos2");
	if ($m2infoGTI[0] != 0.0) {$m2_GTI_file=$m2GTI[1];} else {$m2_edetect_flag = 0;}
    } else {$m2_edetect_flag = 0;}     

    if ($Log_GTIEvt_Flag == 0) {
	close(GTILOG);
	$Log_GTIEvt_Flag = 1;	
    }

#.. Test for existance of event files    
    if ($Log_Evt_Flag == 0) {
	my $event_log=$results_directory."event_".$source_name."\.log";
	if (-e $event_log) {unlink("$event_log") or die "Cannot rm file: $!";}    
	open(EVTLOG, "> $event_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

    @test_pnevt=&test_pneventfiles();
    $pn_event_list=$test_pnevt[1];
    if ($test_pnevt[0]) {$pn_event_list=$test_pnevt[1];} else {$pn_edetect_flag = 0;}
    if ($pn_edetect_flag eq 1) {@pninfo=&event_file_inf($pn_event_list,"pn");}
 
    @test_m1evt=&test_mos1eventfiles();
    $m1_event_list=$test_m1evt[1];
    if ($test_m1evt[0]) {$m1_event_list=$test_m1evt[1];} else {$m1_edetect_flag = 0;}
    if ($m1_edetect_flag eq 1) {@m1info=&event_file_inf($m1_event_list,"mos1");}
    
    @test_m2evt=&test_mos2eventfiles();
    $m2_event_list=$test_m2evt[1];
    if ($test_m2evt[0]) {$m2_event_list=$test_m2evt[1];} else {$m2_edetect_flag = 0;}
    if ($m2_edetect_flag eq 1) {@m2info=&event_file_inf($m2_event_list,"mos2");}

    if ($Log_Evt_Flag == 0) {
	close(EVTLOG);
	$Log_Evt_Flag = 1;	
    }

#.. Get instrument's observing mode and prepare things according to shape of source and background regions.  
    $pn_obsmode = $obsid_instruments{$source_obsID}{'pn'}{'mode'};
    $m1_obsmode = $obsid_instruments{$source_obsID}{'mos1'}{'mode'};
    $m2_obsmode = $obsid_instruments{$source_obsID}{'mos2'}{'mode'};
    $pn_obssubmode    = $obsid_instruments{$source_obsID}{'pn'}{'submode'};
    $m1_obssubmode    = $obsid_instruments{$source_obsID}{'mos1'}{'submode'};
    $m2_obssubmode    = $obsid_instruments{$source_obsID}{'mos2'}{'submode'};
    
#.. Create GTI filtered event lists.
    if ($pn_edetect_flag ) 
    {&produce_gticleanfile($pn_event_list,$pn_gti_cleanevents_file,$pn_GTI_file,$pn_obsmode,"pn");}  # PN 
    if ($m1_edetect_flag) 
    {&produce_gticleanfile($m1_event_list,$m1_gti_cleanevents_file,$m1_GTI_file,$m1_obsmode,"mos");} # MOS1
    if ($m2_edetect_flag) 
    {&produce_gticleanfile($m2_event_list,$m2_gti_cleanevents_file,$m2_GTI_file,$m2_obsmode,"mos");} # MOS2

#.. Prepare edetect chain log file    
    chdir($images_directory);

    if ($Log_Edet_Flag == 0) {
	my $edetect_log=$results_directory."edetect_chain_".$source_name."\.log";
	if (-e $edetect_log) {unlink("$edetect_log") or die "Cannot rm file: $!";}    
	open(EDETECTLOG, "> $edetect_log") or die "Cant open output file: $! \n";    #open for write, overwrite
    }

#.. Test if I have to run edetect chain or simply produce images.    
    
    print "\n----> Checking if edetect_chain needs to be ran ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Checking if edetect_chain needs to be ran ....\n";}
      
    if (!-e $pn_fullimage && $pn_edetect_flag eq 1) {&pn_create_energy_bands();}
    if (!-e $pn_emllist_file && $pn_edetect_flag eq 1) {&pn_edetect_chain();} 
    if (!-e $m1_fullimage && $m1_edetect_flag eq 1) {&m1_create_energy_bands();}
    if (!-e $m1_emllist_file && $m1_edetect_flag eq 1) {&m1_edetect_chain();}
    if (!-e $m2_fullimage && $m2_edetect_flag eq 1) {&m2_create_energy_bands();}
    if (!-e $m2_emllist_file && $m2_edetect_flag eq 1) {&m2_edetect_chain();}
#..
#.. Produce Images 
#..
    print "\n----> Producing Instruments Images ..... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n----> Producing Instruments Images ....\n";}
    
#.. For PN
    @pn_closesource = (0.,0.,1000.);
    if ($pn_edetect_flag eq 1){
	if ($pninfo[2] eq "IMAGING") {
	    if (-e $pn_emllist_file) {                                  #.. Has our the source been detected in PN ?
		@pn_closesource=&pn_image();
		if ($pn_closesource[2] <= $gv->EDC_sourcematch){$pn_edetect_flag = 1;}else{$pn_edetect_flag = 0;}   
	    } else {
		print "   \#> No Sources detected in PN within ",join(", ", $gv->EDC_sourcematch)," deg \n";
		if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in PN within",join(", ", $gv->EDC_sourcematch)," deg \n";}
		$pn_edetect_flag = 0;
	    }
	} else {
	    print "   \#> Data is not IMAGING mode. Ignoring PN \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Data is not IMAGING mode. Ignoring PN \n"};
	    $pn_edetect_flag = 0;
	}
    } else {
	print "   \#> No Sources detected in PN \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in PN \n";}
	$pn_edetect_flag = 0;
    }

#.. For MOS1
    @m1_closesource = (0.,0.,1000.);
    if ($m1_edetect_flag eq 1){
	if ($m1info[2] eq "IMAGING") {
	    if (-e $m1_emllist_file) {              #.. Has our the source been detected in MOS1 ?
		@m1_closesource=&m1_image();
		if ($m1_closesource[2] <= $gv->EDC_sourcematch){$m1_edetect_flag = 1;}else{$m1_edetect_flag = 0;}  
	    } else {
		print "   \#> No Sources detected in MOS1 within ",join(", ", $gv->EDC_sourcematch)," deg \n";
		if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in MOS1 within",join(", ", $gv->EDC_sourcematch),"deg \n";}
		$m1_edetect_flag = 0;
	    }
	} else {
	    print "   \#> Data is not IMAGING mode. Ignoring MOS1 \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Data is not IMAGING mode. Ignoring MOS1 \n"};
	    $m1_edetect_flag = 0;
	} 
    } else {
	print "   \#> No Sources detected in MOS1 \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in MOS1 \n";}
	$m1_edetect_flag = 0;
    }

#.. For MOS2
    @m2_closesource = (0.,0.,1000.);
    if ($m2_edetect_flag eq 1){
	if ($m2info[2] eq "IMAGING") {
	    if (-e $m2_emllist_file) {               #.. Has our the source been detected in MOS2 ?
		@m2_closesource=&m2_image();
		if ($m2_closesource[2] <= $gv->EDC_sourcematch){$m2_edetect_flag = 1;}else{$m2_edetect_flag = 0;}  
	    } else {
		print "   \#> No Sources detected in MOS2 within ",join(", ", $gv->EDC_sourcematch)," deg \n";
		if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in MOS2 within",join(", ", $gv->EDC_sourcematch)," deg \n";}
		$m2_edetect_flag = 0;
	    }
	} else {
	    print "   \#> Data is not IMAGING mode. Ignoring MOS2 \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Data is not IMAGING mode. Ignoring MOS2 \n"};
	    $m2_edetect_flag = 0;
	} 
    } else {
	print "   \#> No Sources detected in MOS2 \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> No Sources detected in MOS2 \n";}
	$m2_edetect_flag = 0;
    }

#.. Is the source detected in the different instruments the same ?
    my $pnmos1det     = "NO";
    my $pnmos1_dist   = 1000.;
    my $pnmos2det     = "NO";
    my $pnmos2_dist   = 1000.;
    my $mos1mos2det   = "NO";
    my $mos1mos2_dist   = 1000.;

    if ($pn_edetect_flag eq 1 && $m1_edetect_flag eq 1){
	$pnmos1_dist = &Angular_separation($pn_closesource[0],$m1_closesource[0],$pn_closesource[1],$m1_closesource[1]);
 	if ($pnmos1_dist <= $gv->EDC_cameramatch()) {$pnmos1det = "YES";} 
    }
    if ($pn_edetect_flag eq 1 && $m2_edetect_flag eq 1){
	$pnmos2_dist = &Angular_separation($pn_closesource[0],$m2_closesource[0],$pn_closesource[1],$m2_closesource[1]);
	if ($pnmos2_dist <= $gv->EDC_cameramatch()) {$pnmos2det = "YES";} 
    }
    if ($m1_edetect_flag eq 1 && $m2_edetect_flag eq 1){
	$mos1mos2_dist = &Angular_separation($m1_closesource[0],$m2_closesource[0],$m1_closesource[1],$m2_closesource[1]);
	if ($mos1mos2_dist <= $gv->EDC_cameramatch()) {$mos1mos2det = "YES";} 
    }

#.. Get count rate information if source has been detected

    if (-e $pn_emllist_file && $pn_edetect_flag eq 1) 
    {@pnsrcrate=&get_source_count_rate($pn_emllist_file,"pn",0,$pn_closesource[0],$pn_closesource[1]);} 
    if (-e $pn_emllist_file && $pn_edetect_flag eq 1) 
    {@pnbkgrate=&get_bkg_count_rate($pn_emllist_file,"pn",$pn_closesource[0],$pn_closesource[1]);} 
    if (-e $m1_emllist_file && $m1_edetect_flag eq 1) 
    {@m1srcrate=&get_source_count_rate($m1_emllist_file,"mos1",0,$m1_closesource[0],$m1_closesource[1]);}
    if (-e $m1_emllist_file && $m1_edetect_flag eq 1) 
    {@m1bkgrate=&get_bkg_count_rate($m1_emllist_file,"mos1",$m1_closesource[0],$m1_closesource[1]);}
    if (-e $m2_emllist_file && $m2_edetect_flag eq 1) 
    {@m2srcrate=&get_source_count_rate($m2_emllist_file,"mos2",0,$m2_closesource[0],$m2_closesource[1]);}
    if (-e $m2_emllist_file&& $m2_edetect_flag eq 1) 
    {@m2bkgrate=&get_bkg_count_rate($m2_emllist_file,"mos2",$m2_closesource[0],$m2_closesource[1]);}

#.. Get Source count rate information if source has been detected in different energy bands
    print "\n   \#> Count Rate Information: \n";
    if ($Log_Edet_Flag == 0){print EDETECTLOG "\n   \#> Count Rate Information: \n";}
    if ($Log_Flag == 0){print MYOUTFILE "\n   \#> Count Rate Information: \n";}

    if (-e $pn_emllist_file && $pn_edetect_flag eq 1){
	@pnsrcrate_E1=&get_source_count_rate($pn_emllist_file,"pn",1,$pn_closesource[0],$pn_closesource[1]);
	@pnsrcrate_E2=&get_source_count_rate($pn_emllist_file,"pn",2,$pn_closesource[0],$pn_closesource[1]);
	@pnsrcrate_E3=&get_source_count_rate($pn_emllist_file,"pn",3,$pn_closesource[0],$pn_closesource[1]);
	@pnsrcrate_E4=&get_source_count_rate($pn_emllist_file,"pn",4,$pn_closesource[0],$pn_closesource[1]);
	@pnsrcrate_E5=&get_source_count_rate($pn_emllist_file,"pn",5,$pn_closesource[0],$pn_closesource[1]);
    } 
    if (-e $m1_emllist_file && $m1_edetect_flag eq 1){
	@m1srcrate_E1=&get_source_count_rate($m1_emllist_file,"mos1",1,$m1_closesource[0],$m1_closesource[1]);
	@m1srcrate_E2=&get_source_count_rate($m1_emllist_file,"mos1",2,$m1_closesource[0],$m1_closesource[1]);
	@m1srcrate_E3=&get_source_count_rate($m1_emllist_file,"mos1",3,$m1_closesource[0],$m1_closesource[1]);
	@m1srcrate_E4=&get_source_count_rate($m1_emllist_file,"mos1",4,$m1_closesource[0],$m1_closesource[1]);
	@m1srcrate_E5=&get_source_count_rate($m1_emllist_file,"mos1",5,$m1_closesource[0],$m1_closesource[1]);
    }
    if (-e $m2_emllist_file && $m2_edetect_flag eq 1){ 
	@m2srcrate_E1=&get_source_count_rate($m2_emllist_file,"mos2",1,$m2_closesource[0],$m2_closesource[1]);
	@m2srcrate_E2=&get_source_count_rate($m2_emllist_file,"mos2",2,$m2_closesource[0],$m2_closesource[1]);
	@m2srcrate_E3=&get_source_count_rate($m2_emllist_file,"mos2",3,$m2_closesource[0],$m2_closesource[1]);
	@m2srcrate_E4=&get_source_count_rate($m2_emllist_file,"mos2",4,$m2_closesource[0],$m2_closesource[1]);
	@m2srcrate_E5=&get_source_count_rate($m2_emllist_file,"mos2",5,$m2_closesource[0],$m2_closesource[1]);
    }

#.. Output rate information onto different log files  
    print "       Source IDs (from emllist.fits table): \n";
    if ($pn_edetect_flag eq 1) {print "        PN SRC ID  : $pnsrcrate[0] \n";} else {print "        No positive PN identification\n";}
    if ($m1_edetect_flag eq 1) {print "        MOS1 SRC ID: $m1srcrate[0] \n";} else {print "        No positive MOS1 identification\n";}
    if ($m2_edetect_flag eq 1) {print "        MOS2 SRC ID: $m2srcrate[0] \n";} else {print "        No positive MOS2 identification\n";}
    print "       CCD IDs (from emllist.fits table): \n";
    if ($pn_edetect_flag eq 1) {print "        PN CCD ID  : $pnsrcrate[3] \n";} else {print "        No positive PN identification\n";}
    if ($m1_edetect_flag eq 1) {print "        MOS1 CCD ID: $m1srcrate[3] \n";} else {print "        No positive MOS1 identification\n";}
    if ($m2_edetect_flag eq 1) {print "        MOS2 CCD ID: $m2srcrate[3] \n";} else {print "        No positive MOS2 identification\n";}
    print "       Source and BKG Count Rates in entire energy range considered: \n";
    if ($pn_edetect_flag eq 1) {print "        PN  : \n";} else {print "        No positive PN identification\n";}
    if ($pn_edetect_flag eq 1) {print "          SRC Rate (cts/s): $pnsrcrate[1] +/- $pnsrcrate[2] \n";}
    if ($pn_edetect_flag eq 1) {print "          BKG Rate (cts/s): $pnbkgrate[1] +/- $pnbkgrate[2] \n";}
    if ($m1_edetect_flag eq 1) {print "        MOS1: \n";} else {print "        No positive MOS1 identification\n";}
    if ($m1_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m1srcrate[1] +/- $m1srcrate[2] \n";}
    if ($m1_edetect_flag eq 1) {print "          BKG Rate (cts/s): $m1bkgrate[1] +/- $m1bkgrate[2] \n";}
    if ($m2_edetect_flag eq 1) {print "        MOS2: \n";} else {print "        No positive MOS2 identification\n";}
    if ($m2_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m2srcrate[1] +/- $m2srcrate[2] \n";}
    if ($m2_edetect_flag eq 1) {print "          BKG Rate (cts/s): $m2bkgrate[1] +/- $m2bkgrate[2] \n";}
    print "       Source Count Rates in different energy ranges considered: \n";
    if ($pn_edetect_flag eq 1) {print "        PN  : \n";} else {print "        No positive PN identification\n";}
    if ($pn_edetect_flag eq 1) {print "          SRC Rate (cts/s): $pnsrcrate_E1[1] +/- $pnsrcrate_E1[2] ($pn_ene_low[0] - $pn_ene_high[0]) \n";}
    if ($pn_edetect_flag eq 1) {print "          SRC Rate (cts/s): $pnsrcrate_E2[1] +/- $pnsrcrate_E2[2] ($pn_ene_low[1] - $pn_ene_high[1]) \n";}
    if ($pn_edetect_flag eq 1) {print "          SRC Rate (cts/s): $pnsrcrate_E3[1] +/- $pnsrcrate_E3[2] ($pn_ene_low[2] - $pn_ene_high[2]) \n";}
    if ($pn_edetect_flag eq 1) {print "          SRC Rate (cts/s): $pnsrcrate_E4[1] +/- $pnsrcrate_E4[2] ($pn_ene_low[3] - $pn_ene_high[3]) \n";}
    if ($pn_edetect_flag eq 1) {print "          SRC Rate (cts/s): $pnsrcrate_E5[1] +/- $pnsrcrate_E5[2] ($pn_ene_low[4] - $pn_ene_high[4]) \n";}
    if ($m1_edetect_flag eq 1) {print "        MOS1: \n";} else {print "        No positive MOS1 identification\n";}
    if ($m1_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m1srcrate_E1[1] +/- $m1srcrate_E1[2] ($m1_ene_low[0] - $m1_ene_high[0]) \n";}
    if ($m1_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m1srcrate_E2[1] +/- $m1srcrate_E2[2] ($m1_ene_low[1] - $m1_ene_high[1]) \n";}
    if ($m1_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m1srcrate_E3[1] +/- $m1srcrate_E3[2] ($m1_ene_low[2] - $m1_ene_high[2]) \n";}
    if ($m1_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m1srcrate_E4[1] +/- $m1srcrate_E4[2] ($m1_ene_low[3] - $m1_ene_high[3]) \n";}
    if ($m1_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m1srcrate_E5[1] +/- $m1srcrate_E5[2] ($m1_ene_low[4] - $m1_ene_high[4]) \n";}
    if ($m2_edetect_flag eq 1) {print "        MOS2: \n";} else {print "        No positive MOS2 identification\n";}
    if ($m2_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m2srcrate_E1[1] +/- $m2srcrate_E1[2] ($m2_ene_low[0] - $m2_ene_high[0]) \n";}
    if ($m2_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m2srcrate_E2[1] +/- $m2srcrate_E2[2] ($m2_ene_low[1] - $m2_ene_high[1]) \n";}
    if ($m2_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m2srcrate_E3[1] +/- $m2srcrate_E3[2] ($m2_ene_low[2] - $m2_ene_high[2]) \n";}
    if ($m2_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m2srcrate_E4[1] +/- $m2srcrate_E4[2] ($m2_ene_low[3] - $m2_ene_high[3]) \n";}
    if ($m2_edetect_flag eq 1) {print "          SRC Rate (cts/s): $m2srcrate_E5[1] +/- $m2srcrate_E5[2] ($m2_ene_low[4] - $m2_ene_high[4]) \n";}
    print "\n   \#> Cross Correlation Information for EPIC: \n";
    print "       SOURCE           PN               MOS1             MOS2   \n";
    print "  RA       DEC     RA       DEC      RA       DEC    RA        DEC \n";
    print " $source_ra $source_dec $pn_closesource[0] $pn_closesource[1] $m1_closesource[0] $m1_closesource[1] $m2_closesource[0] $m2_closesource[1] \n";
    print "  Distance to closest source in each instrument: \n";
    print "  (If RA,DEC of the source is within ", join(", ", $gv->EDC_sourcematch)," deg of RA,DEC of detected \n";
    print "  camera source, it is assumed the source is the same) \n";
    print "    SOURCE-PN : $pn_closesource[2] deg. Same source ?: $pn_edetect_flag \n";
    print "    SOURCE-M1 : $m1_closesource[2] deg. Same source ?: $m1_edetect_flag \n";
    print "    SOURCE-M2 : $m2_closesource[2] deg. Same source ?: $m2_edetect_flag \n";
    print "  Comparing Source detection on different cameras: \n";
    print "  (If RA,DEC of one camera is within ", join(", ", $gv->EDC_cameramatch)," deg of RA,DEC of another camera \n";
    print "  it is assumed the source in both cameras is the same) \n";
    print "    Distance PN-MOS1    : $pnmos1_dist deg. Same source ?: $pnmos1det \n";
    print "    Distance PN-MOS2    : $pnmos2_dist deg. Same source ?: $pnmos2det \n";
    print "    Distance MOS1-MOS2  : $mos1mos2_dist deg. Same source ?: $mos1mos2det \n";
    if ($Log_Edet_Flag == 0) {
	print EDETECTLOG "       Source IDs (from emllist.fits table): \n";
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "        PN SRC ID  : $pnsrcrate[0] \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "        MOS1 SRC ID: $m1srcrate[0] \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "        MOS2 SRC ID: $m2srcrate[0] \n";}
	print EDETECTLOG "       CCD IDs (from emllist.fits table): \n";
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "        PN CCD ID   : $pnsrcrate[3] \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "        MOS1 CCD ID : $m1srcrate[3] \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "        MOS2 CCD ID : $m2srcrate[3] \n";}
	print EDETECTLOG "       Source and BKG Count Rates in entire energy range considered: \n";
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "        PN  : \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          PN SRC Rate (cts/s): $pnsrcrate[1] +/- $pnsrcrate[2] \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          PN BKG Rate (cts/s): $pnbkgrate[1] +/- $pnbkgrate[2] \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "        MOS1: \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          MOS1 SRC Rate (cts/s): $m1srcrate[1] +/- $m1srcrate[2] \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          MOS1 BKG Rate (cts/s): $m1bkgrate[1] +/- $m1bkgrate[2] \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "        MOS2: \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          MOS2 SRC Rate (cts/s): $m2srcrate[1] +/- $m2srcrate[2] \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          MOS2 BKG Rate (cts/s): $m2bkgrate[1] +/- $m2bkgrate[2] \n";}  
	print EDETECTLOG "       Source Count Rates in different energy ranges considered: \n";
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "        PN  : \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $pnsrcrate_E1[1] +/- $pnsrcrate_E1[2] ($pn_ene_low[0] - $pn_ene_high[0]) \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $pnsrcrate_E2[1] +/- $pnsrcrate_E2[2] ($pn_ene_low[1] - $pn_ene_high[1]) \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $pnsrcrate_E3[1] +/- $pnsrcrate_E3[2] ($pn_ene_low[2] - $pn_ene_high[2]) \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $pnsrcrate_E4[1] +/- $pnsrcrate_E4[2] ($pn_ene_low[3] - $pn_ene_high[3]) \n";}
	if ($pn_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $pnsrcrate_E5[1] +/- $pnsrcrate_E5[2] ($pn_ene_low[4] - $pn_ene_high[4]) \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "        MOS1: \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m1srcrate_E1[1] +/- $m1srcrate_E1[2] ($m1_ene_low[0] - $m1_ene_high[0]) \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m1srcrate_E2[1] +/- $m1srcrate_E2[2] ($m1_ene_low[1] - $m1_ene_high[1]) \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m1srcrate_E3[1] +/- $m1srcrate_E3[2] ($m1_ene_low[2] - $m1_ene_high[2]) \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m1srcrate_E4[1] +/- $m1srcrate_E4[2] ($m1_ene_low[3] - $m1_ene_high[3]) \n";}
	if ($m1_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m1srcrate_E5[1] +/- $m1srcrate_E5[2] ($m1_ene_low[4] - $m1_ene_high[4]) \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "        MOS2: \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m2srcrate_E1[1] +/- $m2srcrate_E1[2] ($m2_ene_low[0] - $m2_ene_high[0]) \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m2srcrate_E2[1] +/- $m2srcrate_E2[2] ($m2_ene_low[1] - $m2_ene_high[1]) \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m2srcrate_E3[1] +/- $m2srcrate_E3[2] ($m2_ene_low[2] - $m2_ene_high[2]) \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m2srcrate_E4[1] +/- $m2srcrate_E4[2] ($m2_ene_low[3] - $m2_ene_high[3]) \n";}
	if ($m2_edetect_flag eq 1) {print EDETECTLOG "          SRC Rate (cts/s): $m2srcrate_E5[1] +/- $m2srcrate_E5[2] ($m2_ene_low[4] - $m2_ene_high[4]) \n";}
	print EDETECTLOG "\n   \#> Cross Correlation Information for EPIC: \n";
	print EDETECTLOG "       SOURCE           PN               MOS1             MOS2   \n";
	print EDETECTLOG "  RA       DEC     RA       DEC      RA       DEC    RA        DEC \n";
	print EDETECTLOG " $source_ra $source_dec $pn_closesource[0] $pn_closesource[1] $m1_closesource[0] $m1_closesource[1] $m2_closesource[0] $m2_closesource[1] \n";
	print EDETECTLOG "  Distance to closest source in each instrument: \n";
	print EDETECTLOG "  (If RA,DEC of the source is within ", join(", ", $gv->EDC_sourcematch)," deg of RA,DEC of detected \n";
	print EDETECTLOG "  camera source, it is assumed the source is the same) \n";
	print EDETECTLOG "    SOURCE-PN : $pn_closesource[2] deg. Same source ?: $pn_edetect_flag \n";
	print EDETECTLOG "    SOURCE-M1 : $m1_closesource[2] deg. Same source ?: $m1_edetect_flag \n";
	print EDETECTLOG "    SOURCE-M2 : $m2_closesource[2] deg. Same source ?: $m2_edetect_flag \n";
	print EDETECTLOG "  Comparing Source detection on different cameras: \n";
	print EDETECTLOG "  (If RA,DEC of one camera is within ", join(", ", $gv->EDC_cameramatch)," deg of RA,DEC of another camera \n";
	print EDETECTLOG "  it is assumed the source in both cameras is the same) \n";
	print EDETECTLOG "    Distance PN-MOS1    : $pnmos1_dist deg. Same source ?: $pnmos1det \n";
	print EDETECTLOG "    Distance PN-MOS2    : $pnmos2_dist deg. Same source ?: $pnmos2det \n";
	print EDETECTLOG "    Distance MOS1-MOS2  : $mos1mos2_dist deg. Same source ?: $mos1mos2det \n";
    }	
    if ($Log_Flag == 0) {
	print MYOUTFILE "       Source IDs (from emllist.fits table): \n";
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "        PN SRC ID  : $pnsrcrate[0] \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "        MOS1 SRC ID: $m1srcrate[0] \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "        MOS2 SRC ID: $m2srcrate[0] \n";}
	print MYOUTFILE "       CCD IDs (from emllist.fits table): \n";
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "        PN CCD ID  : $pnsrcrate[3] \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "        MOS1 CCD ID: $m1srcrate[3] \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "        MOS2 CCD ID: $m2srcrate[3] \n";}
	print MYOUTFILE "       Source and BKG Count Rates in entire energy range considered: \n";
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "        PN  : \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          PN SRC Rate (cts/s): $pnsrcrate[1] +/- $pnsrcrate[2] \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          PN BKG Rate (cts/s): $pnbkgrate[1] +/- $pnbkgrate[2] \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "        MOS1: \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          MOS1 SRC Rate (cts/s): $m1srcrate[1] +/- $m1srcrate[2] \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          MOS1 BKG Rate (cts/s): $m1bkgrate[1] +/- $m1bkgrate[2] \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "        MOS2: \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          MOS2 SRC Rate (cts/s): $m2srcrate[1] +/- $m2srcrate[2] \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          MOS2 BKG Rate (cts/s): $m2bkgrate[1] +/- $m2bkgrate[2] \n";} 
	print MYOUTFILE "       Source Count Rates in different energy ranges considered: \n";
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "        PN  : \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $pnsrcrate_E1[1] +/- $pnsrcrate_E1[2] ($pn_ene_low[0] - $pn_ene_high[0]) \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $pnsrcrate_E2[1] +/- $pnsrcrate_E2[2] ($pn_ene_low[1] - $pn_ene_high[1]) \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $pnsrcrate_E3[1] +/- $pnsrcrate_E3[2] ($pn_ene_low[2] - $pn_ene_high[2]) \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $pnsrcrate_E4[1] +/- $pnsrcrate_E4[2] ($pn_ene_low[3] - $pn_ene_high[3]) \n";}
	if ($pn_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $pnsrcrate_E5[1] +/- $pnsrcrate_E5[2] ($pn_ene_low[4] - $pn_ene_high[4]) \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "        MOS1: \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m1srcrate_E1[1] +/- $m1srcrate_E1[2] ($m1_ene_low[0] - $m1_ene_high[0]) \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m1srcrate_E2[1] +/- $m1srcrate_E2[2] ($m1_ene_low[1] - $m1_ene_high[1]) \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m1srcrate_E3[1] +/- $m1srcrate_E3[2] ($m1_ene_low[2] - $m1_ene_high[2]) \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m1srcrate_E4[1] +/- $m1srcrate_E4[2] ($m1_ene_low[3] - $m1_ene_high[3]) \n";}
	if ($m1_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m1srcrate_E5[1] +/- $m1srcrate_E5[2] ($m1_ene_low[4] - $m1_ene_high[4]) \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "         MOS2: \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m2srcrate_E1[1] +/- $m2srcrate_E1[2] ($m2_ene_low[0] - $m2_ene_high[0]) \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m2srcrate_E2[1] +/- $m2srcrate_E2[2] ($m2_ene_low[1] - $m2_ene_high[1]) \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m2srcrate_E3[1] +/- $m2srcrate_E3[2] ($m2_ene_low[2] - $m2_ene_high[2]) \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m2srcrate_E4[1] +/- $m2srcrate_E4[2] ($m2_ene_low[3] - $m2_ene_high[3]) \n";}
	if ($m2_edetect_flag eq 1) {print MYOUTFILE "          SRC Rate (cts/s): $m2srcrate_E5[1] +/- $m2srcrate_E5[2] ($m2_ene_low[4] - $m2_ene_high[4]) \n";}
	print MYOUTFILE "\n   \#> Cross Correlation Information for EPIC: \n";
	print MYOUTFILE "       SOURCE           PN               MOS1             MOS2   \n";
	print MYOUTFILE "  RA       DEC     RA       DEC      RA       DEC    RA        DEC \n";
	print MYOUTFILE " $source_ra $source_dec $pn_closesource[0] $pn_closesource[1] $m1_closesource[0] $m1_closesource[1] $m2_closesource[0] $m2_closesource[1] \n";
	print MYOUTFILE "  Distance to closest source in each instrument: \n";
	print MYOUTFILE "  (If RA,DEC of the source is within ", join(", ", $gv->EDC_sourcematch)," deg of RA,DEC of detected \n";
	print MYOUTFILE "  camera source, it is assumed the source is the same) \n";
	print MYOUTFILE "    SOURCE-PN : $pn_closesource[2] deg. Same source ?: $pn_edetect_flag \n";
	print MYOUTFILE "    SOURCE-M1 : $m1_closesource[2] deg. Same source ?: $m1_edetect_flag \n";
	print MYOUTFILE "    SOURCE-M2 : $m2_closesource[2] deg. Same source ?: $m2_edetect_flag \n";
	print MYOUTFILE "  Comparing Source detection on different cameras: \n";
	print MYOUTFILE "  (If RA,DEC of one camera is within ", join(", ", $gv->EDC_cameramatch)," deg of RA,DEC of another camera \n";
	print MYOUTFILE "  it is assumed the source in both cameras is the same) \n";
	print MYOUTFILE "    Distance PN-MOS1    : $pnmos1_dist deg. Same source ?: $pnmos1det \n";
	print MYOUTFILE "    Distance PN-MOS2    : $pnmos2_dist deg. Same source ?: $pnmos2det \n";
	print MYOUTFILE "    Distance MOS1-MOS2  : $mos1mos2_dist deg. Same source ?: $mos1mos2det \n";
    }

    if ($Log_Edet_Flag == 0) {close(EDETECTLOG);}
    &error_code(0); 

    if ($pn_edetect_flag eq 0 && $m1_edetect_flag eq 0 && $m2_edetect_flag eq 0) {return(1);}
    else{return(0);} 
}

## @method void pn_create_energy_bands()
# Produce EPIC-pn images in 5 user define energy bands.
# @return Void

#========================================================================
sub pn_create_energy_bands(){
    print "   \#> PN: \n";
    print "      Events File Used: $pn_gti_cleanevents_file \n";
    print "      GTI File Used   : $pn_GTI_file \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> PN: \n";
	print MYOUTFILE "      Events File Used: $pn_gti_cleanevents_file \n";
	print MYOUTFILE "      GTI File Used   : $pn_GTI_file \n";
    }
	
    my $number_of_bands = $#pn_ene_low;
    my $xbinsize        = 40;
    my $ybinsize        = 40;

    chdir($images_directory);

    print "      Energy Band: $pn_ene_low[0] - $pn_ene_high[$number_of_bands] \n";
    if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $pn_ene_low[0] - $pn_ene_high[$number_of_bands] \n";}

    @args = ("evselect"
	     ,"table=$pn_gti_cleanevents_file:EVENTS"
	     ,"imagebinning=binSize"
	     ,"imageset=$pn_fullimage"
	     ,"withimageset=yes"
	     ,"xcolumn=X","ycolumn=Y"
	     ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
	     ,"expression=(PI in \[$pn_ene_low[0]:$pn_ene_high[$number_of_bands]\])");
    system(@args)== 0 or &error_code(5);
    
    my $ene_band = 0;
    while ($ene_band <= $number_of_bands){
	
	print "      Energy Band($ene_band) : $pn_ene_low[$ene_band] - $pn_ene_high[$ene_band] \n";
	if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $ene_band : $pn_ene_low[$ene_band] - $pn_ene_high[$ene_band] \n";}
	$image = "pn_image_b".$ene_band.".fits";
	@args = ("evselect"
		 ,"table=$pn_gti_cleanevents_file:EVENTS"
		 ,"imagebinning=binSize"
		 ,"imageset=$image"
		 ,"withimageset=yes"
		 ,"xcolumn=X","ycolumn=Y"
		 ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
		 ,"expression=(PI in \[$pn_ene_low[$ene_band]:$pn_ene_high[$ene_band]\])");
	system(@args)== 0 or &error_code(5);	
	$ene_band++;
	
    }
    
    return (0);
}

## @method void pn_edetect_chain()
# Runs the SAS task edetect_chain for EPIC-pn
# @return Void

#========================================================================
sub pn_edetect_chain(){
    chdir($pn_directory);

    my @pnAttitudelist = glob('*AttHk.ds');
    if ($#pnAttitudelist < 0) {&error_code(5);}
    my $pnAttitudefile = $pn_directory.$pnAttitudelist[0]; 

    my @filter=&event_file_inf($pn_gti_cleanevents_file,"pn");
    my $pn_filter=$filter[1];

    chdir($images_directory);
    
    my @pn_ecf      = &pn_ecfvalues($pn_filter);
    my $number_of_bands = $#pn_ene_low; 
    
    print "\n   \#> Running edetect chain for PN .... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Running edetect chain for PN .... \n";}

    my $img0=$images_directory."pn_image_b0\.fits";
    my $img1=$images_directory."pn_image_b1\.fits";
    my $img2=$images_directory."pn_image_b2\.fits";
    my $img3=$images_directory."pn_image_b3\.fits";
    my $img4=$images_directory."pn_image_b4\.fits";
    @args = ("edetect_chain","imagesets=$img0 $img1 $img2 $img3 $img4" 
	     ,"eventsets=$pn_gti_cleanevents_file"
	     ,"attitudeset=$pnAttitudefile" 
	     ,"pimin=$pn_ene_low[0] $pn_ene_low[1] $pn_ene_low[2] $pn_ene_low[3] $pn_ene_low[4]"
	     ,"pimax=$pn_ene_high[0] $pn_ene_high[1] $pn_ene_high[2] $pn_ene_high[3] $pn_ene_high[4]" 
	     ,"ecf=$pn_ecf[0] $pn_ecf[1] $pn_ecf[2] $pn_ecf[3] $pn_ecf[4]" 
	     ,"eboxl_list=$pn_eboxllist_file"
	     ,"eboxm_list=$pn_eboxmlist_file" 
	     ,"esp_nsplinenodes=16"
	     ,"eml_list=$pn_emllist_file"
	     ,"esen_mlmin=$pn_likelihood_detection"
	     ,"eml_fitextent=\"yes\"");
    system(@args)== 0 or &error_code(5);

    print "      PN ECF Values  : "; foreach (@pn_ecf) {print "$_ ";}; print "\n\n";
    if ($Log_Flag == 0) {print MYOUTFILE "      PN ECF Values  : "; foreach (@pn_ecf) {print MYOUTFILE "$_ ";}; print MYOUTFILE "\n\n";}
			 
    return(0);
}

## @method void m1_create_energy_bands()
# Produce EPIC-MOS1 images in 5 user define energy bands.
# @return Void

#========================================================================
sub m1_create_energy_bands(){
    print "   \#> MOS1: \n";
    print "      Events File Used: $m1_gti_cleanevents_file \n";
    print "      GTI File Used   : $m1_GTI_file \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> MOS1: \n";
	print MYOUTFILE "      Events File Used: $m1_gti_cleanevents_file \n";
	print MYOUTFILE "      GTI File Used   : $m1_GTI_file \n";
    }
	
    my $number_of_bands = $#m1_ene_low;
    my $xbinsize        = 40;
    my $ybinsize        = 40;

    chdir($images_directory);
 
    print "      Energy Band: $m1_ene_low[0] - $m1_ene_high[$number_of_bands] \n";
    if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $m1_ene_low[0] - $m1_ene_high[$number_of_bands] \n";}

    @args = ("evselect"
	     ,"table=$m1_gti_cleanevents_file:EVENTS"
	     ,"imagebinning=binSize"
	     ,"imageset=$m1_fullimage"
	     ,"withimageset=yes"
	     ,"xcolumn=X","ycolumn=Y"
	     ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
	     ,"expression=(PI in \[$m1_ene_low[0]:$m1_ene_high[$number_of_bands]\])");
    system(@args)== 0 or &error_code(5);
    
    my $ene_band = 0;
    while ($ene_band <= $number_of_bands){
	
	print "      Energy Band($ene_band) : $m1_ene_low[$ene_band] - $m1_ene_high[$ene_band] \n";
	if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $ene_band : $m1_ene_low[$ene_band] - $m1_ene_high[$ene_band] \n";}
	$image = "m1_image_b".$ene_band.".fits";
	@args = ("evselect"
		 ,"table=$m1_gti_cleanevents_file:EVENTS"
		 ,"imagebinning=binSize"
		 ,"imageset=$image"
		 ,"withimageset=yes"
		 ,"xcolumn=X","ycolumn=Y"
		 ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
		 ,"expression=(PI in \[$m1_ene_low[$ene_band]:$m1_ene_high[$ene_band]\])");
	system(@args)== 0 or &error_code(5);	
	$ene_band++;
	
    }

    return(0);
}

## @method void m1_edetect_chain()
# Runs the SAS task edetect_chain for EPIC-MOS1
# @return Void

#========================================================================
sub m1_edetect_chain(){
    chdir($mos_directory);

    my @MOSAttitudelist = glob('*AttHk.ds');
    if ($#MOSAttitudelist < 0) {&error_code(5);}
    my $MOSAttitudefile = $mos_directory.$MOSAttitudelist[0]; 

    my @filter=&event_file_inf($m1_gti_cleanevents_file,"mos1");
    my $m1_filter=$filter[1];

    chdir($images_directory);
   
    my @m1_ecf      = &mos1_ecfvalues($m1_filter);
    my $number_of_bands = $#m1_ene_low;
   
    print "\n   \#> Running edetect chain for MOS1 .... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Running edetect chain for MOS1 .... \n";}

    my $img0=$images_directory."m1_image_b0\.fits";
    my $img1=$images_directory."m1_image_b1\.fits";
    my $img2=$images_directory."m1_image_b2\.fits";
    my $img3=$images_directory."m1_image_b3\.fits";
    my $img4=$images_directory."m1_image_b4\.fits";
    @args = ("edetect_chain","imagesets=$img0 $img1 $img2 $img3 $img4" 
	     ,"eventsets=$m1_gti_cleanevents_file"
	     ,"attitudeset=$MOSAttitudefile" 
	     ,"pimin=$m1_ene_low[0] $m1_ene_low[1] $m1_ene_low[2] $m1_ene_low[3] $m1_ene_low[4]"
	     ,"pimax=$m1_ene_high[0] $m1_ene_high[1] $m1_ene_high[2] $m1_ene_high[3] $m1_ene_high[4]" 
	     ,"ecf=$m1_ecf[0] $m1_ecf[1] $m1_ecf[2] $m1_ecf[3] $m1_ecf[4]" 
	     ,"eboxl_list=$m1_eboxllist_file"
	     ,"eboxm_list=$m1_eboxmlist_file" 
	     ,"esp_nsplinenodes=16"
	     ,"eml_list=$m1_emllist_file"
	     ,"esen_mlmin=$mos_likelihood_detection");
    system(@args)== 0 or &error_code(5);

    print "      MOS1 ECF Values  : "; foreach (@m1_ecf) {print "$_ ";} ; print "\n\n";
    if ($Log_Flag == 0) {print MYOUTFILE "      MOS1 ECF Values  : "; foreach (@m1_ecf) {print MYOUTFILE "$_ ";}; print MYOUTFILE "\n\n";}
			 
    return(0);
}

## @method void m2_create_energy_bands()
# Produce EPIC-MOS2 images in 5 user define energy bands.
# @return Void

#========================================================================
sub m2_create_energy_bands(){
    print "   \#> MOS2: \n";
    print "      Events File Used: $m2_gti_cleanevents_file \n";
    print "      GTI File Used   : $m2_GTI_file \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> MOS2: \n";
	print MYOUTFILE "      Events File Used: $m2_gti_cleanevents_file \n";
	print MYOUTFILE "      GTI File Used   : $m2_GTI_file \n";
    }
	
    my $number_of_bands = $#m2_ene_low;
    my $xbinsize        = 40;
    my $ybinsize        = 40;

    chdir($images_directory);
 
    print "      Energy Band: $m2_ene_low[0] - $m2_ene_high[$number_of_bands] \n";
    if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $m2_ene_low[0] - $m2_ene_high[$number_of_bands] \n";}
    
    @args = ("evselect"
	     ,"table=$m2_gti_cleanevents_file:EVENTS"
	     ,"imagebinning=binSize"
	     ,"imageset=$m2_fullimage"
	     ,"withimageset=yes"
	     ,"xcolumn=X","ycolumn=Y"
	     ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
	     ,"expression=(PI in \[$m2_ene_low[0]:$m2_ene_high[$number_of_bands]\])");
    system(@args)== 0 or &error_code(5);
    
    my $ene_band = 0;
    while ($ene_band <= $number_of_bands){
	
	print "      Energy Band($ene_band) : $m2_ene_low[$ene_band] - $m2_ene_high[$ene_band] \n";
	if ($Log_Flag == 0) {print MYOUTFILE "     Energy Band: $ene_band : $m2_ene_low[$ene_band] - $m2_ene_high[$ene_band] \n";}
	$image = "m2_image_b".$ene_band.".fits";
	@args = ("evselect"
		 ,"table=$m2_gti_cleanevents_file:EVENTS"
		 ,"imagebinning=binSize"
		 ,"imageset=$image"
		 ,"withimageset=yes"
		 ,"xcolumn=X","ycolumn=Y"
		 ,"ximagebinsize=$xbinsize","yimagebinsize=$ybinsize"
		 ,"expression=(PI in \[$m2_ene_low[$ene_band]:$m2_ene_high[$ene_band]\])");
	system(@args)== 0 or &error_code(5);	
	$ene_band++;
	
    }

    return(0);
}

## @method void m2_edetect_chain()
# Runs the SAS task edetect_chain for EPIC-MOS2
# @return Void

#========================================================================
sub m2_edetect_chain(){
    chdir($mos_directory);

    my @MOSAttitudelist = glob('*AttHk.ds');
    if ($#MOSAttitudelist < 0) {return(1);}
    my $MOSAttitudefile = $mos_directory.$MOSAttitudelist[0]; 

    my @filter=&event_file_inf($m2_gti_cleanevents_file,"mos2");
    my $m2_filter=$filter[1];

    chdir($images_directory);

    my @m2_ecf      = &mos2_ecfvalues($m2_filter);
    my $number_of_bands = $#m2_ene_low;

    print "\n   \#> Running edetect chain for MOS2 .... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "\n   \#> Running edetect chain for MOS2 .... \n";}

    my $img0=$images_directory."m2_image_b0\.fits";
    my $img1=$images_directory."m2_image_b1\.fits";
    my $img2=$images_directory."m2_image_b2\.fits";
    my $img3=$images_directory."m2_image_b3\.fits";
    my $img4=$images_directory."m2_image_b4\.fits";
    @args = ("edetect_chain","imagesets=$img0 $img1 $img2 $img3 $img4" 
	     ,"eventsets=$m2_gti_cleanevents_file"
	     ,"attitudeset=$MOSAttitudefile" 
	     ,"pimin=$m2_ene_low[0] $m2_ene_low[1] $m2_ene_low[2] $m2_ene_low[3] $m2_ene_low[4]"
	     ,"pimax=$m2_ene_high[0] $m2_ene_high[1] $m2_ene_high[2] $m2_ene_high[3] $m2_ene_high[4]" 
	     ,"ecf=$m2_ecf[0] $m2_ecf[1] $m2_ecf[2] $m2_ecf[3] $m2_ecf[4]" 
	     ,"eboxl_list=$m2_eboxllist_file"
	     ,"eboxm_list=$m2_eboxmlist_file" 
	     ,"esp_nsplinenodes=16"
	     ,"eml_list=$m2_emllist_file"
	     ,"esen_mlmin=$mos_likelihood_detection");
    system(@args)== 0 or &error_code(5);

    print "      MOS2 ECF Values  : "; foreach (@m2_ecf) {print "$_ ";}; print "\n\n";
    if ($Log_Flag == 0) {print MYOUTFILE "      MOS2 ECF Values  : "; foreach (@m2_ecf) {print MYOUTFILE "$_ ";}; print MYOUTFILE "\n\n";}
			 
    return(0);
}

## @method pn_image()
# Produce an EPIC-pn Image and region file from Source List. Produces an EPIC-pn Image with regions from region file overlaid.
# @return RA (deg) and DEC (deg) of closest EPIC-pn detected source to the source of interes. Angular distance (deg) between EPIC-pn and source of interest.

#========================================================================
sub pn_image(){
    chdir ($images_directory);
     
    my $pnregion_file = $images_directory."pnregion\.txt"; 
    my $source_ra     = $gv->SRC_RA;
    my $source_dec    = $gv->SRC_DEC;
    my $source_name   = $gv->SRC_Name;

#.. Run the SAS commands srcdisplay to produce region file
    @args=("srcdisplay"
	   ,"boxlistset=$pn_emllist_file" 
	   ,"imageset=$pn_fullimage"
	   ,"sourceradius=0.01"
	   ,"withregionfile=yes"
	   ,"regionfile=$pnregion_file"); 
    system(@args);                         # == 0 or &error_code(5); NOTE: If I put this bit, it wont run on the grid

#.. From the list of detected sources in the region file find the closest source to yours
#.. and add your source to the region file to be plotted. 
    if (-e $pnregion_file){
	@pnclose_source=&find_closest_source($pnregion_file);
	&addsourcetoregionfile($pnregion_file,$pnclose_source[0],$pnclose_source[1],0.,0.008,"red","","circle","fk5");
	&addsourcetoregionfile($pnregion_file,$source_ra,$source_dec,0.,0.009,"green",$source_name,"circle","fk5");	
    } else {
	print "   \#> PN Region file $pnregion_file does not exist \n";
	if ($Log_Flag == 0) {print MYOUTFILE "   \#> PN Region file $pnregion_file does not exist \n";}	
	return(100.,100.,100.);
    }

    rename($pnregion_file,$source_pnregion_file);
    print "   \#> PN Region file produced: $source_pnregion_file \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> PN Region file produced: $source_pnregion_file \n";}

#.. Run DS9. Arguments: image file, region file, title, output name
    &produce_ps_file($pn_fullimage,$source_pnregion_file,"PN",$images_directory.$source_name."_PN_image.ps");

    return($pnclose_source[0],$pnclose_source[1],$pnclose_source[2]);
}

## @method m1_image()
# Produce an EPIC-MOS1 Image and region file from Source List. Produces an EPIC-MOS1 Image with regions from region file overlaid.
# @return RA (deg) and DEC (deg) of closest EPIC-MOS1 detected source to the source of interes. Angular distance (deg) between EPIC-MOS1 and source of interest.

#========================================================================
sub m1_image(){
    chdir ($images_directory);

    my $m1region_file = $images_directory."m1region\.txt";
    my $source_ra     = $gv->SRC_RA;
    my $source_dec    = $gv->SRC_DEC;
    my $source_name   = $gv->SRC_Name;

#.. Run the SAS commands srcdisplay to produce region file
    @args=("srcdisplay"
           ,"boxlistset=$m1_emllist_file"
           ,"imageset=$m1_fullimage"
           ,"sourceradius=0.01"
           ,"withregionfile=yes"
           ,"regionfile=$m1region_file");
    system(@args);                          # == 0 or &error_code(5); NOTE: If I put this bit, it wont run on the grid

#.. From the list of detected sources in the region file find the closest source to yours
#.. and add your source to the region file to be plotted.
    if (-e $m1region_file){
        @m1close_source=&find_closest_source($m1region_file);
        &addsourcetoregionfile($m1region_file,$m1close_source[0],$m1close_source[1],0.,0.008,"red","","circle","fk5");
        &addsourcetoregionfile($m1region_file,$source_ra,$source_dec,0.,0.009,"green",$source_name,"circle","fk5");     
    } else {
        print "   \#> MOS1 Region file $m1region_file does not exist \n";
        if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS1 Region file $m1region_file does not exist \n";}
        return(100.,100.,100.);
    }
   
   rename($m1region_file,$source_m1region_file);
   print "   \#> MOS1 Region file produced: $source_m1region_file \n";
   if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS1 Region file produced: $source_m1region_file \n";}

#.. Run DS9. Arguments: image file, region file, title, output name
   &produce_ps_file($m1_fullimage,$source_m1region_file,"MOS1",$images_directory.$source_name."_MOS1_image.ps");

   return($m1close_source[0],$m1close_source[1],$m1close_source[2]);  
}

## @method m2_image()
# Produce an EPIC-MOS2 Image and region file from Source List. Produces an EPIC-MOS2 Image with regions from region file overlaid.
# @return RA (deg) and DEC (deg) of closest EPIC-MOS22 detected source to the source of interes. Angular distance (deg) between EPIC-MOS2 and source of interest.

#========================================================================
sub m2_image(){
    chdir ($images_directory);

    my $m2region_file = $images_directory."m2region\.txt";
    my $source_ra     = $gv->SRC_RA;
    my $source_dec    = $gv->SRC_DEC;
    my $source_name   = $gv->SRC_Name;

#.. Run the SAS commands srcdisplay to produce region file
    @args=("srcdisplay"
           ,"boxlistset=$m2_emllist_file"
           ,"imageset=$m2_fullimage"
           ,"sourceradius=0.01"
           ,"withregionfile=yes"
           ,"regionfile=$m2region_file");
    system(@args);                            # == 0 or &error_code(5); NOTE: If I put this bit, it wont run on the grid

#.. From the list of detected sources in the region file find the closest source to yours
#.. and add your source to the region file to be plotted.
    if (-e $m2region_file){
        @m2close_source=&find_closest_source($m2region_file);
        &addsourcetoregionfile($m2region_file,$m2close_source[0],$m2close_source[1],0.,0.008,"red","","circle","fk5");
        &addsourcetoregionfile($m2region_file,$source_ra,$source_dec,0.,0.009,"green",$source_name,"circle","fk5");        
    } else {
        print "   \#> MOS2 Region file $m2region_file does not exist \n";
        if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS2 Region file $m2region_file does not exist \n";}
        return(100.,100.,100.);
    }

    rename($m2region_file,$source_m2region_file);
    print "   \#> MOS2 Region file produced: $source_m2region_file \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> MOS2 Region file produced: $source_m2region_file \n";}

#.. Run DS9. Arguments: image file, region file, title, output name
    &produce_ps_file($m2_fullimage,$source_m2region_file,"MOS2",$images_directory.$source_name."_MOS2_image.ps");

    return($m2close_source[0],$m2close_source[1],$m2close_source[2]);
}

## @method pn_ecfvalues($filter)
# Definition of Energy Conversion Factors (ECF) for EPIC-pn
# @param filter EPIC-pn Filter
# @return Array with ECF for 5 user defined energy bins

#========================================================================
sub pn_ecfvalues(){

    my $filter = $_[0];

    print "       pn Filter ECF Set: $filter \n";
    $filter =~ s/\s+//;
    if ($filter eq "Open") {return(15.711,7.5030,1.9837,0.9415,0.2399);} 
    if ($filter eq "Thick") {return(4.990,5.315,1.847,0.927,0.2390);} 
    if ($filter eq "Medium") {return(8.010,6.442,1.930,0.9394,0.2400);} 
    if ($filter eq "Thin" or $filter eq "Thin1" or $filter eq "Thin2") {return(9.041,6.596,1.953,0.941,0.2401);} 
    return(1,1,1,1,1);
}

## @method mos1_ecfvalues($filter)
# Definition of Energy Conversion Factors (ECF) for EPIC-MOS1
# @param filter EPIC-MOS1 Filter
# @return Array with ECF for 5 user defined energy bins

#========================================================================
sub mos1_ecfvalues(){

    my $filter = $_[0];

    print "       mos1 Filter ECF Set: $filter \n";
    $filter =~ s/\s+//;
    if ($filter eq "Open") {return(3.0377,2.2011,0.75244,0.26492,0.02774);}
    if ($filter eq "Thick") {return(0.984,1.622,0.7024,0.260,0.02760);} 
    if ($filter eq "Medium") {return(1.564,1.983,0.7325,0.2639,0.02772);}
    if ($filter eq "Thin" or $filter eq "Thin1" or $filter eq "Thin2") {return(1.756,1.980,0.741,0.2644,0.02773);}
    return(1,1,1,1,1);
}

## @method mos2_ecfvalues($filter)
# Definition of Energy Conversion Factors (ECF) for EPIC-MOS2
# @param filter EPIC-MOS2 Filter
# @return Array with ECF for 5 user defined energy bins

#========================================================================
sub mos2_ecfvalues(){

    my $filter = $_[0];

    print "       mos2 Filter ECF Set: $filter \n";
    $filter =~ s/\s+//;
    if ($filter eq "Open") {return(3.0618,2.1963,0.75597,0.27772,0.029915);} 
    if ($filter eq "Thick") {return(0.994,1.620,0.7059,0.2727,0.02976);} 
    if ($filter eq "Medium") {return(1.579,1.935,0.7361,0.2767,0.02988);} 
    if ($filter eq "Thin" or $filter eq "Thin1" or $filter eq "Thin2") {return(1.772,1.977,0.7445,0.2771,0.0299);} 
    return(1,1,1,1,1);
}

## @method find_closest_source($region_file)
# Given a Region File, find the closest source in the region file to the source of interest.
# @param region_file Name of Region File
# @return RA (deg), DEC (deg) Angular distance (deg) of closest detected source to the source of interest.

#========================================================================
sub find_closest_source(){

    my $source_ra=$gv->SRC_RA;
    my $source_dec=$gv->SRC_DEC;
    
    my $region_file = $_[0];

    my @ra_list;
    my @dec_list;

    if (-e $region_file) {
	
	open (REGIONFILE, $region_file); # or die "couldn't open file $region_file: $! \n";
	
	while (<REGIONFILE>) {
	    my $line = $_;
	    next if /^global/;  # skip comment lines
	    next if /^[\#*]/;   # skip comment lines
	    next if /^(\s)*$/;  # skip blank lines
	    next if !/(color\=white)$/;  # use only lines that come from edetect_chain
	    chomp($line);
	    $line =~ s/fk5\;circle\(//;
	    $line =~ s/\,0.01\) \# color\=white//;
	    my($ra, $dec) = split(/\,/, $line);
	    push @ra_list, $ra;
	    push @dec_list, $dec;
	}
	close(REGIONFILE);
	$counter = 0;
	$mindist = 100.;
	foreach(@ra_list){
	    $angsep = &Angular_separation($ra_list[$counter],$source_ra,$dec_list[$counter],$source_dec);
	    if ($angsep < $mindist){
		$mindist    = $angsep; 
		$mincounter = $counter;
	    }
	    $counter ++;
	}
    } else {
	$mincounter            = 0;
	$ra_list[$mincounter]  = 0.;
	$dec_list[$mincounter] = 0.;
	$mindist = 1000.;
    }
    
    return($ra_list[$mincounter],$dec_list[$mincounter],$mindist);
}

## @method get_source_count_rate($src_file,$instr,$energyband,$ra,$dec)
# Get source count rate from the Source List
# @param src_file Source List
# @param instr Instrument
# @param energyband Energy Band of interest
# @param ra Source RA
# @param dec Source DEC
# @return Source ID in the Source List, Source Count Rate and Error, CCD Number where the source is.

#========================================================================
sub get_source_count_rate(){

    my $src_file    = $_[0];
    my $instr       = $_[1]; 
    my $energyband  = $_[2];
    my $ra          = $_[3];
    my $dec         = $_[4];

    my $RALOW       = $ra-0.0005;
    my $RAHIG       = $ra+0.0005;
    my $DELOW       = $dec-0.0005;
    my $DEHIG       = $dec+0.0005;
    my $temp_file   = $images_directory."temp";
    my $out_file    = $src_file.".temp";
    my $src_ml_id   = 0.;
    my $src_ccd_id  = 0.;
    my $tot_srcrate = 0.;
    my $tot_esrcrate= 0.;

    if ($instr eq "pn") {$Instrument_ID = 1;}
    if ($instr eq "mos1") {$Instrument_ID = 2;}
    if ($instr eq "mos2") {$Instrument_ID = 3;}

#.. Create a reduced list from the input table of detected instrument sources with the one closest to our source
    @args = ("fselect $src_file $out_file \"ID_BAND==$energyband&&ID_INST==$Instrument_ID&&RA>$RALOW&&RA<$RAHIG&&DEC>$DELOW&&DEC<$DEHIG\" clobber=yes");
    system(@args)== 0 or &error_code(5);

    chomp(my $source   = `fkeyprint infile=$out_file+1  keynam=NAXIS2 outfil=STDOUT exact=yes | grep NAXIS2 | tail -1 | cut -c12-30`);
    $source     =~ s/\s+//;
    $source     =~ s/\'+//;

    if ($source == 1){
	
#.. Get Source ID from reduced list
	@args = ("fdump $out_file $temp_file \"ML_ID_SRC\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	system(@args)== 0 or &error_code(5);
	chomp($src_ml_id = `cat $temp_file`);
	$src_ml_id =~ s/\s+//;

#.. Get CCD ID in which source falls from reduced list
	@args = ("fdump $out_file $temp_file \"CCDNR\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	system(@args)== 0 or &error_code(5);
	chomp($src_ccd_id = `cat $temp_file`);
	$src_ccd_id =~ s/\s+//;

#.. Get Source Rate from reduced list
	@args = ("fdump $out_file $temp_file \"RATE\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	system(@args)== 0 or &error_code(5);
	chomp($tot_srcrate = `cat $temp_file`);
	$tot_srcrate =~ s/\s+//;
	
#.. Get Source Rate Error from reduced list
	@args = ("fdump $out_file $temp_file \"RATE_ERR\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	system(@args)== 0 or &error_code(5);
	chomp($tot_esrcrate = `cat $temp_file`);
	$tot_esrcrate =~ s/\s+//;

#.. Delete Temporary Files
	unlink("$temp_file") or die "Cannot rm file $temp_file: $!";
	unlink("$out_file") or die "Cannot rm file $out_file: $!";
	
    } else {
	$src_ml_id    = 10000.;
	$tot_srcrate  = 0.;
	$tot_esrcrate = 0.;
	if ($source > 1) {
	    print "      More than one source identified: $source \n"; 	    
	    if ($Log_Flag == 0) {print  MYOUTFILE "      More than one source identified: $source \n";}
	} elsif ($source == 0) {
	    print "      No Sources identified in $_[0] \n";
	    if ($Log_Flag == 0) {print MYOUTFILE "      No Sources identified in $_[0] \n";}
	}
    }
 
    return($src_ml_id,$tot_srcrate,$tot_esrcrate,$src_ccd_id);

}

## @method get_bkg_count_rate($src_file,$instr,$ra,$dec)
# Get background count rate from the Source List
# @param src_file Source List
# @param instr Instrument
# @param ra Source RA
# @param dec Source DEC
# @return Source ID in the Source List, Background Count Rate and Error.

#========================================================================
sub get_bkg_count_rate{
    
    my $src_file     = $_[0];
    my $instr        = $_[1];
    my $ra           = $_[2];
    my $dec          = $_[3];

    my $RALOW        = $ra-0.0005;
    my $RAHIG        = $ra+0.0005;
    my $DELOW        = $dec-0.0005;
    my $DEHIG        = $dec+0.0005;
    my $energyband   = 1;
    my $number_of_bands = 5;
    my $temp_file    = $images_directory."temp";
    my $out_file     = $src_file.".temp";
    my $src_ml_id    = 0.;
    my $tot_bkgrate  = 0.;
    my $tot_ebkgrate = 0.;
    my $BG_MAP       = 0.;
    my $EXP_MAP      = 1.;
    my $CUTRAD       = 0.;

    if ($instr eq "pn") {$Instrument_ID = 1;}
    if ($instr eq "mos1") {$Instrument_ID = 2;}
    if ($instr eq "mos2") {$Instrument_ID = 3;}

    while ($energyband <= $number_of_bands) {
#.. Create a reduced list from the input table of detected instrument sources with the one closest to our source
	@args = ("fselect $src_file $out_file \"ID_BAND==$energyband&&ID_INST==$Instrument_ID&&RA>$RALOW&&RA<$RAHIG&&DEC>$DELOW&&DEC<$DEHIG\" clobber=yes");
	system(@args)== 0 or &error_code(5);

	chomp(my $source   = `fkeyprint infile=$out_file+1 keynam=NAXIS2 outfil=STDOUT exact=yes | grep NAXIS2 | tail -1 | cut -c12-30`);
	$source     =~ s/\s+//;
	$source     =~ s/\'+//;

	if ($source == 1){

#.. Get Source ID from reduced list
	    @args = ("fdump $out_file $temp_file \"ML_ID_SRC\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($src_ml_id = `cat $temp_file`);
	    $src_ml_id =~ s/\s+//;
	    
#.. Get Source BG_MAP from reduced list
	    @args = ("fdump $out_file $temp_file \"BG_MAP\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($BG_MAP = `cat $temp_file`);
	    $BG_MAP =~ s/\s+//;
	    
#.. Get Source EXP_MAP from reduced list	
	    @args = ("fdump $out_file $temp_file \"EXP_MAP\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($EXP_MAP = `cat $temp_file`);
	    $EXP_MAP =~ s/\s+//;
	    
#.. Get Source CUTRAD from reduced list	
	    @args = ("fdump $out_file $temp_file \"CUTRAD\" - prhead=no showcol=no showunit=no showrow=no clobber=yes");
	    system(@args)== 0 or &error_code(5);
	    chomp($CUTRAD = `cat $temp_file`);
	    $CUTRAD =~ s/\s+//;
	    
#.. Workout bkg rate for a given energy band and accumulate 	
	    print "   \#> Values used for Background count rate (Energy $energyband): \n";
	    print "      $_[1] Exposure map  : $EXP_MAP \n";
	    print "      $_[1] Background map: $BG_MAP \n";
	    print "      $_[1] Cut Radius    : $CUTRAD \n";
	    if ($Log_Flag == 0) {
		print MYOUTFILE "   \#> Values used for Background count rate (Energy $energyband): \n";
		print MYOUTFILE "      $_[1] Exposure map  : $EXP_MAP \n";
		print MYOUTFILE "      $_[1] Background map: $BG_MAP \n";
		print MYOUTFILE "      $_[1] Cut Radius    : $CUTRAD \n";
	    }
	    if ($EXP_MAP != 0 || $EXP_MAP eq "" || $BG_MAP != 0 || $BG_MAP eq "" || $CUTRAD != 0 || $CUTRAD eq ""){ 
		$tot_bkgrate    = $tot_bkgrate+(($BG_MAP*(3.14159265*$CUTRAD**2))/$EXP_MAP);
		$tot_ebkgrate   = $tot_ebkgrate+($BG_MAP*((3.14159265*$CUTRAD**2)/$EXP_MAP)**2);
	    } else {
		print "      Could not workout bakground rate. \n";
		if ($Log_Flag == 0) {print MYOUTFILE "      Could not workout bakground rate. \n";}
		$tot_bkgrate  = 0;
		$tot_ebkgrate = 0;
	    }	
	    
#.. Delete Temporary Files
	    unlink("$temp_file") or die "Cannot rm file $temp_file: $!";
	    unlink("$out_file") or die "Cannot rm file $out_file: $!";
	    
	} else {
	    $src_ml_id    = 10000.;
	    $tot_bkgrate  = 0.;
	    $tot_ebkgrate = 0.;
	    if ($source > 1) {
		print "      More than one source identified: $source \n"; 	    
		if ($Log_Flag == 0) {print  MYOUTFILE "      More than one source identified: $source \n";}
	    } elsif ($source == 0) {
		print "      No Sources identified in $_[0] \n";
		if ($Log_Flag == 0) {print MYOUTFILE "      No Sources identified in $_[0] \n";}
	    }
	}

	$energyband++;
	
    }
    
    $tot_ebkgrate   = sqrt($tot_ebkgrate);
    
    return($src_ml_id,$tot_bkgrate,$tot_ebkgrate);
}
1;



